unit ASDStream;
{<|Author : Adler3D|>}
{<|e-mail : Adler3D@mail.ru|>}
{<|site : Adler3D.narod.ru|>}
interface
{$D-}
uses Windows,eXgine,QuickList,Messages;
const
  { TFileStream create mode }
  fmCreate=$FFFF;
  { File open modes }
  fmOpenRead=$0000;
  fmOpenWrite=$0001;
  fmOpenReadWrite=$0002;

  fmShareCompat=$0000;
  fmShareExclusive=$0010;
  fmShareDenyWrite=$0020;
  fmShareDenyRead=$0030;
  fmShareDenyNone=$0040;
type
  { TStream seek origins }
  TSeekOrigin=(soBeginning,soCurrent,soEnd);

  { TStream abstract class }
  TStream=class(TObject)
  private
    function GetPosition:Integer;
    procedure SetPosition(const Pos:Integer);
  protected
    function GetSize:Integer; virtual;
    procedure SetSize(const NewSize:Integer); virtual;
  public
    function Read(var Buffer; Count:Longint):Longint; virtual; abstract;
    function Write(const Buffer; Count:Longint):Longint; virtual; abstract;
    function Seek(const Offset:Integer; Origin:TSeekOrigin=soBeginning):
      Integer; virtual;
    procedure ReadBuffer(var Buffer; Count:Longint);
    procedure WriteBuffer(const Buffer; Count:Longint);
    function CopyFrom(Source:TStream; Count:Integer):Integer;
    function Valid:Boolean; virtual;
    property Position:Integer read GetPosition write SetPosition;
    property Size:Integer read GetSize write SetSize;
  end;

  { THandleStream class }

  THandleStream=class(TStream)
  protected
    FHandle:HFile;
    procedure SetSize(const NewSize:Integer); override;
  public
    constructor Create(AHandle:Integer);
    function Read(var Buffer; Count:Longint):Longint; override;
    function Write(const Buffer; Count:Longint):Longint; override;
    function Seek(const Offset:Integer; Origin:TSeekOrigin=soBeginning):
      Integer; override;
    function Handle:HFile;
  end;

  { TFileStream class }

  TFileStream=class(THandleStream)
  private
    FFileName:string;
  public
    constructor Create(const FileName:string; Mode:Word);
    destructor Destroy; override;
    property FileName:string read FFileName;
  end;

  { TCustomMemoryStream abstract class }

  TCustomMemoryStream=class(TStream)
  private
    FMemory:Pointer;
    FSize,FPosition:Integer;
  protected
    procedure SetPointer(Ptr:Pointer; Size:Integer);
  public
    function Read(var Buffer; Count:Longint):Integer; override;
    function Seek(const Offset:Integer; Origin:TSeekOrigin=soBeginning):
      Integer; override;
    procedure SaveToStream(Stream:TStream);
    procedure SaveToFile(const FileName:string);
    function Memory:Pointer;
  end;

{ TThread }

  TThreadMethod = procedure of object;

  TThreadPriority = (tpIdle, tpLowest, tpLower, tpNormal, tpHigher, tpHighest,
    tpTimeCritical);

  PSynchronizeRecord = ^TSynchronizeRecord;
  TSynchronizeRecord = record
    FThread: TObject;
    FMethod: TThreadMethod;
    FSynchronizeException: TObject;
  end;

  TThread = class
  private
    FHandle: THandle;
    FThreadID: THandle;
    FCreateSuspended: Boolean;
    FTerminated: Boolean;
    FSuspended: Boolean;
    FFreeOnTerminate: Boolean;
    FFinished: Boolean;
    FReturnValue: Integer;
    FSynchronize: TSynchronizeRecord;
    FFatalException: TObject;
    class procedure Synchronize(ASyncRec: PSynchronizeRecord); overload;
    //procedure CallOnTerminate;
    function GetPriority: TThreadPriority;
    procedure SetPriority(Value: TThreadPriority);
    procedure SetSuspended(Value: Boolean);
  protected
    procedure CheckThreadError(ErrCode: Integer); overload;
    procedure CheckThreadError(Success: Boolean); overload;
//    procedure DoTerminate; virtual;
    procedure Execute; virtual; abstract;
    procedure Synchronize(Method: TThreadMethod); overload;
    property ReturnValue: Integer read FReturnValue write FReturnValue;
    property Terminated: Boolean read FTerminated;
  public
    constructor Create(CreateSuspended: Boolean);
    destructor Destroy; override;
    procedure AfterConstruction; override;
    procedure Resume;
    procedure Suspend;
    procedure Terminate;
    function WaitFor: LongWord;
    class procedure Synchronize(AThread: TThread; AMethod: TThreadMethod); overload;
    class procedure StaticSynchronize(AThread: TThread; AMethod: TThreadMethod);
    property FatalException: TObject read FFatalException;
    property FreeOnTerminate: Boolean read FFreeOnTerminate write FFreeOnTerminate;
    property Handle: THandle read FHandle;
    property Priority: TThreadPriority read GetPriority write SetPriority;
    property Suspended: Boolean read FSuspended write SetSuspended;
    property ThreadID: THandle read FThreadID;
  end;

function CheckSynchronize(Timeout: Integer = 0): Boolean;

var
  //WakeMainThread: TNotifyEvent = nil;
  SyncEvent: THandle;
type
  TWndMethod = procedure(var Message: TMessage) of object;

implementation

procedure AddLog(S:string);
begin
  log.Print(PChar(S));
end;

function FileRead(Handle:Integer; var Buffer; Count:LongWord):Integer;
begin
  if not ReadFile(THandle(Handle),Buffer,Count,LongWord(Result),nil) then
    Result:=-1;
end;

function FileWrite(Handle:Integer; const Buffer; Count:LongWord):Integer;
begin
  if not WriteFile(THandle(Handle),Buffer,Count,LongWord(Result),nil) then
    Result:=-1;
end;

function FileCreate(const FileName:string):Integer;
begin
  Result:=Integer(CreateFile(PChar(FileName),GENERIC_READ or GENERIC_WRITE,
    0,nil,CREATE_ALWAYS,FILE_ATTRIBUTE_NORMAL,0));
end;

function FileOpen(const FileName:string; Mode:LongWord):Integer;
const
  AccessMode:array[0..2] of LongWord=(
    GENERIC_READ,
    GENERIC_WRITE,
    GENERIC_READ or GENERIC_WRITE);
  ShareMode:array[0..4] of LongWord=(
    0,
    0,
    FILE_SHARE_READ,
    FILE_SHARE_WRITE,
    FILE_SHARE_READ or FILE_SHARE_WRITE);
begin
  Result:=-1;
  if ((Mode and 3)<=fmOpenReadWrite)and
    ((Mode and $F0)<=fmShareDenyNone) then
    Result:=Integer(CreateFile(PChar(FileName),AccessMode[Mode and 3],
      ShareMode[(Mode and $F0)shr 4],nil,OPEN_EXISTING,
      FILE_ATTRIBUTE_NORMAL,0));
end;

procedure FileClose(Handle:Integer);
begin
  CloseHandle(THandle(Handle));
end;

{ TStream }

function TStream.GetPosition:Integer;
begin
  Result:=Seek(0,soCurrent);
end;

procedure TStream.SetPosition(const Pos:Integer);
begin
  Seek(Pos,soBeginning);
end;

function TStream.GetSize:Integer;
var
  Pos:Integer;
begin
  Pos:=Seek(0,soCurrent);
  Result:=Seek(0,soEnd);
  Seek(Pos,soBeginning);
end;

function TStream.Seek(const Offset:Integer; Origin:TSeekOrigin):Integer;
type
  TSeek64=function(const Offset:Integer; Origin:TSeekOrigin):Integer of
    object;
var
  Impl:TSeek64;
  Base:TSeek64;
  ClassTStream:TClass;
begin
  Result:=0;
  Impl:=Seek;
  ClassTStream:=Self.ClassType;
  while (ClassTStream<>nil)and(ClassTStream<>TStream) do
    ClassTStream:=ClassTStream.ClassParent;
  if ClassTStream=nil then
  begin
    AddLog('Error: TStream.Seek'); Exit;
  end;
  Base:=TStream(@ClassTStream).Seek;
  if TMethod(Impl).Code=TMethod(Base).Code then
  begin
    AddLog('Error: TStream.Seek'); Exit;
  end;
  Result:=Seek(Integer(Offset),TSeekOrigin(Origin));
end;

procedure TStream.ReadBuffer(var Buffer; Count:Longint);
begin
  if (Count<>0)and(Read(Buffer,Count)<>Count) then
  begin
    AddLog('Error: TStream.ReadBuffer'); Exit;
  end;
end;

procedure TStream.WriteBuffer(const Buffer; Count:Longint);
begin
  if (Count<>0)and(Write(Buffer,Count)<>Count) then
  begin
    AddLog('Error: TStream.WriteBuffer'); Exit;
  end;
end;

function TStream.CopyFrom(Source:TStream; Count:Integer):Integer;
const
  MaxBufSize=$F000;
var
  BufSize,N:Integer;
  Buffer:PChar;
begin
  if Count=0 then
  begin
    Source.Position:=0;
    Count:=Source.Size;
  end;
  Result:=Count;
  if Count>MaxBufSize then
    BufSize:=MaxBufSize
  else
    BufSize:=Count;
  GetMem(Buffer,BufSize);
  try
    while Count<>0 do
    begin
      if Count>BufSize then
        N:=BufSize
      else
        N:=Count;
      Source.ReadBuffer(Buffer^,N);
      WriteBuffer(Buffer^,N);
      Dec(Count,N);
    end;
  finally
    FreeMem(Buffer,BufSize);
  end;
end;

procedure TStream.SetSize(const NewSize:Integer);
begin

end;

function TStream.Valid:Boolean;
begin
  Result:=False;
end;

{ THandleStream }

constructor THandleStream.Create(AHandle:Integer);
begin
  FHandle:=AHandle;
end;

function THandleStream.Read(var Buffer; Count:Longint):Longint;
begin
  Result:=FileRead(FHandle,Buffer,Count); if Result=-1 then Result:=0;
end;

function THandleStream.Write(const Buffer; Count:Longint):Longint;
begin
  Result:=FileWrite(FHandle,Buffer,Count); if Result=-1 then Result:=0;
end;

function THandleStream.Seek(const Offset:Integer; Origin:TSeekOrigin):
  Integer;
begin
  Result:=SetFilePointer(THandle(Handle),Offset,nil,Ord(Origin));
end;

procedure THandleStream.SetSize(const NewSize:Integer);
begin
  Seek(NewSize,soBeginning);
end;

function THandleStream.Handle:HFile;
begin
  Result:=FHandle;
end;

{ TFileStream }

constructor TFileStream.Create(const FileName:string; Mode:Word);
const
  SFCreateErrorEx='Cannot create file "';
  SFOpenErrorEx='Cannot open file "';
begin
  FFileName:=FileName;
  if Mode=fmCreate then
  begin
    inherited Create(FileCreate(FileName));
    if FHandle=DWORD(-1) then
    begin
      AddLog('Error: TFileStream.Create'); Exit;
    end;
  end
  else
  begin
    inherited Create(FileOpen(FileName,Mode));
    if FHandle=DWORD(-1) then
    begin
      AddLog('Error: TFileStream.Create'); Exit;
    end;
  end;
end;

destructor TFileStream.Destroy;
begin
  if FHandle<>DWORD(-1) then FileClose(FHandle);
  inherited Destroy;
end;

{ TCustomMemoryStream }

procedure TCustomMemoryStream.SetPointer(Ptr:Pointer; Size:Longint);
begin
  FMemory:=Ptr;
  FSize:=Size;
end;

function TCustomMemoryStream.Read(var Buffer; Count:Longint):Longint;
begin
  if (FPosition>=0)and(Count>=0) then
  begin
    Result:=FSize-FPosition;
    if Result>0 then
    begin
      if Result>Count then
        Result:=Count;
      Move(Pointer(Longint(FMemory)+FPosition)^,Buffer,Result);
      Inc(FPosition,Result);
      Exit;
    end;
  end;
  Result:=0;
end;

function TCustomMemoryStream.Seek(const Offset:Integer; Origin:TSeekOrigin):
  Integer;
begin
  case Origin of
    soBeginning:FPosition:=Offset;
    soCurrent:Inc(FPosition,Offset);
    soEnd:FPosition:=FSize+Offset;
  end;
  Result:=FPosition;
end;

procedure TCustomMemoryStream.SaveToStream(Stream:TStream);
begin
  if FSize<>0 then
    Stream.WriteBuffer(FMemory^,FSize);
end;

procedure TCustomMemoryStream.SaveToFile(const FileName:string);
var
  Stream:TStream;
begin
  Stream:=TFileStream.Create(FileName,fmCreate);
  try
    SaveToStream(Stream);
  finally
    Stream.Free;
  end;
end;

function TCustomMemoryStream.Memory:Pointer;
begin
  Result:=FMemory;
end;

type
  TSyncProc = record
    SyncRec: PSynchronizeRecord;
{$IFDEF MSWINDOWS}
    Signal: THandle;
{$ENDIF}
{$IFDEF LINUX}
    Signal: TCondVar;
{$ENDIF}
  end;
  PSyncProc = ^TSyncProc;

var
  SyncList: TList = nil;
  ThreadLock: TRTLCriticalSection;
  ThreadCount: Integer;

procedure InitThreadSynchronization;
begin
  InitializeCriticalSection(ThreadLock);
  SyncEvent := CreateEvent(nil, True, False, '');
  if SyncEvent = 0 then Exit; //RaiseLastOSError;
end;

procedure DoneThreadSynchronization;
begin
  DeleteCriticalSection(ThreadLock);
  CloseHandle(SyncEvent);
end;

procedure ResetSyncEvent;
begin
  ResetEvent(SyncEvent);
end;

procedure WaitForSyncEvent(Timeout: Integer);
begin
  if WaitForSingleObject(SyncEvent, Timeout) = WAIT_OBJECT_0 then ResetSyncEvent;
end;

procedure SignalSyncEvent;
begin
  SetEvent(SyncEvent);
end;

procedure AddThread;
begin
  InterlockedIncrement(ThreadCount);
end;

procedure RemoveThread;
begin
  InterlockedDecrement(ThreadCount);
end;

function CheckSynchronize(Timeout: Integer = 0): Boolean;
var
  SyncProc: PSyncProc;
  LocalSyncList: TList;
begin
  Result:=False;
  if GetCurrentThreadID <> MainThreadID then Exit;
    //raise EThread.CreateResFmt(@SCheckSynchronizeError, [GetCurrentThreadID]);
  if Timeout > 0 then
    WaitForSyncEvent(Timeout)
  else
    ResetSyncEvent;
  LocalSyncList := nil;
  EnterCriticalSection(ThreadLock);
  try
    Integer(LocalSyncList) := InterlockedExchange(Integer(SyncList), Integer(LocalSyncList));
    try
      Result := (LocalSyncList <> nil) and (LocalSyncList.Count > 0);
      if Result then
      begin
        while LocalSyncList.Count > 0 do
        begin
          SyncProc := LocalSyncList[0];
          LocalSyncList.Delete(0);
          LeaveCriticalSection(ThreadLock);
          try
            try
              SyncProc.SyncRec.FMethod;
            except
              SyncProc.SyncRec.FSynchronizeException := AcquireExceptionObject;
            end;
          finally
            EnterCriticalSection(ThreadLock);
          end;
          SetEvent(SyncProc.signal);
        end;
      end;
    finally
      LocalSyncList.Free;
    end;
  finally
    LeaveCriticalSection(ThreadLock);
  end;
end;

function ThreadProc(Thread: TThread): Integer;
var
  FreeThread: Boolean;
begin
  try
    if not Thread.Terminated then
    try
      Thread.Execute;
    except
      Thread.FFatalException := AcquireExceptionObject;
    end;
  finally
    FreeThread := Thread.FFreeOnTerminate;
    Result := Thread.FReturnValue;
    //Thread.DoTerminate;
    Thread.FFinished := True;
    SignalSyncEvent;
    if FreeThread then Thread.Free;
    EndThread(Result);
  end;
end;

constructor TThread.Create(CreateSuspended: Boolean);
begin
  inherited Create;
  AddThread;
  FSuspended := CreateSuspended;
  FCreateSuspended := CreateSuspended;
  FHandle := BeginThread(nil, 0, @ThreadProc, Pointer(Self), CREATE_SUSPENDED, FThreadID);
  if FHandle = 0 then Exit;
    //raise EThread.CreateResFmt(@SThreadCreateError, [SysErrorMessage(GetLastError)]);
end;

destructor TThread.Destroy;
begin
  if (FThreadID <> 0) and not FFinished then
  begin
    Terminate;
    if FCreateSuspended then
      Resume;
    WaitFor;
  end;
  if FHandle <> 0 then CloseHandle(FHandle);
  inherited Destroy;
  FFatalException.Free;
  RemoveThread;
end;

procedure TThread.AfterConstruction;
begin
  if not FCreateSuspended then
    Resume;
end;

procedure TThread.CheckThreadError(ErrCode: Integer);
begin
  if ErrCode <> 0 then Exit;
    //raise EThread.CreateResFmt(@SThreadError, [SysErrorMessage(ErrCode), ErrCode]);
end;

procedure TThread.CheckThreadError(Success: Boolean);
begin
  if not Success then
    CheckThreadError(GetLastError);
end;

//procedure TThread.CallOnTerminate;
//begin
  //if Assigned(FOnTerminate) then FOnTerminate(Self);
//end;

//procedure TThread.DoTerminate;
//begin
  //if Assigned(FOnTerminate) then Synchronize(CallOnTerminate);
//end;

const
  Priorities: array [TThreadPriority] of Integer =
   (THREAD_PRIORITY_IDLE, THREAD_PRIORITY_LOWEST, THREAD_PRIORITY_BELOW_NORMAL,
    THREAD_PRIORITY_NORMAL, THREAD_PRIORITY_ABOVE_NORMAL,
    THREAD_PRIORITY_HIGHEST, THREAD_PRIORITY_TIME_CRITICAL);

function TThread.GetPriority: TThreadPriority;
var
  P: Integer;
  I: TThreadPriority;
begin
  P := GetThreadPriority(FHandle);
  CheckThreadError(P <> THREAD_PRIORITY_ERROR_RETURN);
  Result := tpNormal;
  for I := Low(TThreadPriority) to High(TThreadPriority) do
    if Priorities[I] = P then Result := I;
end;

procedure TThread.SetPriority(Value: TThreadPriority);
begin
  CheckThreadError(SetThreadPriority(FHandle, Priorities[Value]));
end;

class procedure TThread.Synchronize(ASyncRec: PSynchronizeRecord);
var
  SyncProc: TSyncProc;
begin
  if GetCurrentThreadID = MainThreadID then
    ASyncRec.FMethod
  else
  begin
    SyncProc.Signal := CreateEvent(nil, True, False, nil);
    try
      EnterCriticalSection(ThreadLock);
      try
        if SyncList = nil then
          SyncList := TList.Create;
        SyncProc.SyncRec := ASyncRec;
        SyncList.Add(@SyncProc);
        SignalSyncEvent;
        //if Assigned(WakeMainThread) then WakeMainThread(SyncProc.SyncRec.FThread);
        LeaveCriticalSection(ThreadLock);
        try
          WaitForSingleObject(SyncProc.Signal, INFINITE);
        finally
          EnterCriticalSection(ThreadLock);
        end;
      finally
        LeaveCriticalSection(ThreadLock);
      end;
    finally
      CloseHandle(SyncProc.Signal);
    end;
    if Assigned(ASyncRec.FSynchronizeException) then raise ASyncRec.FSynchronizeException;
  end;
end;

procedure TThread.Synchronize(Method: TThreadMethod);
begin
  FSynchronize.FThread := Self;
  FSynchronize.FSynchronizeException := nil;
  FSynchronize.FMethod := Method;
  Synchronize(@FSynchronize);
end;

class procedure TThread.Synchronize(AThread: TThread; AMethod: TThreadMethod);
var
  SyncRec: TSynchronizeRecord;
begin
  if AThread <> nil then
    AThread.Synchronize(AMethod)
  else
  begin
    SyncRec.FThread := nil;
    SyncRec.FSynchronizeException := nil;
    SyncRec.FMethod := AMethod;
    TThread.Synchronize(@SyncRec);
  end;
end;

class procedure TThread.StaticSynchronize(AThread: TThread; AMethod: TThreadMethod);
begin
  Synchronize(AThread, AMethod);
end;

procedure TThread.SetSuspended(Value: Boolean);
begin
  if Value <> FSuspended then
    if Value then
      Suspend
    else
      Resume;
end;

procedure TThread.Suspend;
var
  OldSuspend: Boolean;
begin
  OldSuspend := FSuspended;
  try
    FSuspended := True;
    CheckThreadError(Integer(SuspendThread(FHandle)) >= 0);
  except
    FSuspended := OldSuspend;
    raise;
  end;
end;

procedure TThread.Resume;
var
  SuspendCount: Integer;
begin
  SuspendCount := ResumeThread(FHandle);
  CheckThreadError(SuspendCount >= 0);
  if SuspendCount = 1 then
    FSuspended := False;
end;

procedure TThread.Terminate;
begin
  FTerminated := True;
end;

function TThread.WaitFor: LongWord;
var
  H: array[0..1] of THandle;
  WaitResult: Cardinal;
  Msg: TMsg;
begin
  H[0] := FHandle;
  if GetCurrentThreadID = MainThreadID then
  begin
    WaitResult := 0;
    H[1] := SyncEvent;
    repeat
      { This prevents a potential deadlock if the background thread
        does a SendMessage to the foreground thread }
      if WaitResult = WAIT_OBJECT_0 + 2 then
        PeekMessage(Msg, 0, 0, 0, PM_NOREMOVE);
      WaitResult := MsgWaitForMultipleObjects(2, H, False, 1000, QS_SENDMESSAGE);
      CheckThreadError(WaitResult <> WAIT_FAILED);
      if WaitResult = WAIT_OBJECT_0 + 1 then
        CheckSynchronize;
    until WaitResult = WAIT_OBJECT_0;
  end else WaitForSingleObject(H[0], INFINITE);
  CheckThreadError(GetExitCodeThread(H[0], Result));
end;

initialization
  InitThreadSynchronization;
  //AddModuleUnloadProc(ModuleUnload);
  //GlobalNameSpace := TMultiReadExclusiveWriteSynchronizer.Create;
  //RegGroups := TRegGroups.Create;
  //IntConstList := TThreadList.Create;
  //GlobalFixupList := TThreadList.Create;

finalization
  //FreeIntConstList;
  //FreeAndNil(GlobalFixupList);
  //FreeAndNil(GlobalLists);
  //FreeAndNil(RegGroups);
  //RemoveModuleUnloadProc(ModuleUnload);
  //FreeAndNil(SyncList);
  DoneThreadSynchronization;
  //FreeAndNil(FindGlobalComponentProcs);

end.

