unit EffectSystem;
{<|Author : Adler3D|>}
{<|e-mail : Adler3D@mail.ru|>}
{<|site : Adler3D.narod.ru|>}
interface

uses
  Windows,OpenGL,eXgine,Tools,ASDVector,QuickSprites,QuickList,GameRender,ASDStream;
const
  MaxDynPartCount=5000;
type
  PPart=^TPart;
  TPart=record
    Next,Prev:PPart;
    ID:DWORD;
    T,A,R:Real;
    Col:TRGBA;
    case Byte of
      0:(P,V:TVector);
      1:(pX,pY,vX,vY:Real);
  end;
  { TQuadEngine }
  TQuadEngine=class(TSpriteEngine){-=[TQuadEngine]=-}
  protected
{$I GetClassID.Inc}{$I DoLoad&Save.Inc}
  public
    {=[$Field$]=}
    Tick:Integer; // 
    {=[#Field#]=}
  end;
  TTickSprite=class(TSprite){-=[TTickSprite]=-}
  protected
    procedure DoMove; override;
{$I GetClassID.Inc}{$I DoLoad&Save.Inc}
  public
    {=[$Field$]=}
    Tick:Integer; // 
    {=[#Field#]=}
  end;
  { TEffectEngine }
  TEffect=class;
  TEffectEngine=class(TSpriteEngine){-=[TEffectEngine]=-}
  private
    function GetEffects(Index:Integer):TEffect;
    procedure InitDrawQuad;
  protected
{$I GetClassID.Inc}{$I DoLoad&Save.Inc}
  public
    DrawQuad:Cardinal;
    {=[$Field$]=}
    Temp:Byte; //Temp
    {=[#Field#]=}
    constructor Create(AParent:TSprite); override;
    function LoadTexture(FileName:string):TTexture;
    property Effects[Index:Integer]:TEffect read GetEffects; default;
  end;
  { TEffect }
  TEffect=class(TSprite){-=[TEffect]=-}
  private
    Tick,LastTick,MC,PartCount:Integer;
    First,Last:Pointer;
  protected
    VerArr:PVector2f;
    TexArr:PVector2f;
    ColArr:PRGBA;
    IndArr:PCardinal;
    procedure DoDraw; override;
    procedure DoMove; override;
    procedure MovePart(F:PPart); virtual; abstract;
    function GenPart:TPart; virtual; abstract;
{$I GetClassID.Inc}{$I DoLoad&Save.Inc}
  public
    //TextureFile:TString32;
    Effs:TEffectEngine;
    {=[$Field$]=}
    Texture:TTexture;
    Pos:TVector; //System Position
    PPos:TVector; //System Prev Position
    SLT:Real; //System LifeTime
    PLT:Real; //Part LifeTime
    EPS:Word; //Emission per second
    Loop:Boolean; //Loop system (ignore System LifeTime)
    {=[#Field#]=}
    constructor Create(AParent:TSprite); override;
    procedure AddPart(const P:TPart);
    procedure Reset;
    destructor Destroy; override;
  end;

  TDynEffect=class(TEffect){-=[TDynEffect]=-}
  protected
    function GenPart:TPart; override;
    procedure MovePart(F:PPart); override;
{$I GetClassID.Inc}{$I DoLoad&Save.Inc}
  public
    {=[$Field$]=}
    Col:TRGB; //Color
    Grav:TVector; //Gravity
    BegR:Real; //Start Radius
    EndR:Real; //End Radius
    Dir:Real; //Direction
    DirVib:Real; //Direction Vibration
    Mag:Real; //Magnitude
    MagVib:Real; //Magnitude Vibration
    {=[#Field#]=}
    constructor Create(AParent:TSprite); override;
  end;

  TRectEffect=class(TDynEffect){-=[TRectEffect]=-}
  protected
    function GenPart:TPart; override;
{$I GetClassID.Inc}{$I DoLoad&Save.Inc}
  public
    {=[$Field$]=}
    PosXVib:Real; //Position X Vibration
    PosYVib:Real; //Position Y Vibration
    {=[#Field#]=}
  end;

const
  C_LineWidth:Real=1;

implementation
uses
  Global;
//{-=| Warning!!! |=-}{$I WorldSystem_Scan.Inc}{-=| Warning!!! |=-}

function PointInRect(const Point:TVector; const Rect:TRect):Boolean;
begin
  Result:=(Point.X>=Rect.Left)and
    (Point.X<=Rect.Right)and
    (Point.Y>=Rect.Top)and
    (Point.Y<=Rect.Bottom);
end;

{ TTickSprite }

procedure TTickSprite.DoMove;
begin
  Inc(Tick);
end;

{ TEffectEngine }

constructor TEffectEngine.Create(AParent:TSprite);
begin
  inherited;
  ItemsList:=TList.Create; InitDrawQuad;
end;

function TEffectEngine.GetEffects(Index:Integer):TEffect;
begin
  Result:=TEffect(Items[Index]);
end;

function TEffectEngine.LoadTexture(FileName:string):TTexture;
begin
  Result:=tex.Load(PChar(ImgPartPath+FileName))
end;

procedure TEffectEngine.InitDrawQuad;
begin
  DrawQuad:=glGenLists(1);
  glNewList(DrawQuad,GL_COMPILE);
  glBegin(GL_QUADS);

  glTexCoord2d(0,0); glVertex2f(-1,-1);
  glTexCoord2d(1,0); glVertex2f(1,-1);
  glTexCoord2d(1,1); glVertex2f(1,1);
  glTexCoord2d(0,1); glVertex2f(-1,1);

  glEnd;
  glEndList;
end;

{ TEffect }

procedure TEffect.AddPart(const P:TPart);
var
  Dyn:PPart;
begin
  if EPS*PLT>MaxDynPartCount then
  begin
    EPS:=Trunc(MaxDynPartCount/PLT); Exit;
  end;
  New(Dyn); Dyn^:=P;
  if Last<>nil then PPart(Last).Next:=Dyn;
  if First=nil then First:=Dyn;
  Dyn.Next:=nil; Dyn.Prev:=Last; Last:=Dyn;
  Dyn.ID:=MC; Inc(MC);
end;

constructor TEffect.Create(AParent:TSprite);
begin
  GetMem(VerArr,SizeOf(VerArr^)*4*MaxDynPartCount);
  GetMem(TexArr,SizeOf(TexArr^)*4*MaxDynPartCount);
  GetMem(IndArr,SizeOf(IndArr^)*4*MaxDynPartCount);
  GetMem(ColArr,SizeOf(ColArr^)*4*MaxDynPartCount);
  inherited;
  if AParent is TEffectEngine then Effs:=AParent as TEffectEngine;
  Pos:=NulVector;
  SLT:=10;
  PLT:=5;
  EPS:=100;
  Tick:=0;
  Loop:=True;
  Last:=nil; First:=nil; PartCount:=0;
end;

destructor TEffect.Destroy;
begin
  FreeMem(TexArr);
  FreeMem(VerArr);
  FreeMem(IndArr);
  FreeMem(ColArr);
  inherited;
end;

procedure TEffect.DoDraw;
var
  C,J:Integer;
  F:PPart;
  dX,dY:Extended;
  xV,xT:PVector2f;
  xI:PCardinal;
  xC:PRGBA;
const
  P1:TVector=(X:-1; Y:-1);
  P2:TVector=(X:1; Y:1);
begin
  ogl.Blend(BT_ADD);
  glPushMatrix;
  tex.Enable(Texture);
  if Tick<>LastTick then
  begin
    F:=First; xV:=VerArr; xI:=IndArr; xT:=TexArr; xC:=ColArr; {J:=0;} C:=0;
    while F<>nil do
    begin
      if F.T>0 then
      begin
        SinCos(F.A*Pi/180,dY,dX); dX:=dX*F.R; dY:=dY*F.R; J:=C*4;
        xV.X:=F.pX-dX; xV.Y:=F.pY-dY; xC^:=F.Col;
        xT.X:=1; xT.Y:=1; xI^:=J; Inc(xV); Inc(xT); Inc(xI); Inc(xC); Inc(J);
        xV.X:=F.pX+dY; xV.Y:=F.pY-dX; xC^:=F.Col;
        xT.X:=0; xT.Y:=1; xI^:=J; Inc(xV); Inc(xT); Inc(xI); Inc(xC); Inc(J);
        xV.X:=F.pX+dX; xV.Y:=F.pY+dY; xC^:=F.Col;
        xT.X:=0; xT.Y:=0; xI^:=J; Inc(xV); Inc(xT); Inc(xI); Inc(xC); Inc(J);
        xV.X:=F.pX-dY; xV.Y:=F.pY+dX; xC^:=F.Col;
        xT.X:=1; xT.Y:=0; xI^:=J; Inc(xV); Inc(xT); Inc(xI); Inc(xC); //Inc(J);
        Inc(C);
      end;
      F:=F.Next;
    end; PartCount:=C; LastTick:=Tick;
  end;
  {if (Mode mod 600)<300 then
  begin}
  glEnableClientState(GL_TEXTURE_COORD_ARRAY);
  glTexCoordPointer(2,GL_FLOAT,0,PGLuint(TexArr));
  glEnableClientState(GL_VERTEX_ARRAY);
  glVertexPointer(2,GL_FLOAT,0,PGLuint(VerArr));
  glEnableClientState(GL_COLOR_ARRAY);
  glColorPointer(4,GL_UNSIGNED_BYTE,0,PGLuint(ColArr));
  glDrawElements(GL_QUADS,PartCount*4,GL_UNSIGNED_INT,PGLuint(IndArr));
  glDisableClientState(GL_COLOR_ARRAY);
  glDisableClientState(GL_VERTEX_ARRAY);
  glDisableClientState(GL_TEXTURE_COORD_ARRAY);
  //end else
    {while F<>nil do
    begin
      glColor4ubv(@F.Col);
      glPushMatrix;
      glTranslated(F.pX,F.pY,0);
      glRotated(F.A,0,0,1);
      glBegin(GL_QUADS);
      glTexCoord2d(0,0); glVertex2f(-F.R,-F.R);
      glTexCoord2d(1,0); glVertex2f(F.R,-F.R);
      glTexCoord2d(1,1); glVertex2f(F.R,F.R);
      glTexCoord2d(0,1); glVertex2f(-F.R,F.R);
      glEnd;
      glPopMatrix;
      F:=F.Next;
    end;}
  tex.Disable;
  glPopMatrix;
end;

procedure TEffect.DoMove;
var
  I,C:Integer;
  F,U:PPart;
  dT:Real;
begin// Mode:=Tick;
  Inc(Tick); F:=First; dT:=1/App.UPS; C:=0;
  while F<>nil do
  begin
    if F.T<=0 then
    begin
      if F.Prev=nil then First:=F.Next else F.Prev.Next:=F.Next;
      if F.Next=nil then Last:=F.Prev else F.Next.Prev:=F.Prev;
      U:=F; F:=U.Next; FreeMem(U); Continue;
    end else
    begin
      MovePart(F); F.T:=F.T-dT; F:=F.Next; Inc(C);
    end;
  end; PartCount:=C;
  if (SLT*App.UPS>=Tick)or Loop then
  begin
    C:=Round(Tick*EPS/App.UPS)-MC;
    for I:=1 to C do AddPart(GenPart);
  end else if PartCount=0 then Dead; PPos:=Pos;
end;

procedure TEffect.Reset;
begin
  MC:=0; Tick:=0;
end;

{ TDynEffect }

constructor TDynEffect.Create(AParent:TSprite);
const
  G:TVector=(X:0; Y:-0.0002);
begin
  inherited;
  Col:=RGB(255,255,255);
  Pos:=NulVector;
  SLT:=10;
  PLT:=5;
  EPS:=100;
  Tick:=0;
  Mag:=1; MagVib:=0;
  Dir:=0; DirVib:=PI2;
  BegR:=8;
  EndR:=8;
  Loop:=True;
  Grav:=G;
  Last:=nil; First:=nil; PartCount:=0;
end;

function TDynEffect.GenPart:TPart;
begin
  Result.vX:=RndReal(Dir-DirVib/2,Dir+DirVib/2,0.001);
  Result.vY:=RndReal(Mag-MagVib/2,Mag+MagVib/2,0.001);
  Result.V:=MakeVectorEx(Result.vX,Result.vY);
  Result.A:=RndReal(0,PI2);
  Result.P:=Pos;
  Result.Col:=RGBA(Col.R,Col.G,Col.B,255);
  Result.T:=PLT;
  Result.R:=BegR;
end;

procedure TDynEffect.MovePart(F:PPart);
begin
  F.V:=VectorAdd(F.V,Grav);
  F.P:=VectorAdd(F.V,F.P);
  F.R:=((BegR-EndR)*F.T/PLT+EndR)*sqrt(2);
  if (F.ID mod 2)=0 then F.A:=F.A+1 else F.A:=F.A-1;
  if F.T<=0 then
    F.Col:=RGBA(0,0,0,0) else
    F.Col:=RGBA(Col.R,Col.G,Col.B,Round(F.T*255/PLT));
end;

{TRectEffect}

function TRectEffect.GenPart:TPart;
begin
  Result:=inherited GenPart;
  Result.pX:=Result.pX+RndReal(-PosXVib,PosXVib);
  Result.pY:=Result.pY+RndReal(-PosYVib,PosYVib);
end;

procedure Copyright;
begin
  { <<< EffectSystem 0.1 by Adler3D >>> }
end;

exports
  Copyright name#10#13#10#13'<<< EffectSystem 0.1 by Adler3D >>>'#10#13#10#13;

end.

