unit QuickSprites;
{<|Author : Adler3D|>}
{<|e-mail : Adler3D@mail.ru|>}
{<|site : Adler3D.narod.ru|>}
interface
uses
  Windows,eXgine,ASDVector,QuickList,Tools,ASDStream;
//{$D-}
type
  TString32=string[32];

  TSprite=class;
  TSpriteEngine=class;

  TSpriteClass=class of TSprite;

  TSprite=class(TAccessObject){-=[Sprite]=-}
  private
    FEngine:TSpriteEngine;
    FParent:TSprite;
    FList:TList;
    {=[$Field$]=}
    {FieldName:FieldType; //FieldComment}
    {*..,..,..:FieldType; //FieldComment}
    FVisible:Boolean; //
    FMoved:Boolean; //
    FAllCount:Integer; // 
    FTag:Integer; // 
    FSID:Integer; //save id
    FDeaded:Boolean; //
    FCaption:string; //- 
    {=[#Field#]=}
    function GetCount:integer;
    function GetItems(Index:Integer):TSprite;
    procedure Add(Sprite:TSprite); virtual;
  protected
    procedure DoMove; virtual;
    procedure DoDraw; virtual;
    procedure DoSave(F:TStream); virtual;
    procedure DoLoad(F:TStream); virtual;
    class function GetClassID:Word; virtual;
  public
    constructor Create(AParent:TSprite); virtual;
    procedure Free; override;
    procedure Draw; virtual;
    procedure Move; virtual;
    procedure Dead; virtual;
    procedure Clear; virtual;
    procedure LoadFromFile(FileName:string); virtual;
    procedure LoadFromStream(F:TStream); virtual;
    procedure SaveToFile(FileName:string); virtual;
    procedure SaveToStream(F:TStream); virtual;
    property Engine:TSpriteEngine read FEngine;
    property Visible:Boolean read FVisible write FVisible;
    property Moved:Boolean read FMoved write FMoved;
    property Tag:Integer read FTag write FTag;
    property Count:integer read GetCount;
    property Deaded:boolean read FDeaded write FDeaded;
    property Caption:string read FCaption;
    property AllCount:Integer read FAllCount;
    property Parent:TSprite read FParent;
    property Items[Index:Integer]:TSprite read GetItems; default;
    property ItemsList:TList read FList write FList;
    property SaveID:Integer read FSID write FSID;
  end;

  TSpriteEngine=class(TSprite){-=[SpriteEngine]=-}
  protected
    FDeadList:TQuickList;
    class function GetClassID:Word; override;
    procedure GrabID(const A: TList);
  public
    constructor Create(AParent:TSprite); override;
    destructor Destroy; override;
    procedure Clear; override;
    procedure Dead; override;
    procedure Unmount;
    procedure ReplaceLink; virtual;
    procedure RestoreLink(A:TList); virtual;
    procedure LoadFromStream(F:TStream); override;
    procedure SaveToStream(F:TStream); override;
  end;

function TestColision(const P1,P2:TVector; const R1,R2:Real):Boolean;
function CreateSpriteFromID(AParent:TSprite; ID:Word):TSprite;
function RegSpriteClass(Reg:TSpriteClass):Word;

implementation
var
  Factory:array[0..100] of TSpriteClass;
  FactoryCount:Integer;

function CreateSpriteFromID(AParent:TSprite; ID:Word):TSprite;
begin
  Result:=Factory[ID].Create(AParent);
end;

function RegSpriteClass(Reg:TSpriteClass):Word;
begin
  if FactoryCount=0 then
  begin
    Factory[0]:=TSprite; Factory[1]:=TSpriteEngine; FactoryCount:=2;
  end;
  Factory[FactoryCount]:=Reg; Result:=FactoryCount; Inc(FactoryCount);
end;

function TestColision(const P1,P2:TVector; const R1,R2:Real):Boolean;
begin
  Result:=((P1.X-P2.X)*(P1.X-P2.X)+(P1.Y-P2.Y)*(P1.Y-P2.Y))<=((R1+R2)*(R1+R2));
end;

{ TSprite }

procedure TSprite.Add(Sprite:TSprite);
begin
  if FList=nil then
  begin
    //FList:=TQuickList.Create;
    FList:=TList.Create;
  end;
  FList.Add(Sprite);
end;

constructor TSprite.Create(AParent:TSprite);
begin
  inherited Create;
  FParent:=AParent;
  if FParent<>nil then
  begin
    FParent.Add(Self);
    if FParent is TSpriteEngine then
      FEngine:=TSpriteEngine(FParent)
    else
      FEngine:=FParent.Engine;
    Inc(FEngine.FAllCount);
    FCaption:=ClassName+' #'+IntToStr(FEngine.AllCount);
  end
  else
    FCaption:=ClassName+' Boss';
  FMoved:=True;
  FVisible:=True;
  Lock;
end;

procedure TSprite.Dead;
begin
  if (FEngine<>nil)and not FDeaded then
    FEngine.FDeadList.Add(Self);
  FDeaded:=True;
end;

procedure TSprite.Clear;
begin
  while Count>0 do
    Items[Count-1].Free;
  if FList<>nil then
  begin
    FList.Free;
    FList:=nil;
  end;
end;

procedure TSprite.Free;
begin
  Clear;
  UnLock;
end;

function TSprite.GetCount:integer;
begin
  if FList<>nil then
    Result:=FList.Count
  else
    Result:=0;
end;

function TSprite.GetItems(Index:Integer):TSprite;
begin
  Result:=FList[Index];
end;

procedure TSprite.Move;
var
  i:integer;
begin
  if FMoved then
  begin
    DoMove;
    for i:=0 to Count-1 do
      Items[i].Move;
  end;
end;

procedure TSprite.DoDraw;
begin
end;

procedure TSprite.DoMove;
begin
end;

procedure TSprite.Draw;
var
  i:integer;
begin
  if FVisible then
  begin
    DoDraw;
    for I:=0 to Count-1 do Items[I].Draw;
  end;
end;

procedure TSprite.LoadFromFile(FileName:string);
var
  F:TFileStream;
begin
  F:=TFileStream.Create(FileName,fmOpenRead or fmShareDenyRead);
  LoadFromStream(F);
  F.Free;
end;

procedure TSprite.LoadFromStream(F:TStream);
var
  I,C:Integer;
  ID:Word;
begin
  Clear;
  DoLoad(F); F.ReadBuffer(C,SizeOf(C));
  for I:=0 to C-1 do
  begin
    F.ReadBuffer(ID,SizeOf(ID)); //This ID number is uniquely identify ClassType
    CreateSpriteFromID(Self,ID).LoadFromStream(F);
  end;
end;

procedure TSprite.SaveToFile(FileName:string);
var
  F:TFileStream;
begin
  F:=TFileStream.Create(FileName,fmCreate);
  SaveToStream(F);
  F.Free;
end;

procedure TSprite.SaveToStream(F:TStream);
var
  I,C:Integer;
  ID:Word;
begin
  DoSave(F); C:=Count; F.WriteBuffer(C,SizeOf(C));
  for I:=0 to C-1 do
  begin
    ID:=Items[I].GetClassID; //This ID number is uniquely identify ClassType
    F.WriteBuffer(ID,SizeOf(ID)); Items[I].SaveToStream(F)
  end;
end;

procedure TSprite.DoLoad(F:TStream);
(*{=[$Field$]=}
{FieldName:FieldType; //FieldComment}
{*..,..,..:FieldType; //FieldComment}
FVisible:Boolean; //
FMoved:Boolean; //
FAllCount:Integer; // 
FTag:Integer; // 
FDeaded:boolean; //
FCaption:TString32; //- 
{=[#Field#]=}*)
var
  C:Integer;
begin
  //F.ReadBuffer(FieldName,SizeOf(FieldType)); {FieldComment}
  F.ReadBuffer(FVisible,SizeOf(FVisible)); {}
  F.ReadBuffer(FMoved,SizeOf(FMoved)); {}
  F.ReadBuffer(FAllCount,SizeOf(FAllCount)); { }
  F.ReadBuffer(FTag,SizeOf(FTag)); { }
  F.ReadBuffer(FSID,SizeOf(FSID)); {save id}
  F.ReadBuffer(FDeaded,SizeOf(FDeaded)); {}
  F.ReadBuffer(C,SizeOf(C));
   SetLength(FCaption,C);
    F.ReadBuffer(FCaption[1],C); {- }
    if Self=nil then Exit;
end;

procedure TSprite.DoSave(F:TStream);
(*{=[$Field$]=}
{FieldName:FieldType; //FieldComment}
{*..,..,..:FieldType; //FieldComment}
FVisible:Boolean; //
FMoved:Boolean; //
FAllCount:Integer; // 
FTag:Integer; // 
FDeaded:boolean; //
FCaption:TString32; //- 
{=[#Field#]=}*)
var
  C:Integer;
begin
  //F.WriteBuffer(FieldName,SizeOf(FieldType)); {FieldComment}
  F.WriteBuffer(FVisible,SizeOf(FVisible)); {}
  F.WriteBuffer(FMoved,SizeOf(FMoved)); {}
  F.WriteBuffer(FAllCount,SizeOf(FAllCount)); { }
  F.WriteBuffer(FTag,SizeOf(FTag)); { }
  F.WriteBuffer(FSID,SizeOf(FSID)); {save id}
  F.WriteBuffer(FDeaded,SizeOf(FDeaded)); {}
  C:=Length(FCaption);
   F.WriteBuffer(C,SizeOf(C));
    F.WriteBuffer(FCaption[1],C); {- }
    if Self=nil then Exit;
end;

class function TSprite.GetClassID:Word;
{class function ClassType.GetClassID: Word;}
begin
  {Result:=ClassID}
  Result:=0;
end;

{ TSpriteEngine }

procedure TSpriteEngine.Clear;
var
  I:Integer;
begin
  for I:=0 to Count-1 do
  begin
    if Items[I]is TSpriteEngine then Items[I].Clear else Items[I].Dead;
  end;
  Dead;
end;

constructor TSpriteEngine.Create(AParent:TSprite);
begin
  inherited;
  FDeadList:=TQuickList.Create;
end;

procedure TSpriteEngine.Dead;
var
  I,J:Integer;
  S:TSprite;
begin
  for I:=FDeadList.Count-1 downto 0 do
  begin
    S:=TSprite(FDeadList[I]);
    for J:=FList.Count-1 downto 0 do if FList[J]=S then FList.Delete(J);
    FDeadList.Delete(I); S.Free;
  end;
end;

destructor TSpriteEngine.Destroy;
begin
  FDeadList.Free;
  inherited;
end;

class function TSpriteEngine.GetClassID:Word;
{class function ClassType.GetClassID: Word;}
begin
  {Result:=ClassID}
  Result:=1;
end;

procedure TSpriteEngine.LoadFromStream(F:TStream);
var
  I,C:Integer;
  ID:Word;
begin
  Clear; //dead only TSprite!
  F.ReadBuffer(C,SizeOf(C));
  if C>0 then //Count Engine
  begin
    for I:=0 to Count-1 do {if Items[I] is TSpriteEngine then} Dec(C);
    if C<>0 then
    begin
      for I:=0 to Count-1 do TSpriteEngine(Items[I]).Unmount; Dead;
      inherited; Exit;
    end;
    DoLoad(F); F.ReadBuffer(C,SizeOf(C));
    if Count<>C then Exit;
    for I:=0 to C-1 do
    begin
      F.ReadBuffer(ID,SizeOf(ID)); //This ID number is uniquely identify ClassType
      if Factory[ID]=Items[I].ClassType then Items[I].LoadFromStream(F) else
      begin
        log.Print('TSpriteEngine:Loading Error.'); Exit;
      end;
    end;
  end else inherited;
end;

procedure TSpriteEngine.SaveToStream(F:TStream);
var
  I,C:Integer;
begin
  C:=0; for I:=0 to Count-1 do if Items[I]is TSpriteEngine then Inc(C);
  F.WriteBuffer(C,SizeOf(C));
  inherited;
end;

procedure TSpriteEngine.Unmount;
begin
  inherited Dead;
end;

procedure TSpriteEngine.GrabID(const A:TList);
var
  I:Integer;
begin
  for I:=0 to Count-1 do
    if not(Items[I] is TSpriteEngine) then
    begin
      Items[I].SaveID:=A.Count; A.Add(Items[I]);
    end else TSpriteEngine(Items[I]).GrabID(A);
end;

procedure TSpriteEngine.ReplaceLink;
var
  I:Integer;
begin
  for I:=0 to Count-1 do if Items[I] is TSpriteEngine then TSpriteEngine(Items[I]).ReplaceLink;
end;

procedure TSpriteEngine.RestoreLink(A: TList);
var
  I:Integer;
begin
  for I:=0 to Count-1 do if Items[I] is TSpriteEngine then TSpriteEngine(Items[I]).RestoreLink(A);
end;

end.

