unit Scene;
{<|Author : Adler3D|>}
{<|e-mail : Adler3D@mail.ru|>}
{<|site : Adler3D.narod.ru|>}
interface
uses
  Windows,Messages,eXgine,OpenGL,Tools,QuickList,TextSystem;
type
  TStr32=string[32];
  TScene=class(TObject)
  private
    FName:string;
  protected
    procedure SystemUpdate; virtual;
    procedure SystemRender; virtual;
  public
    constructor Create(Name:string); virtual;
    procedure Render; virtual;
    procedure Update; virtual;
    property SceneName:string read FName;
    destructor Destroy; override;
  end;

procedure Update;
procedure Render;
procedure UseMessage(Msg:LongWord; wP,lP:Integer);
procedure InputStart(S:string='');
function InputStop:string;
procedure InputClear;
function InputRead:string;
procedure InputWrite(S:string);

var
  ActivScene:TScene;
  ShowFPS:Boolean;
implementation

uses Global,ASDStream;
var
  SceneList:TQuickList;
  Select:Integer;
  InputStr:string;
  RunInput:Boolean=False;
  eXRun:Boolean=True;

procedure DefRender;
var
  X,Y,I:Integer;
  procedure AddText(S:string; AX:Word=0; AY:Word=0);
  begin
    if AX<>0 then X:=AX; if AY<>0 then Y:=AY;
    DrawTextQ3(0,X,Y,'^7'+S); Inc(Y,15);
    //ogl.TextOut(0,X,Y,PChar(S)); Inc(Y,15);
  end;
begin
  glClearColor(0,0,0,0); ogl.Clear; ogl.Set2D(0,0,wnd.Width,wnd.Height);
  with App do
  begin
    AddText('^1System menu',10,10);
    AddText('Concurs : '+Concurs);
    AddText('AppName : '+Name);
    AddText('AppVer : '+Ver);
    AddText('Author : '+Author);
    if SceneList.Count=0 then AddText('SelectScene : ^2# - ^3nil') else
      AddText('SelectScene : ^2'+IToS(Select)+' - ^3'+TScene(SceneList[Select]).FName);
    AddText('Scene Count : '+IToS(SceneList.Count));
    glColor3f(1,1,0); AddText('');
    for I:=0 to SceneList.Count-1 do
      AddText('^2'+IToS(I)+' - ^3'+TScene(SceneList[I]).FName);
    AddText('^2Time : '+TimeToStr(GetTime),Wnd.Width-200,10);
    AddText('^2Date : '+DateToStr(GetDate));
    AddText('^2FPS : '+IToS(ogl.FPS));
  end;
end;

procedure DefUpdate;
begin
  if inp.Down(27) then eX.Quit;
  with SceneList do
  begin
    if inp.Down(VK_UP)and(Select<Count-1) then Inc(Select);
    if inp.Down(VK_Down)and(Select>0) then Dec(Select);
    if inp.Down(VK_RETURN)and(Select>=0) then ActivScene:=TScene(Items[Select]);
  end;
  inp.Reset;
end;

procedure Update;
begin
  if ActivScene<>nil then ActivScene.SystemUpdate else DefUpdate;
end;

procedure Render;
begin
  if ActivScene<>nil then ActivScene.SystemRender else DefRender;
end;

procedure UseMessage(Msg:LongWord; wP,lP:Integer);
const
  Path='..\ScreenShots\';
var
  TS,DS:Char;
begin
  if (msg=WM_KEYUP)and(wp=VK_SNAPSHOT) then
  begin
    if not DirExists(Path) then MkDir(Path);
    TS:=TimeSeparator; DS:=DateSeparator; TimeSeparator:='-'; DateSeparator:='-';
    ogl.ScreenShot(PChar(Path+App.Name+DateToStr(GetDate)+'-'+TimeToStr(GetTime)+'.bmp'));
    TimeSeparator:=TS; DateSeparator:=DS;
    inp.Reset;
  end;
  if (msg=WM_SYSKEYDOWN)and(wp=VK_F10)and(ActivScene<>nil) then
  begin
    Select:=SceneList.IndexOf(ActivScene); ActivScene:=nil;
  end;
  if (msg=WM_CHAR)and RunInput then
  begin
    if Chr(wp)in [chr(32)..chr(63),'A'..'z',''..''] then
      InputStr:=InputStr+Chr(wp) else
      case wp of
        VK_BACK:InputStr:=Copy(InputStr,1,Length(InputStr)-1);
      end;
  end;
  if App.IsRun then CheckSynchronize;
end;

procedure InputStart(S:string);
begin
  RunInput:=True;
  InputStr:=S;
end;

function InputStop:string;
begin
  RunInput:=False;
  Result:=InputStr;
end;

procedure InputClear;
begin
  InputStr:='';
end;

function InputRead:string;
begin
  Result:=InputStr;
end;

procedure InputWrite(S:string);
begin
  InputStr:=S;
end;

{ TScene }

constructor TScene.Create(Name:string);
begin
  FName:=Name; SceneList.Add(Self);
end;

destructor TScene.Destroy;
begin
  SceneList.Extract(Self); inherited;
end;

procedure TScene.Render;
begin
  DefRender;
end;

procedure TScene.SystemRender;
begin
  Render;
  if ShowFPS then DrawTextQ3(0,10,10,'^7FPS : '+IToS(ogl.FPS));
end;

procedure TScene.SystemUpdate;
begin
  Update;
end;

procedure TScene.Update;
begin
  DefUpdate;
end;

initialization
  SceneList:=TQuickList.Create; ShowFPS:=True;
end.

