unit ASDLine;
{<|Author : Adler3D|>}
{<|e-mail : Adler3D@mail.ru|>}
{<|site : Adler3D.narod.ru|>}
interface

uses eXgine,ASDVector,OpenGL,QuickSprites,QuickList,ASDStream;

type
  TLineFunction=(lfCollision,lfDead,lfStop);
  PLineBot=^TLineBot;

  TLineBot=record
    F:TLineFunction;
    C:TRGBA;
    case Byte of
      1:(
        L:TLine);
      2:(
        A:TVector;
        B:TVector);
  end;

  THeaderASD=record
    Magic:string[4];
    Count:Integer;
  end;

  TLineEngine=class(TSpriteEngine){MAP}
  private
    FList:TList;
    FChange:Boolean;
    function GetCount:Cardinal;
    function GetLines(Index:Integer):TLine;
    procedure SetLine(Index:Integer; const Value:TLine);
    function GetLineBots(Index:Integer):TLineBot;
    procedure SetLineBots(Index:Integer; const Value:TLineBot);
  protected
    procedure DoDraw; override;
  public
    constructor Create(AParent:TSprite); override;
    destructor Destroy; override;
    property Lines[Index:Integer]:TLine read GetLines write SetLine; default;
    property LineBots[Index:Integer]:TLineBot read GetLineBots write
    SetLineBots;
    property Count:Cardinal read GetCount;
    property Change:Boolean read FChange write FChange;
    procedure Add(Line:TLineBot);
    procedure AddRect(X,Y,W,H:Real);
    procedure Clear; override;
    procedure LoadFromStream(F:TStream); override;
    procedure SaveToStream(F:TStream); override;
  end;

function DoCollision(var P,V:TVector; Line:TLine; R:Real):Boolean;
function ColideLine(const A,B,C,D:TVector; var O:TVector):Boolean;

implementation

function DoCollision(var P,V:TVector; Line:TLine; R:Real):Boolean;
const//Line is not linear segment!!!
  C_LineWidth=1;
  //  Kof_Up=1.5;
var
  AB,_V,_P:TVector;
  L:Real;
begin
  Result:=False;
  with Line do
  begin
    AB:=VectorSub(B,A); L:=VectorMagnitude(AB);
    _V:=VectorRot(V,AB);
    _P:=VectorRot(VectorSub(P,A),AB);
    if Abs(_P.Y)>R then Exit;
    if _P.X<R then Exit;
    if _P.X+R>L then Exit;
    //if _P.X+R<L then Exit;
    if _V.Y<0 then Exit;
    //_V.Y:=_V.Y-(R-Abs(_P.Y))*Kof_Up;
    //_V.Y:=_V.Y-Abs(_P.Y)*0.1;
    _V.Y:=-_V.Y;
    //_P.Y:=(Abs(_P.Y)*R/_P.Y){+_V.X};
    V:=VectorUnRot(_V,AB);
    //P:=VectorAdd(A,VectorUnRot(_P,AB));
  end;
  Result:=True;
end;

//(*

function ColideLine(const A,B,C,D:TVector; var O:TVector):Boolean;
{$I .Inc}
begin
  Result:=LineCollision(A,B,C,D,O);
end;
//*)
(*
function ColideLine(const A,B,C,D:TVector; var O:TVector):Boolean;
{$I (by_Dobord).inc}
begin
  Result:=PerLines(A,B,C,D,O)=1;
end;
*)

{ TLineFile }

procedure TLineEngine.Add(Line:TLineBot);
var
  P:PLineBot;
begin
  New(P);
  P^:=Line;
  FList.Add(P);
  FChange:=True;
end;

procedure TLineEngine.AddRect(X,Y,W,H:Real);
const
  R=8;
var
  L:TLineBot;
begin
  L.F:=lfCollision;
  L.C:=RGBA(255,128,0,255);
  if H<0 then
  begin
    L.A:=MakeVector(X,Y);
    L.B:=MakeVector(X+W,Y);
    Add(L); //Y=0
    L.A:=L.B; L.B:=MakeVector(X+W,Y+H);
    Add(L); //X=0
    L.A:=L.B; L.B:=MakeVector(X,Y+H);
    Add(L); //Y=0
    L.A:=L.B; L.B:=MakeVector(X,Y);
    Add(L); //X=0
  end else
  begin
    L.A:=MakeVector(X-R,Y);
    L.B:=MakeVector(X+W+R,Y);
    Add(L); //Y=0
    L.A:=MakeVector(X+W,Y); L.B:=MakeVector(X+W,Y+H);
    Add(L); //X=0
    L.A:=MakeVector(X+W+R,Y+H); L.B:=MakeVector(X-R,Y+H);
    Add(L); //Y=0
    L.A:=MakeVector(X,Y+H); L.B:=MakeVector(X,Y);
    Add(L); //X=0    
  end;
end;

procedure TLineEngine.Clear;
begin
  FList.Clear;
  FChange:=True;
end;

constructor TLineEngine.Create(AParent:TSprite);
begin
  inherited;
  FList:=TList.Create;
  FChange:=False;
end;

destructor TLineEngine.Destroy;
begin
  FList.Free;
  inherited;
end;

procedure TLineEngine.DoDraw;
var
  I:Integer;
  L:TLineBot;
begin
  inherited;
  glPushMatrix;
  //glLineWidth(1);
  glBegin(GL_LINES);
  for I:=0 to Count-1 do
  begin
    L:=LineBots[I];
    glColor4ubv(@L.C);
    glVertex2dv(@L.A);
    glVertex2dv(@L.B);
  end;
  glEnd;
  glPopMatrix;
end;

function TLineEngine.GetCount:Cardinal;
begin
  Result:=FList.Count;
end;

function TLineEngine.GetLineBots(Index:Integer):TLineBot;
begin
  Result:=PLineBot(FList[Index])^;
end;

function TLineEngine.GetLines(Index:Integer):TLine;
begin
  Result:=PLineBot(FList[Index])^.L;
end;

procedure TLineEngine.LoadFromStream(F:TStream);
const
  C_LoadError:PChar='Load Error : "this file is not ASDFile"';
var
  H:THeaderASD;
  I,C:Integer;
  PL:PLineBot;
begin
  F.ReadBuffer(H,SizeOf(H));
  if H.Magic<>'ASD ' then Log.Print(C_LoadError);
  FList.Clear;
  F.ReadBuffer(H,SizeOf(H));
  C:=H.Count;
  for I:=1 to C do
  begin
    New(PL);
    F.ReadBuffer(PL^,SizeOf(TLineBot));
    FList.Add(PL);
  end;
end;

procedure TLineEngine.SaveToStream(F:TStream);
var
  H:THeaderASD;
  I:Integer;
begin
  H.Magic:='ASD ';
  H.Count:=FList.Count;
  F.WriteBuffer(H,SizeOf(H));
  for I:=0 to FList.Count-1 do
  begin
    F.WriteBuffer(PLineBot(FList[I])^,SizeOf(TLineBot));
  end;
end;

procedure TLineEngine.SetLine(Index:Integer; const Value:TLine);
begin
  PLineBot(FList[Index])^.L:=Value;
end;

procedure TLineEngine.SetLineBots(Index:Integer; const Value:TLineBot);
begin
  PLineBot(FList[Index])^:=Value;
end;

end.

