unit GameScene;
{<|Author : Adler3D|>}
{<|e-mail : Adler3D@mail.ru|>}
{<|site : Adler3D.narod.ru|>}
interface
uses
  Windows,eXgine,Scene,OpenGL,ASDVector,QuickSprites,ASDLine,GameMouse,GameEntity,EffectSystem;
type
  TWorldEngine=class(TSpriteEngine)
  public
    procedure Dead; override;
  end;
  TGameScene=class(TScene)
  private
    FNearBonus:TAward;
    FCurMapID:Integer;
    procedure SetNearBonus(const Value:TAward);
  protected
    function OneShot(Weapon:PWeapon; Owner:TEntity):TBullet;
    function TwoShot(Weapon:PWeapon; Owner:TEntity):TBullet;
    function DoubleShot(Weapon:PWeapon; Owner:TEntity):TBullet;
    function PusShot(Weapon:PWeapon; Owner:TEntity):TBullet;
    procedure EventHandler(Sender:TEntity);
    procedure EventKill(Sender:TEntity);
    procedure EvenFinish(Sender:TEntity);
    procedure RenderHint;
    procedure NextMap;
    function SetupEffFromBase(Eff:TDynEffect; ID:Byte):TDynEffect;
  public
    {#<Engines#}
    World:TWorldEngine;
    LE:TLineEngine;
    TE:TTowerEngine;
    BE:TBulletEngine;
    EE:TExtraEngine;
    AE:TAwardEngine;
    ES:TEffectEngine;
    {#Engines>#}
    constructor Init;
    procedure NewGame;
    procedure ReloadMap;
    procedure LoadNextMap;
    procedure GenPoint(A,B:TVector);
    procedure Collider;
    procedure RenderPA;
    procedure Render; override;
    procedure Update; override;
    procedure GenLevel1;
    procedure GenLevel2;
    procedure GenLevel3;
    procedure ScanWorld;
    function FindNearBonus:TAward;
    procedure AddExplosion(P:TVector; Force:Real; B:TBullet);
    property NearBonus:TAward read FNearBonus write SetNearBonus;
  end;
  TGenMapProc=procedure of object;
var
  Weapons:array[0..10] of TWeapon;
  User:TUserTower;
  UserEff:TDynEffect;
  CamPos,CamV:TVector;
  P,MP,V:TVector;
  FPA:TVector;
  FPAM:Real;
  PA:array[0..1000] of TVector;
  PACount:Integer;
  Grab:Boolean;
  AllBonus,Tick:Integer;
  EndOfMap:Boolean;
  AwardFinish:TTextAward;
  Maps:array[1..3] of TGenMapProc;
const
  D=16;
implementation
uses
  GameRender,Tools,TextSystem,Global;

{$I FMOD\ufmod.inc *** uFMOD API: WINMM version}

function EquipWeapon(Target:TEntity; DoShot:TShotProc; TexID:Integer):TWeapon;
begin
  Result.Target:=Target;
  Result.DoShot:=DoShot;
  Result.LastBullet:=nil;
  Result.TexID:=TexID;
end;

{ TWorldEngine }

procedure TWorldEngine.Dead;
var
  I:Integer;
begin
  for I:=0 to Count-1 do
  begin
    Items[I].Dead;
  end;
end;

{ TGameScene }

procedure TGameScene.Collider;
{$I 2.Inc}
var
  I:Integer;
  A:TTower;
  B:TBullet;
  //  L:TLine;
  Fail:TVector;
begin
  Fail:=User.V;
  for I:=0 to LE.Count-1 do
  begin
    if DoCollision(User.P,User.V,LE.Lines[I],D/2) then
    begin
      User.V:=VectorMul(User.V,0.7);
    end;
  end;
  if VectorABDlina(User.V,Fail)>0.5 then snd.Play(Wav[6],0,0,0);
  //  ,    
  {for I:=0 to LE.Count-1 do
  begin
    L:=LE[I];
    if TestColision(User.P,L.A,User.R,0.5) then
    begin
      CollisionBool(User.P,L.A,1,10,User.V,Fail); User.V:=VectorMul(User.V,0.7);
      if VectorMagnitude(User.V)>0.3 then snd.Play(Wav[6],0,0,0); Break;
    end;
    if TestColision(User.P,L.B,User.R,0.5) then
    begin
      CollisionBool(User.P,L.B,1,1000,User.V,Fail); User.V:=VectorMul(User.V,0.7);
      if VectorMagnitude(User.V)>0.3 then snd.Play(Wav[6],0,0,0); Break;
    end;
  end;  }
  BE.Collider;
  A:=User;
  for I:=0 to BE.Count-1 do
  begin
    B:=TBullet(BE[I]);
    if TestColision(A.P,B.P,A.R,B.R) then
    begin
      //CollisionBool(A.P,B.P,A.M,B.M,A.V,B.V);
      B.Dead;
      snd.Play(Wav[5],0,0,0);
    end;
  end;
end;

procedure TGameScene.GenPoint(A,B:TVector);
const
  MaxLength=1000;
var
  I:Integer;
  L:TLineBot;
  O:TVector;
begin
  PACount:=0;
  //O:=VectorSub(A,B);
  {if VectorABDlina(A,B)>MaxLength then Exit else }B:=VectorAdd(A,VectorSetDlina(VectorSub(B,A),MaxLength));
  for I:=0 to LE.Count-1 do
  begin
    L:=LE.LineBots[I];
    if ColideLine(A,B,L.A,L.B,O) then
    begin
      PA[PACount]:=O; Inc(PACount);
    end;
  end;
  if PACount<0 then Exit;
  FPA:=PA[0];
  for I:=1 to PACount-1 do
  begin
    if VectorABDlina(P,PA[I])<VectorABDlina(P,FPA) then FPA:=PA[I];
  end;
  FPAM:=5; //VectorMagnitude(VectorSub(FPA,P));
end;

constructor TGameScene.Init;
//var
//  I:Integer;
begin
  inherited Create('Game Scene'); Zoom:=1; Tick:=0;
  {-=Init Engines=-}
  World:=TWorldEngine.Create(nil);
  LE:=TLineEngine.Create(World);
  AE:=TAwardEngine.Create(World);
  BE:=TBulletEngine.Create(World);
  TE:=TTowerEngine.Create(World);
  EE:=TExtraEngine.Create(World);
  ES:=TEffectEngine.Create(World);
  {-=Init Level=-}
  FCurMapID:=1;
  Maps[1]:=GenLevel1;
  Maps[2]:=GenLevel2;
  Maps[3]:=GenLevel3;
  NextMap;
end;

function TGameScene.OneShot(Weapon:PWeapon; Owner:TEntity):TBullet;
begin
  Result:=TBullet.Create(BE);
  Result.SetUPA(2000);
  Result.SetR(8);
  Result.V:=VectorSetDlina(VectorSub(Weapon.Target.P,Owner.P),4);
  if Owner is TUserTower then
    Result.P:=VectorAdd(Owner.P,VectorSetDlina(Result.V,Result.R+Owner.R+1)) else
    Result.P:=VectorAdd(Owner.P,VectorSetDlina(Result.V,Result.R+Owner.R+16));
  Result.C:=clWhite;
  Result.A:=NulVector;
  Result.Target:=Weapon.Target;
  Result.TexID:=9;
  Result.Owner:=Owner;
end;

function TGameScene.TwoShot(Weapon:PWeapon; Owner:TEntity):TBullet;
begin
  Result:=THomingBullet.Create(BE);
  Result.SetUPA(10+Random(5));
  Result.SetR(8);
  Result.V:=VectorSetDlina(VectorSub(Weapon.Target.P,Owner.P),4);
  Result.P:=VectorAdd(Owner.P,VectorSetDlina(Result.V,Result.R+Owner.R+16));
  Result.C:=clWhite;
  Result.A:=NulVector;
  Result.Target:=Weapon.Target;
  Result.TexID:=5;
  Result.Owner:=Owner;
end;

function TGameScene.PusShot(Weapon:PWeapon; Owner:TEntity):TBullet;
begin
  Result:=TPusBullet.Create(BE); //...   .
  Result.SetUPA(10+Random(5));
  Result.SetR(8);
  Result.V:=VectorSetDlina(VectorSub(Weapon.Target.P,Owner.P),4);
  Result.P:=VectorAdd(Owner.P,VectorSetDlina(Result.V,Result.R+Owner.R+16));
  Result.C:=clWhite;
  Result.A:=NulVector;
  Result.Target:=Weapon.Target;
  Result.TexID:=10;
  Result.Owner:=Owner;
end;

function TGameScene.DoubleShot(Weapon:PWeapon; Owner:TEntity):TBullet;
var
  CPos:TVector;
begin
  Result:=TBullet.Create(BE);
  Result.SetUPA(2000);
  Result.SetR(7);
  Result.V:=VectorSetDlina(VectorSub(Weapon.Target.P,Owner.P),4);
  CPos:=VectorAdd(Owner.P,VectorSetDlina(Result.V,Result.R+Owner.R+16));
  Result.P:=VectorAdd(CPos,VectorSetDlina(MakeVector(-Result.VY,Result.VX),Result.R*0.7));
  Result.C:=clWhite;
  Result.A:=NulVector;
  Result.Target:=Weapon.Target;
  Result.TexID:=9;
  Result.Owner:=Owner;

  Result:=TBullet.Create(BE);
  Result.SetUPA(2000);
  Result.SetR(7);
  Result.V:=VectorSetDlina(VectorSub(Weapon.Target.P,Owner.P),4);
  Result.P:=VectorAdd(CPos,VectorSetDlina(MakeVector(+Result.VY,-Result.VX),Result.R*0.7));
  Result.C:=clWhite;
  Result.A:=NulVector;
  Result.Target:=Weapon.Target;
  Result.TexID:=9;
  Result.Owner:=Owner;
end;

procedure TGameScene.Render;
begin
  ogl.Set2D(0,0,wnd.Width,wnd.Height); ogl.Clear;
  ogl.Blend(BT_SUB);
  {-=Render Engine=-}
  glTranslated(-CamPos.X+wnd.Width/2,-CamPos.Y+wnd.Height/2,0);
  RenderHint;
  World.Draw; //LE.Draw; TE.Draw; BE.Draw; EE.Draw; AE.Draw;
  {-=Render Grab-Line=-}
  glColor3d(0,1,1);
  tex.Enable(Img[1]); RenderPA; tex.Disable;
  glColor3d(0.5,0.5,0.5);
  if Grab then DrawLine(P,FPA);
  ogl.Set2D(0,0,wnd.Width,wnd.Height);
  RenderMouse(Img[99],True);
  DrawTextQ3(0,10,50,'^2Mouse Coord : ['+FToS(MP.X,0,0)+' '+FToS(MP.Y,0,0)+']');
  DrawTextQ3(0,10,70,'^3Bonus Last : '+IToS(AllBonus));
end;

procedure TGameScene.RenderPA;
{var
  I:Integer;}
begin
  //for I:=0 to PACount-1 do DrawQuad(PA[I].X,PA[I].Y,8,8);
  if Grab then
  begin
    glColor3d(1,1,1);
    DrawQuad(FPA.X,FPA.Y,8,8);
  end;
end;

procedure TGameScene.Update;
const
  MoveSpeed=0.1; //Direct Move Speed
  G:TVector=(X:0; Y:0.05); //Gravity acceleration
  Kof_Upr=0.01; //Kof elasticity
  Max_UprAcc=0.1; //Maximum elasticity acceleration
var
  Temp:TVector;
  IG:TBullet;
begin
  Inc(Tick);
  {-=Update Engine=-}
  World.Move;
  World.Dead;
  //TE.Move; TE.Dead; BE.Move; BE.Dead; EE.Move; EE.Dead; AE.Move; AE.Dead;
  Collider;
  {-=Update Input=-}
  UpdateMouse; MP:=VectorAdd(CamPos,VectorSub(GetMousePos,MakeVector(wnd.Width/2,wnd.Height/2)));
  if inp.Down(27) then eX.Quit;
  //if inp.Down(M_BTN_2) then P:=MP;
  if not User.Deaded then
  begin
    if inp.Down(M_BTN_2) then
    begin
      GenPoint(P,MP);
      if (PACount>0)and(VectorABDlina(FPA,P)<(User.R+10)) then AddExplosion(FPA,1,nil)
      else
      begin
        //lbShot:        //[Fatal Error] Censored.pas(285): Internal error: T2335
        IG:=OneShot(@Weapons[0],User); //IG.TexID:=10;
        IG.V:=VectorSetDlina(VectorSub(MP,User.P),4);
        IG.P:=VectorAdd(User.P,VectorSetDlina(IG.V,IG.R+User.R+1));
      end;
      PACount:=0; Grab:=False; Inp.Reset; snd.Play(Wav[4],0,0,0);
    end;
    if inp.Down(M_BTN_1) then
    begin
      if not Grab then GenPoint(P,MP) else GenPoint(P,FPA); Grab:=PACount>0;
      if Grab and(Tick mod 50=0) then snd.Play(Wav[2],0,0,0);
    end else Grab:=False;
    V:=User.V;
    {with V do
    begin
      if inp.Down(VK_LEFT) then X:=X-MoveSpeed;
      if inp.Down(VK_RIGHT) then X:=X+MoveSpeed;
      if inp.Down(VK_UP) then Y:=Y-MoveSpeed;
      if inp.Down(VK_DOWN) then Y:=Y+MoveSpeed;
    end; }
    {-=Update User=-}
    P:=VectorAdd(P,V);
    V:=VectorMul(V,0.995);
    V:=VectorAdd(V,G);
    if Grab then
    begin
      Temp:=VectorSub(FPA,P);
      if VectorMagnitude(Temp)-FPAM>0 then
      begin
        Temp:=VectorSetDlina(Temp,(VectorMagnitude(Temp)-FPAM)*Kof_Upr);
        if VectorMagnitude(Temp)>Max_UprAcc then Temp:=VectorSetDlina(Temp,Max_UprAcc);
        V:=VectorAdd(V,Temp);
      end;
    end;
    User.V:=V; User.P:=P; UserEff.Pos:=User.P;
    {-=Find Near Bonus=-}
    if App.Debug then
    begin
      if inp.Down(VK_SPACE) then
      begin
        AllBonus:=0; Inp.Reset;
      end;
      if inp.Down(VK_HOME) then EndOfMap:=True;
    end;
    if EndOfMap then NextMap;
    if AllBonus<=0 then
    begin
      AllBonus:=0;
      if AwardFinish<>nil then
      begin
        if AwardFinish.AngSpeed=0 then
        begin
          AwardFinish.AngSpeed:=3; AwardFinish.OnAction:=EvenFinish; NearBonus:=AwardFinish;
        end;
      end else NextMap;
    end else NearBonus:=FindNearBonus;
  end else
  begin
    GBac.SceneText:='^1You Death!';
    ActivScene:=GBac; snd.Play(Wav[0],0,0,0);
  end;
  CamV:=VectorMul(VectorAdd(CamV,VectorMul(VectorSub(P,CamPos),1/App.UPS)),0.90);
  CamPos:=VectorAdd(CamV,CamPos); //snd.Pos(P.X,P.Y,0);
end;

procedure TGameScene.EventHandler(Sender:TEntity);
var
  Temp:TAward absolute Sender;
begin
  if not(Sender is TAward) then Exit;
  if TAward(Sender).AngSpeed=0 then Exit;
  if VectorABDlina(Temp.P,User.P)<(User.R+Temp.R) then
  begin
    Temp.AngSpeed:=0;
    SetupEffFromBase(TDynEffect.Create(ES),1).Pos:=Temp.P;
    Dec(AllBonus);
    snd.Play(Wav[1],0,0,0);
  end;
end;

procedure TGameScene.EventKill(Sender:TEntity);
var
  Temp:TAward absolute Sender;
begin
  if not(Sender is TAward) then Exit;
  if VectorABDlina(Temp.P,User.P)<(User.R+Temp.R) then
  begin
    Temp.AngSpeed:=0;
    User.Dead;
  end;
end;

procedure TGameScene.EvenFinish(Sender:TEntity);
var
  Temp:TAward absolute Sender;
begin
  if AllBonus>0 then Exit;
  if not(Sender is TAward) then Exit;
  if TAward(Sender).AngSpeed=0 then Exit;
  if VectorABDlina(Temp.P,User.P)<(User.R+Temp.R) then
  begin
    Temp.AngSpeed:=0; if AllBonus<=0 then EndOfMap:=True;
  end;
end;

procedure TGameScene.GenLevel2;
begin
  uFMOD_PlaySong(PChar('chiptune.xm'),0,XM_FILE or XM_SUSPENDED);
  {-=Init Cam=-}
  P:=MakeVector(150,150); CamPos:=P; CamV:=V;
  {-=Gen map=-}
  LE.AddRect(100,400,500,100);
  LE.AddRect(100,100,200,200);
  LE.AddRect(700,500,200,50);
  LE.AddRect(600,50,100,100);
  LE.AddRect(1,700,1000,50);
  LE.AddRect(200,10,350,50);
  LE.AddRect(-200,-100,350,50);
  LE.AddRect(-500,-300,250,550);
  LE.AddRect(+1100,-1500,100,2000);
  LE.AddRect(+250,-300,400,70);
  LE.AddRect(250,-1000,500,70);
  {-=Gen Tower=-}
  TE.AddTower(678,180,50,Weapons[0]);
  TE.AddTower(50,350,100,Weapons[1]).TexID:=12;
  TE.AddTower(500,-750,100,Weapons[2]);
  {-=Gen Award=-}
  AwardFinish:=AE.AddText(1150,-1600,'^-^2Finish');
  AE.AddAward(850,650,EventHandler,1).AngSpeed:=1;
  AE.AddAward(450,630,EventHandler,1).AngSpeed:=-1.5;
  AE.AddAward(450,630,EventHandler,1).AngSpeed:=-1.5;
  AE.AddAward(0,-300,EventHandler,1).AngSpeed:=+1;
  AE.AddAward(1150,-1550,EventHandler,1).AngSpeed:=+1;
  AE.AddAward(-100,350,EventKill,1).TexID:=3;
  AE.AddAward(225,-50,EventKill,1).TexID:=3;
  AE.AddText(0,-500,'-500');
  AE.AddText(-200,-400,'');
  AE.AddText(+200,-400,'');
  AE.AddText(+1000,-1000,'H=-1000');
  AE.AddText(+1250,-500,'H=-500');
end;

procedure TGameScene.GenLevel3;
const
  RND:array[0..9] of Shortint=(+2,-3,-4,+4,-1,+1,+3,-2,+0,-5);
var
  I:Integer;
  Last:TAward;
begin
  {-=Load Music=-}
  uFMOD_PlaySong(PChar('malinka.xm'),0,XM_FILE or XM_SUSPENDED);
  {-=Init Cam=-}
  P:=MakeVector(1000,1000); CamPos:=P; CamV:=V;
  {-=Gen map=-}
  LE.AddRect(900,1300,350,100);
  LE.AddRect(5,2600,3200,70);
  LE.AddRect(-1000,2800,5000,70);
  LE.AddRect(-1070,-2000,60,5500);
  LE.AddRect(4010,-2000,60,5500);
  LE.AddRect(-1000,-2100,5000,70);
  LE.AddRect(1500,1000,100,500);
  LE.AddRect(1400,1700,200,40);
  for I:=0 to 10 do
  begin
    LE.AddRect(-1001+200*I+RND[I mod 10]*2,200,100,200);
    LE.AddRect(2000+RND[I mod 10]*10,-1001+200*I,200,50);
    LE.AddRect(3000+RND[I mod 10]*10,200*I,500,70);
    Last:=AE.AddAward(2100,-1031+200*I,EventHandler,1);
    Last.C:=RGBA(0,128,255,255);
    if RND[I]=0 then Last.AngSpeed:=2 else
      Last.AngSpeed:=Abs(RND[I])/RND[I]+RND[I mod 10]*0.05;
  end;
  {-=Gen Tower=-}
  TE.AddTower(800,1200,50,Weapons[0]);
  TE.AddTower(2430,2240,70,Weapons[3]).TexID:=12;
  {-=Gen Award=-}
  AwardFinish:=AE.AddText(3202,2574,'^-^2Finish');
  for I:=1 to 49 do
  begin
    AE.AddAward(-1000+I*100,2700,EventKill,1).TexID:=3;
  end;
  AE.AddText(1000,2500,'   :)');
  AE.AddText(1000,1500,'H=1500');
  AE.AddText(1000,500,'H=500');
  AE.AddText(1000,-500,'H=-500');
  AE.AddAward(500,0,EventHandler,1).AngSpeed:=-1.5;
  AE.AddAward(1500,0,EventHandler,1).AngSpeed:=-1.5;
  AE.AddText(1000,-1000,'H=-1000');
  AE.AddText(1000,-500,'H=-500');
  AE.AddText(1000,-1000,'H=-1000');
  AE.AddText(1000,-1500,'H=-1500');
  AE.AddAward(1500,0,EventHandler,1).AngSpeed:=-1.5;
  AE.AddAward(1500,0,EventHandler,1).AngSpeed:=-1.5;
  AE.AddText(2000,-1300,'^5   Special for ^xFFAA1FIGDC!^-^2   Create by ^3Adler3D').C:=RGBA(255,255,0,255);
  AE.AddText(1000,-2000,'  :)').C:=RGBA(255,255,0,255);
end;

procedure TGameScene.GenLevel1;
var
  I:Integer;
begin
  uFMOD_PlaySong(PChar('fighting.xm'),0,XM_FILE or XM_SUSPENDED);
  {-=Init Cam=-}
  P:=MakeVector(5000,5000); CamPos:=P; CamV:=V;
  {-=Gen map=-}
  LE.AddRect(4500,5500,1000,-1000);
  LE.AddRect(4550,4700,200,50);
  LE.AddRect(5250,4700,200,50);
  LE.AddRect(4900,P.Y+200,200,50);
  {-=Gen Tower=-}
  TE.AddTower(4600,4600,100,Weapons[0]);
  TE.AddTower(5400,4600,100,Weapons[1]).TexID:=12;
  {-=Gen Award=-}
  AwardFinish:=AE.AddText(5430,5430,'^-^2Finish');
  AE.AddAward(5000,4550,EventHandler,1).AngSpeed:=-1;
  for I:=-4 to 4 do AE.AddAward(5000+I*100,5400,EventHandler,1).AngSpeed:=1;
  AE.AddAward(4550,4800,EventHandler,1).AngSpeed:=1.5;
  AE.AddAward(5250,4800,EventHandler,1).AngSpeed:=1.5;
  if App.Debug then AE.AddAward(4520,4520,EventKill,1).TexID:=3;
  AE.AddText(5000,P.Y+150,'^3Welcome to DooRocket!');
  AE.AddText(4700,P.Y+100,'^7!    .');
  AE.AddText(5100,P.Y+100,'^7!  ');
  AE.AddText(5000,P.Y+300,'^7 .  ^2^7  ^2^7!');
  AE.AddText(5000,5550,'^7 .  ^3 ^7 !');
  AE.AddText(5000,4400,'^7 .  ^3 ^7!');
  AE.AddText(4582,4975,'  ^3   ^5Readme.rtf^3?');
  if App.Debug then AE.AddText(4500,4400,'^3     ^5');
  if App.Debug then AE.AddText(4500,5580,'  ^7" ^2^7   ^31 ^7  ^34321^7    !"');
end;

procedure TGameScene.ScanWorld;
var
  I:Integer;
begin
  AllBonus:=0;
  for I:=0 to AE.Count-1 do
    if AE[I].ClassType=TAward then
      if TAward(AE[I]).AngSpeed<>0 then
      begin
        Inc(AllBonus);
      end;
end;

function TGameScene.FindNearBonus:TAward;
var
  I:Integer;
  S:TAward;
begin
  Result:=nil;
  if AllBonus=0 then Exit;
  for I:=0 to AE.Count-1 do if AE[I].ClassType=TAward then if TAward(AE[I]).AngSpeed<>0 then
      begin
        Result:=TAward(AE[I]); Break;
      end;
  for I:=I+1 to AE.Count-1 do
    if AE[I].ClassType=TAward then
      if TAward(AE[I]).AngSpeed<>0 then
      begin
        S:=TAward(AE[I]);
        if VectorABDlina(User.P,S.P)<VectorABDlina(User.P,Result.P) then Result:=S;
      end;
end;

procedure TGameScene.SetNearBonus(const Value:TAward);
begin
  if FNearBonus=Value then Exit;
  if FNearBonus<>nil then
  begin
    FNearBonus.UnLock; FNearBonus:=nil;
  end;
  FNearBonus:=Value;
  if FNearBonus<>nil then FNearBonus.Lock;
end;

procedure TGameScene.RenderHint;
const
  C_Mag=300;
var
  Q:TVector;
  Ang,{W,H,} Mag:Real;
  //_A,_B:TVector;
begin
  if NearBonus<>nil then
  begin
    //W:=wnd.Width/2; H:=wnd.Height/2;
    //_A:=MakeVector(+CamPos.X,+CamPos.Y);
    //_A:=MakeVector(wnd.Width/2+CamPos.X,wnd.Height/2+CamPos.Y);
    //ColideLine()
    if VectorABDlina(NearBonus.P,P)>C_Mag then Mag:=C_Mag-64 else Mag:=VectorABDlina(NearBonus.P,P)-64;
    Q:=VectorSub(NearBonus.P,P); Ang:=VectorGetAlfa(Q)/pi*180;
    Q:=VectorAdd(P,VectorSetDlina(Q,Mag));
    glColor4d(0,1,0,0.5);
    tex.Enable(Img[4]);
    DrawQuad(Q.X,Q.Y,100,32,Ang);
    tex.Disable;
  end;
end;

procedure TGameScene.NextMap;
begin
  if High(Maps)<FCurMapID then
  begin
    GBac.SceneText:='^2You Win!';
    ActivScene:=GBac; Snd.Play(Wav[7],0,0,0); Exit;
  end else snd.Play(Wav[3],0,0,0);
  if AwardFinish<>nil then AwardFinish.UnLock; AwardFinish:=nil;
  World.Clear;
  {-=Init User=-}
  User:=TUserTower.Create(TE);
  User.SetUPA(1000000);
  User.SetR(8);
  User.C:=clLime; User.Lock;
  User.TexID:=1;
  UserEff:=TRectEffect.Create(ES); SetupEffFromBase(UserEff,0);
  {-=Init Weapon=-}
  Weapons[0]:=EquipWeapon(User,OneShot,7);
  Weapons[1]:=EquipWeapon(User,TwoShot,7);
  Weapons[2]:=EquipWeapon(User,DoubleShot,8);
  Weapons[3]:=EquipWeapon(User,PusShot,11);
  {-=Gen Map=-}
  Maps[FCurMapID]();
  if AwardFinish<>nil then
  begin
    AwardFinish.Lock; AwardFinish.C:=RGBA(255,255,0,255); AwardFinish.AngSpeed:=0; AwardFinish.SetUPA(2);
  end; EndOfMap:=False;
  AE.AddText(P.X,P.Y,'^2   Start here!').C:=RGBA(255,255,255,255);
  ScanWorld; uFMOD_Resume; Inc(FCurMapID); inp.Reset;
end;

function TGameScene.SetupEffFromBase(Eff:TDynEffect; ID:Byte):TDynEffect;
begin
  //Eff:=TRectEffect.Create(ES);
  if ID=0 then
  begin
    if Eff is TRectEffect then
    begin
      TRectEffect(Eff).PosXVib:=4;
      TRectEffect(Eff).PosYVib:=4;
    end;
    Eff.Texture:=Img[14];
    {if App.Debug then Eff.EPS:=10 else }Eff.EPS:=100;
    Eff.PLT:=2.5;
    Eff.Col:=RGB(255,255,255);
    Eff.Dir:=PId2; Eff.DirVib:=PI/30;
    Eff.Mag:=0.5; Eff.MagVib:=1;
    Eff.BegR:=4; Eff.EndR:=1;
    Eff.Grav:=MakeVector(0,0.001);
  end;
  if ID=1 then
  begin
    if Eff is TRectEffect then
    begin
      TRectEffect(Eff).PosXVib:=4;
      TRectEffect(Eff).PosYVib:=4;
    end;
    Eff.Texture:=Img[13];
    Eff.EPS:=100;
    Eff.SLT:=2;
    Eff.PLT:=2.5;
    Eff.Loop:=False;
    Eff.Col:=RGB(255,255,255);
    Eff.Dir:=PId2; Eff.DirVib:=PI*2;
    Eff.Mag:=0.5; Eff.MagVib:=1;
    Eff.BegR:=1; Eff.EndR:=4;
    Eff.Grav:=MakeVector(0,0.002);
  end;
  if ID=2 then
  begin
    if Eff is TRectEffect then
    begin
      TRectEffect(Eff).PosXVib:=4;
      TRectEffect(Eff).PosYVib:=4;
    end;
    Eff.Texture:=Img[13];
    Eff.EPS:=100;
    Eff.SLT:=0.5;
    Eff.PLT:=0.7;
    Eff.Loop:=False;
    Eff.Col:=RGB(255,255,255);
    Eff.Dir:=PId2; Eff.DirVib:=PI*2;
    Eff.Mag:=0.5; Eff.MagVib:=1;
    Eff.BegR:=5; Eff.EndR:=1;
    Eff.Grav:=MakeVector(0,0.002);
  end;
  if ID=3 then
  begin
    if Eff is TRectEffect then
    begin
      TRectEffect(Eff).PosXVib:=4;
      TRectEffect(Eff).PosYVib:=4;
    end;
    Eff.Texture:=Img[14];
    Eff.EPS:=100;
    Eff.SLT:=0.5;
    Eff.PLT:=0.5;
    Eff.Loop:=False;
    Eff.Col:=RGB(255,255,255);
    Eff.Dir:=PId2; Eff.DirVib:=PI*2;
    Eff.Mag:=0.5; Eff.MagVib:=0.2;
    Eff.BegR:=2; Eff.EndR:=4;
    Eff.Grav:=MakeVector(0,0.001);
  end;
  if ID=4 then
  begin
    if Eff is TRectEffect then
    begin
      TRectEffect(Eff).PosXVib:=4;
      TRectEffect(Eff).PosYVib:=4;
    end;
    Eff.Texture:=Img[14];
    Eff.EPS:=100;
    Eff.SLT:=0.5;
    Eff.PLT:=0.4;
    Eff.Loop:=False;
    Eff.Col:=RGB(64,255,64);
    Eff.Dir:=PId2; Eff.DirVib:=PI*2;
    Eff.Mag:=1; Eff.MagVib:=0.5;
    Eff.BegR:=5; Eff.EndR:=1;
    Eff.Grav:=MakeVector(0,-0.01);
  end;
  Result:=Eff;
end;

procedure TGameScene.AddExplosion(P: TVector; Force: Real; B:TBullet);
var
  ID:Byte;
begin
  EE.AddExplosion(P,Force); ID:=2;
  if B=nil then
  begin
    SetupEffFromBase(TDynEffect.Create(ES),2).Pos:=P; Exit;
  end;
  if B.ClassType=TBullet then ID:=2;
  if B.ClassType=THomingBullet then ID:=3;
  if B.ClassType=TPusBullet then ID:=4;
  SetupEffFromBase(TDynEffect.Create(ES),ID).Pos:=P;
end;

procedure TGameScene.NewGame;
begin
  FCurMapID:=1; NextMap; ActivScene:=Self;
end;

procedure TGameScene.ReloadMap;
begin
  Dec(FCurMapID); NextMap; ActivScene:=Self;
end;

procedure TGameScene.LoadNextMap;
begin
  if High(Maps)<FCurMapID then FCurMapID:=3;
  NextMap; ActivScene:=Self;
end;

end.

