{
           .
}
unit DGuiBuilder;

interface

uses
  DMath,

  DGuiCore,
  DGuiCommon;

type
// () ====
TGuiImage = Integer;
TOnClosed = Integer;
TOnEnter = Integer;
// ==============

TGuiBuilder = class
private
  FGui: TGui;
  FStyle: String;
  FCurrent: TWidget;
  FOwner: TWidget;
public
  constructor Create(Gui: TGui);
  procedure SetStyle(Name: String);
  procedure SetOwner(Owner: TWidget);
  function _Label(Pos, Size: TVec2f; Text: String): TGuiLabel; overload;
  function Caption(Pos, Size: TVec2f; Text: String): TGuiCaption; overload;
  function Button(Pos, Size: TVec2f;
                  Text: String;  
                  OnClick: TOnClick): TGuiButton; overload;
  function Edit(Pos, Size: TVec2f): TGuiEdit;
  function Button(Rect: TRect;
                  Image: TGuiImage; 
                  OnClick: TOnClick): TWidget; overload;
  function CheckBox(Rect: TRect; Text: String; State: Boolean = False): TWidget;
  // ,    StartWindow  EndWindow -   
  function StartWindow(Pos, Size: TVec2f; Name, Caption: String; OnClosed: TProcOfObj): TGuiWindow;
  procedure EndWindow;
  property Gui: TGui read FGui;
  property Style: String read FStyle;
  property Owner: TWidget read FOwner;
end;

implementation

constructor TGuiBuilder.Create(Gui: TGui);
begin
  FGui := Gui;
  FOwner := Gui.Root;
end;

procedure TGuiBuilder.SetStyle(Name: String);
begin
  FStyle := Name;
end;

procedure TGuiBuilder.SetOwner(Owner: TWidget);
begin
  FOwner := Owner;
end;

function TGuiBuilder._Label(Pos, Size: TVec2f; Text: String): TGuiLabel;
begin
  Result := TGuiLabel.Create(FGui, Text, FStyle);
  FGui.Link(FOwner, Result);
  Result.Local := Pos;
  Result.Size := Size;
  Result._Label.Font := 'default';
  Result._Label.Text := Text;
end;

function TGuiBuilder.Caption(Pos, Size: TVec2f; Text: String): TGuiCaption; 
begin
  Result := TGuiCaption.Create(FGui, Text, FStyle);
  Result.Caption := Text;
  Result.LocalRect := Rect(Pos, Pos + Size);
  FGui.Link(FOwner, Result);
end;

function TGuiBuilder.Button(Pos, Size: TVec2f; 
                            Text: String; 
                            OnClick: TOnClick): TGuiButton;
begin
  Result := TGuiButton.Create(FGui, Text, FStyle);
  Result.Caption := Text;
  Result.LocalRect := Rect(Pos, Pos + Size);
  FGui.Link(FOwner, Result);
end;

function TGuiBuilder.Edit(Pos, Size: TVec2f): TGuiEdit;
begin
  Result := TGuiEdit.Create(FGui, 'edit', FStyle);
  Result.Local := Pos;
  Result.Size := Size;
  Result.Parent := FOwner;
end;

function TGuiBuilder.Button(Rect: TRect;
                            Image: TGuiImage; 
                            OnClick: TOnClick): TWidget;
begin
  Result := nil
end;

function TGuiBuilder.CheckBox;
begin
  Result := nil;
end;

function TGuiBuilder.StartWindow;
begin
  Result := TGuiWindow.Create(FGui, Name, FStyle);
  Result.Local := Pos;
  Result.Size := Size;
  Result.Caption := Caption;
  Result.Parent := FOwner;
  FOwner := Result;
end;

procedure TGuiBuilder.EndWindow;
begin
  FOwner := FOwner.Parent;
end;

end.
