{
         -   TGui  TWidget.

         TGui
          *    - , winapi-  ..
          *      ... (!)
}
unit DGuiCore;

interface

uses
  DDebug,
  DClasses,
  DMath,
  DIntersection,
  DEvents,
  DUtils,
  DWinApi,
  DGraph,
  DFont;

type
TGuiHAlign = (haCenter, haLeft, haRight);
TGuiVAlign = (vaCenter, vaUp,   vaDown);
TGuiAlign = record
  H: TGuiHAlign;
  V: TGuiVAlign;
end;

TGuiStyle = class;
TGui = class;
TWidget = class;

// 
P = Integer;

TGuiResource = class
private
  FStyle: TGuiStyle;
  FName: String;
public
//  constructor Create(Style: TGuiStyle; Name: String);
  property Name: String read FName;
end;

TGuiFont = class
private
  FStyle: TGuiStyle;
  FName: String;
  FFont: TFont;
  procedure UpdateFont;
  procedure SetStyle(NewStyle: TGuiStyle);
  procedure SetName(NewName: String);
public
  constructor Create(Style: TGuiStyle; Name: String = 'default');
  procedure Draw(Pos: TVec2f; Text: String); overload;
  procedure Draw(Pos, Align: TVec2f; Text: String); overload;
  procedure Draw(Rect: TRect; Align: TGuiAlign; Text: String); overload;
  function GetSize: TVec2i; overload; inline;
  function GetSpacing: Integer;
  function GetWidth(Text: String): Integer;
  function GetHeight(Text: String): Integer;
  function GetSize(Text: String): TVec2i; overload; inline;
  property Style: TGuiStyle read FStyle write SetStyle;
  property Name: String read FName write SetName;
end;
//TListOfGuiFont = {$IFDEF FLAG_FPC}specialize{$ENDIF} TList<TGuiFont>;

TGuiImage = Integer;

TListOfWidget = {$IFDEF FLAG_FPC}specialize{$ENDIF} TList<TWidget>;

TProcOfObj = procedure of object;
TOnClick = procedure (const Button: TMouseButton; const Pos: TVec2f) of object;
TOnRender = procedure of object;

TGuiFontItem = record 
  Name: String;
  Font: TFont;
end;
TListOfFontItem =  {$IFDEF FLAG_FPC}specialize{$ENDIF} TList<TGuiFontItem>;

TGuiStyle = class
private
  FGui: TGui;
  FName: String;
  FPrimaryNormal: TVec4f;
  FPrimaryDisabled: TVec4f;
  FPrimaryActive: TVec4f;
  FSecondaryNormal: TVec4f;
  FSecondaryDisabled: TVec4f;
  FSecondaryActive: TVec4f;
  FMemoNormal: TVec4f;
  FMemoDisabled: TVec4f;
  FMemoActive: TVec4f;
  FFonts: TListOfFontItem;
public
  constructor Create(Gui: TGui; Name: String);
  destructor Destroy; override;
  procedure Color4f(Color: TVec4f); virtual;
  procedure DrawRect(Rect: TRect; Fill: Boolean = True); virtual;
  procedure DrawCircle(Center: TVec2f; Radius: Single; Fill: Boolean = True); virtual;
  procedure SetLineWidth(Width: Single); virtual;
  procedure DrawLine(A, B: TVec2f); virtual;
  procedure DrawImage(Rect: TRect; Image: TGuiImage); virtual;
  procedure DrawText(Pos, Align: TVec2f; Font: TGuiFont; Text: string); virtual;
  procedure AddFont(Name, FileName: String; Spacing: Integer = 0);
  function FindFont(Name: String): TFont;
  property Gui: TGui read FGui;
  property Name: String read FName;
  property PrimaryColor: TVec4f read FPrimaryNormal write FPrimaryNormal;
  property SecondaryColor: TVec4f read FSecondaryNormal write FSecondaryNormal;
  property ActiveColor: TVec4f read FPrimaryActive write FPrimaryActive;
  property BackColor: TVec4f read FMemoNormal write FMemoNormal;
  property DisabledColor: TVec4f read FPrimaryDisabled write FPrimaryDisabled;


  property PrimaryNormal: TVec4f read FPrimaryNormal write FPrimaryDisabled;
  property PrimaryDisabled: TVec4f read FPrimaryDisabled write FPrimaryDisabled;
  property PrimaryActive: TVec4f read FPrimaryActive write FPrimaryActive;
  property SecondaryNormal: TVec4f read FSecondaryNormal write FSecondaryNormal;
  property SecondaryDisabled: TVec4f read FSecondaryDisabled write FSecondaryDisabled;
  property SecondaryActive: TVec4f read FSecondaryActive write FSecondaryActive;
  property MemoNormal: TVec4f read FMemoNormal write FMemoNormal;
  property MemoDisabled: TVec4f read FMemoDisabled write FMemoDisabled;
  property MemoActive: TVec4f read FMemoActive write FMemoActive;
end;
TListOfGuiStyle = {$IFDEF FLAG_FPC}specialize{$ENDIF} TList<TGuiStyle>;

TGui = class(TEntity)
private
  FGraph: TGraph;
  FWindow: TWindow;
  FFontMan: TFontMan;
  FRoot: TWidget;
  FFocus: TWidget;
  FStyles: TListOfGuiStyle;
  function GetMouse: TVec2f;
  function RecOnMouseDown(Widget: TWidget; Button: TMouseButton;
                          Pos: TVec2f; Flags: Cardinal): Boolean; overload;
  function RecOnMouseDown(Widget: TWidget; Ev: TWinApiMouse): Boolean; overload;
  function RecOnKeyDown(Widget: TWidget; Ev: TWinApiKey): Boolean;
  function RecOnChar(Widget: TWidget; Ev: TWinApiChar): Boolean;
  procedure RecEvent(Widget: TWidget; Ev: TEvent);
public
  constructor Create(Graph: TGraph; Window: TWindow);
  destructor Destroy; override;
  procedure Render; override;
  procedure Update; override;
  procedure OnEvent(Event: TEvent); override;
  procedure OnMouseDown(Button: TMouseButton; Pos: TVec2f; Flags: Cardinal);
  procedure AddStyle(Style: TGuiStyle);
  function FindStyle(Name: String): TGuiStyle;
  procedure Link(Parent, Widget: TWidget);
  procedure Unlink(Widget: TWidget);
  procedure Popup(Widget: TWidget);
  function Trace(Point: TVec2f): TWidget; //     
  procedure SetFocus(Widget: TWidget);
  property Graph: TGraph read FGraph;
  property Window: TWindow read FWindow;
  property FontMan: TFontMan read FFontMan write FFontMan;
  property Root: TWidget read FRoot;
  property Focus: TWidget read FFocus write SetFocus;
  property Styles[Index: String]: TGuiStyle read FindStyle;
  property Mouse: TVec2f read GetMouse;
end;

TWidget = class(TEntity)
private
  FGui: TGui;
  FName: String;
  FStyle: TGuiStyle;
  FParent: TWidget;
  FSub: TListOfWidget;
  FVisible: Boolean;
  FLocal: TVec2f;
  FSize: TVec2f;
  FFocusable: Boolean;
  FAlwaysOnTop: Boolean;
  function GetGlobal: TVec2f;
  procedure SetGlobal(NewPos: TVec2f);
  function GetLocalCenter: TVec2f;
  procedure SetLocalCenter(NewCenter: TVec2f);
  function GetGlobalCenter: TVec2f;
  procedure SetGlobalCenter(NewCenter: TVec2f);
  function GetLocalRect: TRect;
  procedure SetLocalRect(NewRect: TRect);
  function GetGlobalRect: TRect;
  procedure SetGlobalRect(NewRect: TRect);
public
  constructor Create(Gui: TGui; Name, Style: String);
  destructor Destroy; override;
  procedure SetParent(Parent: TWidget);
  function IsLocalPointIn(Point: TVec2f): Boolean; inline;
  function IsGlobalPointIn(Point: TVec2f): Boolean; inline;
  function Trace(Point: TVec2f): TWidget;
  function IsMouseIn: Boolean;
  function OnMouseDown(Button: TMouseButton; Pos: TVec2f; 
                       Flags: Cardinal): Boolean; overload; virtual;
  function OnMouseDown(Ev: TWinApiMouse): Boolean; overload; virtual;
  function OnKeyDown(Ev: TWinApiKey): Boolean; virtual;
  function OnChar(Ev: TWinApiChar): Boolean; virtual;
  procedure OnRender;
  procedure OnUpdate;
  procedure OnLostFocus; virtual;
  procedure SetFocus;
  procedure LostFocus;
  function IsInFocus: Boolean;
  procedure Render; override;
  procedure Update; override;
  procedure Show;
  procedure Hide;
  property Gui: TGui read FGui;
  property Name: String read FName;
  property Style: TGuiStyle read FStyle;
  property Parent: TWidget read FParent write SetParent;
  property Sub: TListOfWidget read FSub;
  property Visible: Boolean read FVisible write FVisible;
  property Local: TVec2f read FLocal write FLocal; 
  property Global: TVec2f read GetGlobal write SetGlobal;
  property LocalCenter: TVec2f read GetLocalCenter write SetLocalCenter;
  property GlobalCenter: TVec2f read GetGlobalCenter write SetGlobalCenter;
  property Size: TVec2f read FSize write FSize;
  property LocalRect: TRect read GetLocalRect write SetLocalRect;
  property GlobalRect: TRect read GetGlobalRect write SetGlobalRect;
  property Focusable: Boolean read FFocusable;
  property AlwaysOnTop: Boolean read FAlwaysOnTop write FAlwaysOnTop;
end;

TWidgetDecorator = class
private
  FWidget: TWidget;
  function GetGui: TGui;
  function GetStyle: TGuiStyle;
public
  constructor Create(Widget: TWidget);
  property Widget: TWidget read FWidget;
  property Gui: TGui read GetGui;
  property Style: TGuiStyle read GetStyle;
end;

// utils
function Align(H: TGuiHAlign; V: TGuiVAlign): TGuiAlign;

implementation

function Align(H: TGuiHAlign; V: TGuiVAlign): TGuiAlign;
begin
  Result.H := H;
  Result.V := V;
end;

{$IFNDEF FLAG_FPC}{$REGION 'TGuiFont'}{$ENDIF}
procedure TGuiFont.UpdateFont;
begin
  if FStyle <> nil then
    FFont := FStyle.FindFont(FName)
  else
    FFont := nil;
end;

procedure TGuiFont.SetStyle(NewStyle: TGuiStyle);
begin
  FStyle := Style;
  UpdateFont;
end;

procedure TGuiFont.SetName(NewName: String);
begin
  FName := NewName;
  UpdateFont;
end;

constructor TGuiFont.Create(Style: TGuiStyle; Name: String = 'default');
begin
  FStyle := Style;
  FName := Name;
  UpdateFont;
end;

procedure TGuiFont.Draw(Pos: TVec2f; Text: String);
begin
  if FFont <> nil then begin
    Style.Gui.Graph.SetBlendMode(bmAlpha);
    FFont.Draw(Pos, Text);
  end;
end;

procedure TGuiFont.Draw(Pos, Align: TVec2f; Text: String);
  var
    OutPos: TVec2f;
begin
  if FFont = nil then
    Exit;
  OutPos := Pos - Align * GetSize(Text);
  FStyle.Gui.Graph.SetBlendMode(bmAlpha);
  FFont.Draw(OutPos, Text);
end;

procedure TGuiFont.Draw(Rect: TRect; Align: TGuiAlign; Text: String);
  var
    Pos: TVec2f;
begin
  case Align.H of
    haLeft: Pos.X := Rect.A.X;
    haCenter: Pos.X := Rect.A.X + (Rect.B.X - Rect.A.X - GetWidth(Text))/2;
    haRight: Pos.X := Rect.B.X - GetWidth(Text);
  end;
  case Align.V of
    vaUp: Pos.Y := Rect.A.Y;
    vaCenter: Pos.Y := Rect.A.Y + (Rect.B.Y - Rect.A.Y - GetHeight(Text))/2;
    vaDown: Pos.Y := Rect.B.Y - GetHeight(Text);
  end;
  Draw(Pos, Text);
end;

function TGuiFont.GetSize: TVec2i; 
begin
  if FFont <> nil then
    Result := Vec2i(FFont.Width, FFont.Height)
  else
    Result := Vec2i(0, 0);
end;

function TGuiFont.GetSpacing: Integer;
begin
end;

function TGuiFont.GetWidth(Text: String): Integer;
begin
  if FFont <> nil then
    Result := FFont.GetWidth(Text)
  else
    Result := 0;
end;

function TGuiFont.GetHeight(Text: String): Integer;
begin
  if FFont <> nil then
    Result := FFont.GetHeight(Text)
  else
    Result := 0;
end;

function TGuiFont.GetSize(Text: String): TVec2i;
begin
  Result := Vec2i(GetWidth(Text), GetHeight(Text));
end;

{$IFNDEF FLAG_FPC}{$ENDREGION}{$ENDIF}

{$IFNDEF FLAG_FPC}{$REGION 'TGuiStyle'}{$ENDIF}
constructor TGuiStyle.Create(Gui: TGui; Name: String);
begin
  FGui := Gui;
  FName := Name;
  FFonts := TListOfFontItem.Create;
end;

destructor TGuiStyle.Destroy;
begin
  FFonts.Free;
end;

procedure TGuiStyle.Color4f(Color: TVec4f);
begin
  Gui.Graph.Color4f(Color);
end;

procedure TGuiStyle.DrawRect; 
begin
  if Fill then
    Gui.Graph.DrawRect(Rect)
  else
    Gui.Graph.DrawRect(Rect, False);
end;

procedure TGuiStyle.DrawCircle(Center: TVec2f; Radius: Single; Fill: Boolean = True);
begin
  Gui.Graph.DrawCircle(Center, Radius, Fill);
end;

procedure TGuiStyle.SetLineWidth(Width: Single);
begin
  Gui.Graph.SetLineWidth(Width);
end;

procedure TGuiStyle.DrawLine(A, B: TVec2f);
begin
  Gui.Graph.DrawLine(A, B);
end;

procedure TGuiStyle.DrawImage(Rect: TRect; Image: TGuiImage);
begin
end;

procedure TGuiStyle.DrawText(Pos, Align: TVec2f; Font: TGuiFont; Text: string);
  var
    OutPos: TVec2f;
begin
  if Font = nil then
    Exit;
  OutPos := Pos - Align * Vec2f(Font.GetWidth(Text), Font.GetHeight(Text));
  Gui.Graph.SetBlendMode(bmAlpha);
  Font.Draw(OutPos, Text);
end;

procedure TGuiStyle.AddFont(Name, FileName: String; Spacing: Integer = 0);
  var
    I: Integer;
    Item: TGuiFontItem;
begin
  for I := 0 to FFonts.Last do
    if FFonts[I].Name = Name then begin
      //FFonts[I].Free;
      // ...;
      Exit;
    end;
  Item.Name := Name;
  Item.Font := Gui.FontMan.CreateFont(FileName, Spacing);
  FFonts.Add(Item);
end;

function TGuiStyle.FindFont(Name: String): TFont;
  var
    I: Integer;
begin
  Result := nil;
  for I := 0 to FFonts.Last do
    if FFonts[I].Name = Name then begin
      Result := FFonts[I].Font;
      Exit;
    end;
end;
{$IFNDEF FLAG_FPC}{$ENDREGION}{$ENDIF}

{$IFNDEF FLAG_FPC}{$REGION 'TGui'}{$ENDIF}
function TGui.GetMouse: TVec2f;
begin
  Result := FWindow.Mouse - FRoot.Global;
end;

function TGui.RecOnMouseDown(Widget: TWidget; Button: TMouseButton;
                        Pos: TVec2f; Flags: Cardinal): Boolean;
  var
    I: Integer;
begin
  Result := True;
  for I := Widget.Sub.Last downto 0 do
    if RecOnMouseDown(Widget.Sub[I], Button, Pos, Flags) then
      Exit;
  if not Widget.OnMouseDown(Button, Pos, Flags) then
    Result := False;
end;

function TGui.RecOnMouseDown(Widget: TWidget; Ev: TWinApiMouse): Boolean;
  var
    I: Integer;
begin
  Result := True;
  for I := Widget.Sub.Last downto 0 do
    if RecOnMouseDown(Widget.Sub[I], Ev) then
      Exit;
  if not Widget.OnMouseDown(Ev) then
    Result := False;
end;

function TGui.RecOnKeyDown(Widget: TWidget; Ev: TWinApiKey): Boolean;
  var
    I: Integer;
begin
  Result := True;
  for I := Widget.Sub.Last downto 0 do
    if RecOnKeyDown(Widget.Sub[I], Ev) then
      Exit;
  if not Widget.OnKeyDown(Ev) then
    Result := False;
end;

function TGui.RecOnChar(Widget: TWidget; Ev: TWinApiChar): Boolean;
  var
    I: Integer;
begin
  Result := True;
  for I := Widget.Sub.Last downto 0 do
    if RecOnChar(Widget.Sub[I], Ev) then
      Exit;
  if not Widget.OnChar(Ev) then
    Result := False;
end;

procedure TGui.RecEvent(Widget: TWidget; Ev: TEvent);
  var
    I: Integer;
begin
  for I := Widget.Sub.Last downto 0 do
    RecEvent(Widget.Sub[I], Ev);
  Widget.OnEvent(Ev);
end;

constructor TGui.Create;
  var
    Style: TGuiStyle;
begin
  FGraph := Graph;
  FWindow := Window;
  FFontMan := TFontMan.Create(FGraph);
  FStyles := TListOfGuiStyle.Create;
  Style := TGuiStyle.Create(Self, '__default__');
  with Style do begin
    PrimaryColor := Vec4f(1, 1, 1, 1);
    SecondaryColor := Vec4f(1.0, 0.5, 0.5, 1.0);
    BackColor := Vec4f(0, 0, 0, 1);
    DisabledColor := Vec4f(0.5, 0.5, 0.5, 1.0);
  end;
  AddStyle(Style);
  FRoot := TWidget.Create(Self, '__root__', '__default__');
  FRoot.LocalRect := Rect(Vec2f(0, 0), FGraph.ScreenSize);
end;

destructor TGui.Destroy;
  var
    I: Integer;
begin
  for I := 0 to FStyles.Last do
    FStyles[I].Free;
  FStyles.Free;
  FFontMan.Free;
  FRoot.Free;
end;

procedure TGui.Render;
begin
  Graph.SetBlendMode(bmNo);
  FRoot.OnRender;
end;

procedure TGui.Update;
begin
  FRoot.OnUpdate;
end;

procedure TGui.OnEvent(Event: TEvent);
begin
  if Event is TWinApiMouse then begin
    with TWinApiMouse(Event) do
      if Down then begin
        OnMouseDown(Button, Pos, Flags);
        RecOnMouseDown(FRoot, TWinApiMouse(Event));
      end;
  end else if Event is TWinApiKey then begin
    if TWinApiKey(Event).Down then
      RecOnKeyDown(FRoot, TWinApiKey(Event));
  end else if Event is TWinApiChar then begin
    RecOnChar(FRoot, TWinApiChar(Event));
  end else
    RecEvent(FRoot, Event);
end;

procedure TGui.OnMouseDown(Button: TMouseButton; Pos: TVec2f; Flags: Cardinal);
begin
  RecOnMouseDown(FRoot, Button, Pos, Flags);
end;

procedure TGui.AddStyle(Style: TGuiStyle);
begin
  FStyles.Add(Style);
end;

function TGui.FindStyle(Name: String): TGuiStyle;
  var
    I: Integer;
begin
  Result := FStyles[0];
  for I := 0 to FStyles.Last do
    if FStyles[I].Name = Name then begin
      Result := FStyles[I];
      Exit;
    end;
end;

procedure TGui.Link(Parent, Widget: TWidget);
begin
  Unlink(Widget);
  if Parent = nil then
    Exit;
  Parent.Sub.Add(Widget);
  Widget.FParent := Parent;
end;

procedure TGui.Unlink(Widget: TWidget);
begin
  if Widget.Parent <> nil then
    Widget.Parent.Sub.Del(Widget, True);
  Widget.FParent := nil;
end;

procedure TGui.Popup(Widget: TWidget);
  var
    Parent: TWidget;
    I: Integer;
begin
  Parent := Widget.Parent;
  if Widget.Parent <> nil then begin
    Parent.Sub.Del(Widget, True);
    Parent.Sub.Add(Widget);
  end;
end;

function TGui.Trace(Point: TVec2f): TWidget;
begin
  Result := FRoot.Trace(Point);
end;

procedure TGui.SetFocus(Widget: TWidget);
begin
  if FFocus <> nil then
    FFocus.OnLostFocus;
  FFocus := Widget;
end;
{$IFNDEF FLAG_FPC}{$ENDREGION}{$ENDIF}

{$IFNDEF FLAG_FPC}{$REGION 'TWidget'}{$ENDIF}
function TWidget.GetGlobal: TVec2f;
begin
  if FParent = nil then
    Result := FLocal
  else
    Result := FParent.Global + FLocal;
end;

procedure TWidget.SetGlobal(NewPos: TVec2f);
begin
  if FParent = nil then
    FLocal := NewPos
  else
    FLocal := NewPos - FParent.Global;
end;

function TWidget.GetLocalCenter: TVec2f;
begin
  Result := (LocalRect.A + LocalRect.B)/2;
end;

procedure TWidget.SetLocalCenter(NewCenter: TVec2f);
begin
  Local := NewCenter - Size/2;
end;

function TWidget.GetGlobalCenter: TVec2f;
begin
  Result := (GlobalRect.A + GlobalRect.B)/2;
end;

procedure TWidget.SetGlobalCenter(NewCenter: TVec2f);
begin
  Global := NewCenter - Size/2;
end;

function TWidget.GetLocalRect: TRect;
begin
  Result := DMath.Rect(FLocal, FLocal + FSize);
end;

procedure TWidget.SetLocalRect(NewRect: TRect);
begin
  Local := NewRect.A;
  Size := NewRect.B - NewRect.A;
end;

function TWidget.GetGlobalRect: TRect;
begin
  if FParent = nil then
    Result := LocalRect
  else begin
    Result.A := Global;
    Result.B := Result.A + Size;
  end;
end;

procedure TWidget.SetGlobalRect(NewRect: TRect);
begin
  Global := NewRect.A;
  Size := NewRect.B - NewRect.A;
end;

constructor TWidget.Create;
begin
  FGui := Gui;
  FName := Name;
  FStyle := FGui.Styles[Style];
  FFocusable := True;
  FParent := nil;
  FSub := TListOfWidget.Create;
  FVisible := True;
  FAlwaysOnTop := False;
end;

destructor TWidget.Destroy;
begin
end;

procedure TWidget.SetParent(Parent: TWidget);
begin
  FGui.Link(Parent, Self);
end;

function TWidget.IsLocalPointIn(Point: TVec2f): Boolean;
begin
  Result := Intersect(LocalRect, Point);
end;

function TWidget.IsGlobalPointIn(Point: TVec2f): Boolean;
begin
  Result := Intersect(GlobalRect, Point);
end;

function TWidget.Trace(Point: TVec2f): TWidget;
  var
    I: Integer;
begin
  for I := FSub.Last downto 0 do begin
    Result := FSub[I].Trace(Point);
    if Result <> nil then
      Exit;
  end;
  if IsGlobalPointIn(Point) then
    Result := Self
  else
    Result := nil;
end;

function TWidget.IsMouseIn: Boolean;
begin
  Result := True;
  if FParent <> nil then
    Result := FParent.IsMouseIn;
  Result := Result and IsGlobalPointIn(Gui.Mouse);
end;

function TWidget.OnMouseDown(Button: TMouseButton; Pos: TVec2f; 
                             Flags: Cardinal): Boolean;
begin
  Result := False;
end;

function TWidget.OnMouseDown(Ev: TWinApiMouse): Boolean;
begin
  Result := False;
end;

function TWidget.OnKeyDown(Ev: TWinApiKey): Boolean;
begin
  Result := False;
end;

function TWidget.OnChar(Ev: TWinApiChar): Boolean;
begin
  Result := False;
end;

procedure TWidget.OnRender;
  var
    I: Integer;
begin
  Render;
  for I := 0 to FSub.Last do
    if FSub[I].Visible then 
      FSub[I].OnRender;
end;

procedure TWidget.OnUpdate;
  var
    I: Integer;
begin
  Update;
  for I := 0 to FSub.Last do
    if FSub[I].Visible then
      FSub[I].OnUpdate;
end;

procedure TWidget.OnLostFocus;
begin
end;

function TWidget.IsInFocus: Boolean;
begin
  Result := Self = Gui.Focus;
end;

procedure TWidget.SetFocus;
begin
  if FFocusable then
    Gui.SetFocus(Self)
  else
    Gui.SetFocus(nil);
end;

procedure TWidget.LostFocus;
begin
  Gui.SetFocus(nil);
end;

procedure TWidget.Render;
begin
end;

procedure TWidget.Update;
begin
end;

procedure TWidget.Show;
begin
  FVisible := True;
end;

procedure TWidget.Hide;
begin
  FVisible := False;
end;
{$IFNDEF FLAG_FPC}{$ENDREGION}{$ENDIF}

function TWidgetDecorator.GetGui: TGui;
begin
  Result := Widget.Gui;
end;

function TWidgetDecorator.GetStyle: TGuiStyle;
begin
  Result := Widget.Style;
end;

constructor TWidgetDecorator.Create(Widget: TWidget);
begin
  FWidget := Widget;
end;

end.
