unit DCore;

interface

uses
  //GL,
  windows,

  DUtils,
  DDebug,
  DMath,
  DEvents,
  DWinApi,
  DSound,
  DGraph,
  DFont;


const
  GWidth = 800;
  GHeight = 600;

  GName = ' II: ';

type
TCore = class(TWinApp)//{{{
private
  FWinApi: TWinApi;
  FWindow: TWindow;
  FSound: TSound;
  FGraph: TGraph;
  FFontMan: TFontMan;
  FEvents: array of TEvent;

  FDragging: Boolean;
  FDragStart: TVec2f;
  FDragDelta: TVec2f;
  FOnDrawDrag: TProcOfObj;
  FKeys: array[0..255] of Boolean;
  FMouseIDs: array[TMouseButton] of Boolean;
  procedure RenderFlags;
  procedure DrawDrag;
public
  constructor Create;
  destructor Destroy; override;
  procedure Run;
  //  
  procedure StartDrag;
  procedure StopDrag;
  //
  procedure Render; override;
  procedure Update; override;
  procedure OnEvent(const Event: TEvent); override;
  function Key(ID: Byte): Boolean;
  function Mouse(ID: TMouseButton): Boolean;
  property WinApi: TWinApi read FWinApi;
  property Window: TWindow read FWindow;
  property Sound: TSound read FSound;
  property FontMan: TFontMan read FFontMan;
  property Graph: TGraph read FGraph;
  property DragDelta: TVec2f read FDragDelta;
  property Dragging: boolean read FDragging;
  property OnDrawDrag: TProcOfObj read FOnDrawDrag;
end;
//}}}

var
  Core: TCore;

implementation

//  TCore//{{{ 
//
procedure TCore.DrawDrag;//{{{
begin
  FGraph.SetColor(0.75, 0.75, 0.75, 1.0);
  FGraph.DrawRect(FDragStart, FDragDelta, False);
end;

//}}}
procedure TCore.RenderFlags;//{{{
  var
    I: Integer;
begin
end;

//}}}
constructor TCore.Create;//{{{
  var
    I: Integer;
begin
  FWinApi := TWinApi.Create;
  FWindow := FWinApi.CreateWindow(GName, GWidth, GHeight);
  FWindow.Subscribe(Self);
  FSound := TSound.Create;
  FGraph := TGraph.Create(FWindow);
  FFontMan := TFontMan.Create(FGraph);
  SetLength(FEvents, 0);
  SDebug.Log('TCore.Create');
  FOnDrawDrag := DrawDrag;
end;
//}}}
destructor TCore.Destroy;//{{{
begin
  SDebug.Log('TCore.Destroy');
  FGraph.Free;
  FSound.Free;
  FWindow.Free;
  FWinApi.Free;
end;
//}}}
procedure TCore.Run;//{{{
begin
  SDebug.Log('TCore.Run');
  FWinApi.Run(30);
end;
//}}}
procedure TCore.StartDrag;//{{{
begin
  FDragging := True;
  FDragStart := FWindow.Mouse;
  FDragDelta := Vec2f(0, 0);
end;

//}}}
procedure TCore.StopDrag;//{{{
begin
  FDragging := False;
end;

//}}}
procedure TCore.Render;//{{{
  var
    I, J: Integer;
begin
  FGraph.BeginRender2D(GWidth, GHeight);
  FGraph.Clear(0.0, 0.0, 0.0);
  FGraph.SetBlendMode(bmAlpha);
  FGraph.PushMatrix;
  // blablabla
  //Translate(Vec2f(GetTimer/100, GetTimer/100));
  FGraph.SetColor(0.25, 0.0, 0.0, 1.0);
  FGraph.DrawRect(Vec2f(50, 50), Vec2f(200, 200), True);
  FGraph.PopMatrix;
  if FDragging then
    FOnDrawDrag;
  FGraph.EndRender;
end;
//}}}
procedure TCore.Update;//{{{
  var
    I, J: Integer;
begin
  if FDragging then
    FOnDrawDrag;
  //for I := 0 to High(FEvents) do begin 
  SetLength(FEvents, 0);
  if FDragging then
    FDragDelta := Vec2f(FWindow.Mouse.X, FWindow.Mouse.Y) - FDragStart;
end;
//}}}
procedure TCore.OnEvent;//{{{
begin
  SetLength(FEvents, Length(FEvents) + 1);
  FEvents[High(FEvents)] := Event;
  if Event is TWinApiKey then
    with TWinApiKey(Event) do
      FKeys[Key] := Boolean(Down);
end;
//}}}
function TCore.Key(ID: Byte): Boolean;//{{{
begin
  Result := FKeys[ID];
end;

//}}}
function TCore.Mouse(ID: TMouseButton): Boolean;//{{{
begin
  Result := FMouseIDs[ID];
end;
 
//}}}
//}}}

end.
