unit DWeapon;

interface

uses
  DUtils,
  DMath,
  DIntersection,
  DGraph,
  DCore,
  DMap,
  DGame,
  DSound;

type
TShotGun = class(TWeapon)//{{{
protected
  //     ,     
  procedure DoFire(Aim: TVec2f; Level: Integer); override;
public
  //  -      
  constructor Create; override;
  //        -   16
  procedure Render; override;
  //  :        , 
  //  . Pos -   ,    
  class procedure RenderItem(Pos: TVec2f); override;
end;
//}}}
TLaser = class(TWeapon)//{{{
protected
  procedure DoFire(Aim: TVec2f; Level: Integer); override;
public
  constructor Create; override;
  procedure Render; override;
  class procedure RenderItem(Pos: TVec2f); override;
end;
//}}}
TLaserPotron = class(TPotron)//{{{
private
  FStart: TVec2f;
  FColor: TVec4f;
  //FSound: TSoundID;
  FWidth: Single;
  FDir: TVec2f;
  FFinish: TVec2f;
  FSavedTime: Integer; 
  FPassed: Boolean;
protected
  procedure Stream; override;
public
  constructor Create(Pos, Vel: TVec2f);
  destructor Destroy; override;
  procedure Update; override;
  procedure Render; override;
  procedure OnCollide(Map: TMap; Block: Integer); overload; override;
  procedure OnCollide(Enemy: TEnemy); overload; override;
  procedure Attack;
end;
//}}}
TPinkWeapon = class(TWeapon)//{{{
protected
  FTex: TTexture;
  procedure DoFire(Aim: TVec2f; Level: Integer); override;
  procedure Stream; override;
public
  constructor Create; override;
  procedure Render; override;
  class procedure RenderItem(Pos: TVec2f); override;
end;
//}}}
TPinkPotron = class(TPotron)//{{{
private
  FStart: TVec2f;
  FColor: TVec4f;
  FDist: Single;
  FSound: TSoundID;
protected
  procedure Stream; override;
public
  constructor Create(Pos, Vel: TVec2f; Size, Dist: Single; Color: TVec4f);
  destructor Destroy; override;
  procedure Update; override;
  procedure Render; override;
  procedure OnCollide(Enemy: TEnemy); override;
end;
//}}}
const
  PinkColor: TVec4f = (x:0.8; y:0.32; z:0.8; w:1.0);

var
  StarTex: Ttexture = -1;
  
implementation

procedure DrawGlassHalf(Pos: TVec2f);
  var
    I: Integer;
    Vertices: array of TVec2f;
begin
  SetLength(Vertices, 10);
  for I := 0 to High(Vertices) do
    Vertices[I] := Vec2f(Pos.X + 8*Cos(I/High(Vertices)*pi),
                         Pos.Y + 8*Sin(I/High(Vertices)*pi));
  with Core.Graph do begin
    DrawPolygon(Vertices);
  end;
end;
//{{{ TShotGun
//////////////////////////////////////////////
////             TShotGun                 ////
//////////////////////////////////////////////

constructor TShotGun.Create;
begin
  //   - 800 
  inherited Create(800);
end;

procedure TShotGun.DoFire(Aim: TVec2f; Level: Integer);
  var
    I: Integer;
    Dir: TVec2f;
    P: TSimplePotron;
    R: TVec2f;
begin
  inherited;
  //  
  Dir := (Aim - Pos) / Len2f(Aim - Pos);
  for I := 1 to 7 do begin
    //  
    R := Vec2f(Random - 0.5, Random - 0.5) / 100;
    //  
    P := TSimplePotron.Create(Pos, (Dir + R) * 10, 
                              4, 2000, Vec4f(0.5, 0.2, 0.0, 1.0));    
    //   ,      
    P.Level := Level;
    //    
    Game.AddPotron(P);
  end;
  Core.Sound.PlaySound('DATA\sounds\ShotgunWeapon.wav');
end;

procedure TShotGun.Render; 
begin
  //      
  Core.Graph.SetBlendMode(bmAlpha);
  Core.Graph.SetColor(0.5, 0.1, 0.0, 0.5);
  DrawGlassHalf(Pos);
  Core.Graph.SetBlendMode(bmNo);
end;

class procedure TShotGun.RenderItem(Pos: TVec2f); 
begin
  Core.Graph.SetBlendMode(bmAlpha);
  Core.Graph.SetColor(0.5, 0.1, 0.0, 0.5);
  DrawGlassHalf(Pos-Vec2f(0,8));
  Core.Graph.SetBlendMode(bmNo);
end;
//}}}
//{{{ TLaser
//////////////////////////////////////////////
////             TLaser                   ////
//////////////////////////////////////////////

constructor TLaser.Create;
begin
  inherited Create(2000);
end;

procedure TLaser.DoFire(Aim: TVec2f; Level: Integer);
var
  Dir: TVec2f;
  P: TLaserPotron;
begin
  inherited;
  Dir := (Aim - Pos) / Len2f(Aim - Pos);
  P := TLaserPotron.Create(Pos, Dir);  
  P.Level := Level;
  P.Attack;
  Game.AddPotron(P);
  Core.Sound.PlaySound('DATA\sounds\LaserWeapon.wav');
end;

procedure TLaser.Render; 
begin
  Core.Graph.SetBlendMode(bmAlpha);
  if GetReady then
    Core.Graph.SetColor(1.0, 0.1, 0.1, 1.0)
  else
    Core.Graph.SetColor(0.8, 0.1, 0.1, 1.0);
  //DrawGlassHalf(Pos);
  Core.Graph.DrawCircle(Pos + Vec2f(0, 2), 8);
  Core.Graph.SetBlendMode(bmNo);
end;

class procedure TLaser.RenderItem(Pos: TVec2f); 
begin
  Core.Graph.SetBlendMode(bmAlpha);
  Core.Graph.SetColor(1.0, 0.0, 0.0, 1.0);
  //DrawGlassHalf(Pos-Vec2f(0,8));
  Core.Graph.DrawCircle(Pos + Vec2f(0, 2), 8);
  Core.Graph.SetBlendMode(bmNo);
end;
//}}}
//{{{ TLaserPotron
//////////////////////////////////////////////
////             TLaserPotron             ////
//////////////////////////////////////////////

procedure TLaserPotron.Stream;
begin
  inherited;
  Stream(@FStart, SizeOf(FStart));
  Stream(@FColor, SizeOf(FColor));
  //Stream(@FSound: TSoundID;
  Stream(@FWidth, SizeOf(FWidth));
  Stream(@FDir, SizeOf(FDir));
  Stream(@FFinish, SizeOf(FFinish));
  Stream(@FSavedTime, SizeOf(FSavedTime)); 
  Stream(@FPassed, SizeOf(FPassed));
end;

constructor TLaserPotron.Create(Pos, Vel: TVec2f);
  var
    I: Integer;
begin
  inherited Create;
  Self.Pos := Pos;
  FStart := Pos;
  FColor := Vec4f(1.0, 0.0, 0.0, 1.0);
  //FSound := Core.Sound.LoadSound('DATA\sounds\LaserWeapon.wav');
  FWidth := 5; 
  FDir := Vel;
  FFinish := Pos + 5000 * FDir;
  //Core.Sound.PlaySound(FSound);
  FSavedTime := Game.Time;    
  FPassed := False;
end;

destructor TLaserPotron.Destroy;
begin
  //Core.Sound.DelSound(FSound);
end;

procedure TLaserPotron.Update; 
begin
  Attack;
  if (Game.Time - FSavedTime) < 1500 then
  begin
    FWidth := (1 - (Game.Time - FSavedTime)/1500) * 5; 
    Core.Graph.SetLineWidth(FWidth);
  end else
    FGarbage := True;   	  
end;

procedure TLaserPotron.Render; 
begin
  Core.Graph.Color4f(FColor);
  Core.Graph.SetLineWidth(FWidth);
  Core.Graph.DrawLine(Pos, FFinish);
end;

procedure TLaserPotron.OnCollide(Map: TMap; Block: Integer);
begin
end;

procedure TLaserPotron.OnCollide(Enemy: TEnemy);
begin
end;

procedure TLaserPotron.Attack;
  var
    I: Integer;
    T, Temp: Single;
begin
  T := 5000;
  for I := 0 to Game.Map.BlocksCount - 1 do
    if (Level <= I) and not (Game.Map.Blocks[I].Speed > 0.1) then
      if Intersect(Pos, FDir, Game.Map.Blocks[I].Rect, Temp) then 
        if Temp < T then begin
          T := Temp;
          FFinish := Pos + T * FDir;
        end;
  {for I := 0 to Game.EnemyCount - 1 do
    if Intersect(Pos, FDir, Game.Enemies[I].Rect, Temp) then
      if Temp < T then begin
        Game.Enemies[I].Damage(1);
      end;}
  if Intersect(Pos, FDir, Game.Player.Rect, Temp) then
    Game.GoToScene('lose');
end;
//}}}
// TPinkWeapon//{{{
//////////////////////////////////////////////
////             TPinkWeapon              ////
//////////////////////////////////////////////
procedure TPinkWeapon.Stream;
begin
  inherited;
  if Loading then
    FTex := Core.Graph.LoadTex(Data.ReadStr)
  else
    Data.WriteStr(Core.Graph.GetTexFileName(FTex));
end;

constructor TPinkWeapon.Create;
begin
  inherited Create(100);
end;

procedure TPinkWeapon.DoFire;
  var
    Potron: TPotron;
begin
  inherited;
  Potron := TPinkPotron.Create(Pos, (Aim-Pos)/Len2f(Aim-Pos)*10, 4, 2000, PinkColor);
  Potron.Level := Level;
  Game.AddPotron(Potron);
  Core.Sound.PlaySound('DATA\sounds\PinkWeapon.wav');
end;

procedure TPinkWeapon.Render;
begin
  if StarTex = -1 then StarTex := Core.Graph.LoadTex('DATA\textures\star.bit');
  Core.Graph.SetTexture(StarTex);
  Core.Graph.SetBlendMode(bmAlpha);
  Core.Graph.Color4f(PinkColor);
  Core.Graph.DrawTexture(Rect(Pos - Vec2f(10, 8), Pos + Vec2f(10, 12)));
  Core.Graph.SetTexture(-1);
  Core.Graph.SetColor(1, 1, 1, 1);  
end;

class procedure TPinkWeapon.RenderItem(Pos: TVec2f); 
begin
  if StarTex = -1 then StarTex := Core.Graph.LoadTex('DATA\textures\star.bit');	
  Core.Graph.SetTexture(StarTex);
  Core.Graph.SetBlendMode(bmAlpha);
  Core.Graph.Color4f(PinkColor);
  Core.Graph.DrawTexture(Rect(Pos - Vec2f(10, 8), Pos + Vec2f(10, 12)));
  Core.Graph.SetTexture(-1);
  Core.Graph.SetColor(1, 1, 1, 1);  
end;
//}}}
//{{{ TPinkPotron
//////////////////////////////////////////////
////             TPinkPotron              ////
//////////////////////////////////////////////

procedure TPinkPotron.Stream;
begin
  inherited;
  Stream(@FStart, SizeOf(FStart));
  Stream(@FColor, SizeOf(FColor));
  Stream(@FDist, SizeOf(FDist));
end;

constructor TPinkPotron.Create;
begin
  inherited Create;
  Self.Pos := Pos;
  FStart := Pos;
  FHalf := Vec2f(Size/2, Size/2);
  FVel := Vel;
  FDist := Dist;
  FColor := Color;
  //FSound := Core.Sound.LoadSound('DATA\sounds\PinkWeapon.wav');
  //Core.Sound.PlaySound(FSound);
end;

destructor TPinkPotron.Destroy;
begin
  //Core.Sound.DelSound(FSound);
end;

procedure TPinkPotron.Update; 
begin
  Pos := Pos + FVel;
  if Len2f(FStart - Pos) > FDist then
    FGarbage := True;
end;

procedure TPinkPotron.Render; 
begin
  Core.Graph.Color4f(FColor);
  Core.Graph.DrawCircle(Pos, FHalf.X, True);
end;

procedure TPinkPotron.OnCollide(Enemy: TEnemy); 
begin
  Enemy.Color := PinkColor;
  Kill;
end;
//}}}

initialization
  Factory.Regist(TShotGun);
  Factory.Regist(TLaser);
  Factory.Regist(TLaserPotron);
  Factory.Regist(TPinkWeapon);
  Factory.Regist(TPinkPotron);
end.
