unit e_enemies;

interface

uses OpenGL, e_types, e_textures, e_models, e_particles, e_sound, e_utils,
     e_global, e_console, e_camera, e_weapons;

type
  TEnemy = class(TCreature)
    timer          : single;
    dead           : boolean;
    visible        : single;

    particles1     : TParticlesEmitter;
    particles2     : TParticlesEmitter;
  public
    constructor Create(_pos : TPoint);                       virtual;
    procedure   GetDamage(damage : single; bullet_pos : TPoint); virtual;
    procedure   RenderBack;                                  virtual;
    procedure   RenderFront;                                 virtual;
    procedure   Update(count : single);                      virtual;
  end;

  TEnemy1 = class(TEnemy)
  public
    constructor Create(_pos : TPoint);                       override;
    procedure   RenderBack;                                  override;
    procedure   RenderFront;                                 override;
    procedure   Update(count : single);                      override;
  end;

  TEnemy2 = class(TEnemy)
    burst : byte;
  public
    constructor Create(_pos : TPoint);                       override;
    procedure   RenderBack;                                  override;
    procedure   RenderFront;                                 override;
    procedure   Update(count : single);                      override;
  end;

  TEnemy3 = class(TEnemy)
    active : boolean;
  public
    constructor Create(_pos : TPoint);                       override;
    procedure   Update(count : single);                      override;
  end;

implementation

uses e_game;

constructor TEnemy.Create(_pos : TPoint);
begin
inherited Create;

size         := 1;
pos          := _pos;
dead         := false;
timer        := 0;
visible      := -1;
end;

procedure TEnemy.GetDamage(damage : single; bullet_pos : TPoint);
begin
end;

procedure TEnemy.RenderBack;
begin
glColor4f(1, 1, 1, 1);
glPushMatrix;
with pos do glTranslatef(x, y, 0);
glScalef(size, size, 1);

SetTexture(texture_enemy3[1]);
glCallList(model_quad);

glRotatef(angle, 0, 0, 1);
SetTexture(texture_enemy3[2]);
glCallList(model_quad);
glPopMatrix;
end;

procedure TEnemy.RenderFront;
begin
glColor4f(1, 1, 1, 1);
glPushMatrix;
with pos do glTranslatef(x, y, 0);
glScalef(size, size, 1);
glRotatef(angle, 0, 0, 1);
SetTexture(texture_enemy3[3]);
glCallList(model_quad);
glPopMatrix;
end;

procedure TEnemy.Update(count : single);
begin
angle := PointsAngle(game.player.pos, pos);
end;

///////////////////

constructor TEnemy1.Create;
var
i : integer;
begin
inherited Create(_pos);
size := 2;
end;

procedure TEnemy1.RenderBack;
begin
glColor4f(1, 1, 1, 1);
glPushMatrix;
with pos do glTranslatef(x, y, 0);
glScalef(size, size, 1);

SetTexture(texture_enemy1[1]);
glCallList(model_quad);

glRotatef(angle, 0, 0, 1);
SetTexture(texture_enemy1[2]);
glCallList(model_quad);
glPopMatrix;
end;

procedure TEnemy1.RenderFront;
begin
glColor4f(1, 1, 1, 1);
glPushMatrix;
with pos do glTranslatef(x, y, 0);
glScalef(size, size, 1);
glRotatef(angle, 0, 0, 1);
SetTexture(texture_enemy1[3]);
glCallList(model_quad);
glPopMatrix;
end;

procedure TEnemy1.Update(count : single);
begin
inherited Update(count);

if timer > 0 then
  timer := timer - count
else
  if (visible < 0) or CollideCircles(pos, game.player.pos, visible) then
  begin
    timer := 3000;
    game.AddBullet(1, self);
  end;
end;

///////////////////

constructor TEnemy2.Create;
var
i : integer;
begin
inherited Create(_pos);

size    := 1.5;
burst   := 0;
end;

procedure TEnemy2.RenderBack;
begin
glColor4f(1, 1, 1, 1);
glPushMatrix;
with pos do glTranslatef(x, y, 0);
glScalef(size, size, 1);

SetTexture(texture_enemy2[1]);
glCallList(model_quad);

glRotatef(angle, 0, 0, 1);
SetTexture(texture_enemy2[2]);
glCallList(model_quad);
glPopMatrix;
end;

procedure TEnemy2.RenderFront;
begin
glColor4f(1, 1, 1, 1);
glPushMatrix;
with pos do glTranslatef(x, y, 0);
glScalef(size, size, 1);
glRotatef(angle, 0, 0, 1);
SetTexture(texture_enemy2[3]);
glCallList(model_quad);
glPopMatrix;
end;

procedure TEnemy2.Update(count : single);
begin
inherited Update(count);

if timer > 0 then
  timer := timer - count
else
  if (visible < 0) or (burst <> 0) or CollideCircles(pos, game.player.pos, visible) then
  begin
    game.AddBullet(2, self);
    timer := 300;
    inc(burst);
    if burst >= 5 then
      begin
        burst := 0;
        timer := 4000;
      end;
  end;
end;

///////////////////

constructor TEnemy3.Create;
var
i : integer;
begin
inherited Create(_pos);

size := 2.5;
active := true;
end;

procedure TEnemy3.Update(count : single);
begin
inherited Update(count);

if active then
  if (visible < 0) or CollideCircles(pos, game.player.pos, visible) then
  begin
    timer := timer - count;
    if timer < 0 then
      begin
        timer := 500;
        game.AddBullet(3, self);
        active := false;
      end;
  end;
end;

end.
