unit e_lines;

interface

uses OpenGL, e_models, e_textures, e_types, e_utils, e_sound;

type
  TLine = record
    a, b    : TPoint;
    ra, rb  : TPoint;
  end;

  TObjectState = record
    pos            : TPoint;
    angle          : single;
  end;

  TRoom = class
    a, b : TPoint;
  public
    constructor Create(x1, y1, x2, y2 : single);
    function    PointIn(p : TPoint) : boolean;
    procedure   RenderEdit;
    procedure   Render;
  end;

  TObject = class
    pos            : TPoint;
    angle          : single;
    size           : TPoint;
    state1, state2 : TObjectState;
    state_target   : byte;
    lines          : array of TLine;
    selected       : boolean;
    texture        : gluint;
    collide_units  : boolean;
    collide_shoots : boolean;

    animation_speed : single;
    animate_speed_pos  : single;
    animate_speed_angle : single;
    animate_rotate_always : boolean;
    animate_loop   : boolean;
  public
    constructor Create(_pos : TPoint; _angle : single; _size : TPoint; _texture : gluint);
    procedure   Count;
    procedure   Animate(x, y, a, sp, sa : single; _loop : boolean = false);
    procedure   AnimateRotation(speed : single);
    procedure   Render;
    procedure   Update(count : single);
  end;

const
  GAME_TIMER_BULLET = 200;
  GAME_HAND_DELTA   = 0.55;

var
  developer_mode : boolean = true;

implementation

constructor TRoom.Create(x1, y1, x2, y2 : single);
begin
a := GetPoint(x1, y1);
b := GetPoint(x2, y2);
end;

function TRoom.PointIn(p : TPoint) : boolean;
begin
result := false;
if (p.x < Min(a.x, b.x)) or (p.x > Max(a.x, b.x)) then exit;
if (p.y < Min(a.y, b.y)) or (p.y > Max(a.y, b.y)) then exit;
result := true;
end;

procedure TRoom.RenderEdit;
begin
glDisable(GL_TEXTURE_2D);

glPushMatrix;
glTranslatef((a.x + b.x) / 2, (a.y + b.y) / 2, 0);
glColor4f(0.5, 0.5, 1, 0.5);
glScalef(abs(a.x - b.x), abs(a.y - b.y), 0);
glCallList(model_quad);
glPopMatrix;


glPushMatrix;
glTranslatef(a.x, a.y, 0);
glColor4f(0, 0, 1, 0.5);
glScalef(0.5, 0.5, 0);
glCallList(model_quad);
glPopMatrix;

glPushMatrix;
glTranslatef(b.x, b.y, 0);
glColor4f(0, 0, 1, 0.5);
glScalef(0.5, 0.5, 0);
glCallList(model_quad);
glPopMatrix;

glEnable(GL_TEXTURE_2D);
end;

procedure TRoom.Render;
begin
glDisable(GL_TEXTURE_2D);

glPushMatrix;
glTranslatef((a.x + b.x) / 2, (a.y + b.y) / 2, 0);
glColor4f(0, 0, 0, 1);
glScalef(abs(a.x - b.x), abs(a.y - b.y), 1);
glCallList(model_quad);
glPopMatrix;

glEnable(GL_TEXTURE_2D);
end;

constructor TObject.Create(_pos : TPoint; _angle : single; _size : TPoint; _texture : gluint);
begin
state1.pos      := _pos;
state1.angle    := _angle;
state2          := state1;
size            := _size;
texture         := _texture;

pos             := state1.pos;
angle           := state1.angle;
collide_units   := true;
collide_shoots  := true;
state_target    := 2;

animation_speed := -1;

SetLength(lines, 4);
lines[0].ra.x := -0.5;
lines[0].ra.y := -0.5;
lines[0].rb.x :=  0.5;
lines[0].rb.y := -0.5;

lines[1].ra.x := -0.5;
lines[1].ra.y :=  0.5;
lines[1].rb.x :=  0.5;
lines[1].rb.y :=  0.5;

lines[2].ra.x := -0.5;
lines[2].ra.y := -0.5;
lines[2].rb.x := -0.5;
lines[2].rb.y :=  0.5;

lines[3].ra.x :=  0.5;
lines[3].ra.y := -0.5;
lines[3].rb.x :=  0.5;
lines[3].rb.y :=  0.5;
end;

procedure TObject.Count;
var
i : integer;
s, c : single;
begin
for i := 0 to high(lines) do
  with lines[i] do
    begin
      s := sin(angle / 180 * pi);
      c := cos(angle / 180 * pi);

      a.x := (ra.x * c * size.x - ra.y * s * size.y) + pos.x;
      a.y := (ra.x * s * size.x + ra.y * c * size.y) + pos.y;
      b.x := (rb.x * c * size.x - rb.y * s * size.y) + pos.x;
      b.y := (rb.x * s * size.x + rb.y * c * size.y) + pos.y;
    end;
end;

procedure TObject.Animate(x, y, a, sp, sa : single; _loop : boolean = false);
begin
animation_speed := 1;
state2.pos.x := x;
state2.pos.y := y;
state2.angle := a;
animate_speed_pos := abs(sp);
animate_speed_angle := sa;
animate_loop := _loop;
end;

procedure TObject.AnimateRotation(speed : single);
begin
Animate(pos.x, pos.y, angle, 0, speed);
animate_rotate_always := true;
end;

procedure TObject.Render;
var
i : integer;
s : single;
begin
glColor4f(1, 1, 1, 1);
SetTexture(texture);
glPushMatrix;
with pos do glTranslatef(x, y, 0);
glRotatef(angle, 0, 0, 1);
glScalef(size.x, size.y, 1);
ScaleTexture(size.x, size.y);
glCallList(model_quad);
UnScaleTexture;
glPopMatrix;


{glDisable(GL_TEXTURE_2D);
glLineWidth(1);
glPointSize(1);
glColor4f(1, 1, 1, 1);
for i := 0 to high(lines) do
    begin     
      glBegin(GL_POINTS);
      glVertex2f(lines[i].a.x, lines[i].a.y);
      glVertex2f(lines[i].b.x, lines[i].b.y);
      glEnd;

      glBegin(GL_LINES);
      glVertex2f(lines[i].a.x, lines[i].a.y);
      glVertex2f(lines[i].b.x, lines[i].b.y);
      glEnd();
    end;
glEnable(GL_TEXTURE_2D);  }
end;

procedure TObject.Update(count : single);
var
s : TObjectState;
b1, b2 : boolean;
begin
if animation_speed <= 0 then exit;

s := state1;
if state_target = 2 then s := state2;

b1 := AspirePoint(pos, s.pos, animation_speed * animate_speed_pos * count * 0.001);
if animate_rotate_always then
  begin
    b2 := true;
    angle := angle + animation_speed * animate_speed_angle * count * pi * 0.005;
  end
else
  b2 := Aspire(angle, s.angle, animation_speed * animate_speed_angle * count * pi * 0.005);
NormalizeAngle(angle);

self.Count;

if animate_loop then
  if b1 and b2 then
    if state_target = 1 then
      state_target := 2
    else
      state_target := 1;   
end;

end.
