unit e_main;

interface

uses Windows, OpenGL, e_render, e_global, e_sound, e_types, e_models, e_particles,
     e_utils, e_screen, e_game, e_textures, e_font, e_camera,
     e_console, e_menu;

procedure GameFinish;     
function  GameStart(number : integer = 1) : boolean;
procedure Initialize;
procedure Deinitialize;
procedure Update(count : single);
procedure Render;

implementation

procedure ResetTime;
begin
count_fps := 0;
count_last := GetTickCount;
count_sec := count_last;
time := 0;
time_scale := 1;
end;

procedure GameFinish;
begin
game.Save;
game_started := false;
game.Free;
particles.Free;
Console.Write('Game finished');
end;

function GameStart(number : integer = 1) : boolean;
begin
result := false;

if game_started then GameFinish;

particles    := TParticlesEngine.Create;
game         := TGame.Create(number);
with game.player do
  begin
    particles_trail := particles.AddEmitter(particles_smoke, pos, 50, GetColor(1, 1, 1, 1));
    particles_trail.parent := game.player;
  end;


ResetTime;
game_started := true;
game_restart := false;

result := true;

Console.Write('Game started: ' + inttostr(number) + ' level');
end;

procedure Initialize;
begin
menu_active := true;
ModelsInitialize;
Console.Write('Models');
TexturesInitialize;
Console.Write('Textures');
SoundInitialize;
Console.Write('Sound');
LogoInitialize;
Console.Write('Intro');
font := TFont.Create;
Console.Write('Font');
clear_color := GetColorB(234, 227, 115, 255);
clear_color := GetColorB(128, 128, 128, 255);

profiles := TProfiles.Create;
profiles.Load;
Console.Write('Profiles');

GameStart(1);

menu_active := true;
end;

procedure Deinitialize;
begin
Console.Write('');
Console.Write('Deinitialization');
profiles.Save;
GameFinish;
SoundDeinitialize;
end;

procedure Update(count : single);
var
a : integer;
begin
console.Update(count);
//count := count * time_scale;
time := time + count;

texture_time := time * 0.0005;

SoundUpdate(count);


if logo_count < logo_time then
  begin
    LogoUpdate(count);
    exit;
  end;

if menu_active then
  MenuUpdate(count);

if (not menu_active) or (game.game_over) then
  if (not game_pause) then
    begin
      game.Update(count);

      if game_restart then
        if game.stage_number_next = 0 then
            quit := true
        else
          begin
            a := game.stage_number_next;
            GameStart(a);
            game.StageSkip(true);
          end;
    end;
end;

procedure CountFPS;
begin
count_new := GetTickCount;

inc(count_fps);
if (count_new - count_sec) >= 1000 then
  begin
    count_sec := count_sec + 1000;
    fps := count_fps;
    count_fps := 0;
  end;
end;

procedure RenderStart;
begin
CountFPS;

glMatrixMode(GL_PROJECTION);
glLoadIdentity;

glViewPort(0, 0, screen.width, screen.height);

with clear_color do glClearColor(r, g, b, a);
glClear(GL_COLOR_BUFFER_BIT or GL_DEPTH_BUFFER_BIT);
end;

procedure RenderFinish;
begin
glMatrixMode(GL_MODELVIEW);
glLoadIdentity;
SwapBuffers(DC);
count_last := count_new;
end;



procedure Render;
begin
RenderStart;

if logo_count < logo_time then
  LogoRender
else  
  begin
    if menu_active then
      MenuRender
    else
      e_render.Render;
  end;

Console.Render;

RenderFinish;
end;

end.
