unit e_menu;

interface

uses OpenGL, windows, e_utils, e_types, e_font, e_models, e_textures,
     e_global, e_about, e_sound,
     e_level1, e_level2, e_level3, e_level4, e_level5, e_level6;

procedure MenuRender;
procedure MenuUpdate(count : single);
procedure LogoInitialize;
procedure LogoRender;
procedure LogoUpdate(count : single);

const
  logo_time = 3000;
  logo_text = 'OUEO factory';

var
  menu_selected : integer = 1;
  menu_selected_pos : single = 1;
  logo_count   : single = 0;

implementation

uses e_main;

function RenderStage(i : integer) : single;
var
a : single;
s : string;
begin
a := 1 - abs(i - menu_selected_pos);
if a <= 0 then
  exit;

glColor4f(1, 0, 0, sqr(a));
font.TextOut(21.5, (i - menu_selected_pos) * 1, 0, inttostr(i), faCenter);

glColor4f(1, 1, 1, a);
font.TextOut(20, (i - menu_selected_pos) * 1, 0, inttostr(i), faCenter);
s := '';
case i of
  1 : s := e_level1.NAME;
  2 : s := e_level2.NAME;
  3 : s := e_level3.NAME;
  4 : s := e_level4.NAME;
  5 : s := e_level5.NAME;
  6 : s := e_level6.NAME;
end;
glColor4f(1, 1, 1, a * 0.7);
font.TextOut(7, (i - menu_selected_pos) * 1 + 1.5, 0, s);
end;

procedure MenuRender;
var
i : integer;
begin
glClearColor(0, 0, 0, 1);
glClear(GL_COLOR_BUFFER_BIT or GL_DEPTH_BUFFER_BIT);

ViewOrtho;

glDisable(GL_LIGHTING);
glDisable(GL_DEPTH_TEST);
glEnable(GL_BLEND);
glEnable(GL_TEXTURE_2D);

glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

glColor4f(1, 1, 1, 0.7);
font.TextOut(7, -1.5, 0, 'lide', faRight);

for i := 1 to max_levels do
  RenderStage(i);

glColor4f(1, 1, 1, 0.5);
font.TextOut(5, -18, -14, 'v0.' + inttostr(program_version), faCenter);
end;

procedure MenuUpdate(count : single);
begin
SqrAspire(menu_selected_pos, menu_selected, count * 0.002);
Aspire(menu_selected_pos, menu_selected, count * 0.0001);

if keys[vk_up] or keys[ord('W')] or keys[vk_left] or keys[ord('A')] then
  begin
    dec(menu_selected);
    keys[vk_up] := false;
    keys[ord('W')] := false;
    keys[vk_left] := false;
    keys[ord('A')] := false;
    SoundPlay(sound_menu, 0);
  end;

if keys[vk_down] or keys[ord('S')] or keys[vk_right] or keys[ord('D')] then
  begin
    inc(menu_selected);
    keys[vk_down] := false;
    keys[ord('S')] := false;
    keys[vk_right] := false;
    keys[ord('D')] := false;
    SoundPlay(sound_menu, 0);
  end;

if keys[vk_return] or keys[vk_space] then
  begin
    GameStart(menu_selected);
    menu_active := false;
    keys[vk_return] := false;
    keys[vk_space] := false;
  end;

if menu_selected > max_levels then
  menu_selected := max_levels;

if menu_selected < 1 then
  menu_selected := 1;
end;

procedure LogoInitialize;
var
i : integer;
begin
if not render_logo then logo_count := logo_time;
end;

function RenderLetter(i : integer) : single;
var
a : single;
s : single;
begin
s := logo_count / (logo_time - 1000) * length(logo_text);

a := 1 - abs(i - 1.2 * s);
if (a < 0.5) and (i < 1.2 * s) then
  a := 0.5;

if s > length(logo_text) - 1 then
  begin
    s := length(logo_text) - 1;
    a := 1 - logo_count / logo_time;
    if a > 0.5 then a := 0.5;    
  end;

glColor4f(1, 1, 1, a);
font.TextOut(20, (i - s) * 1.5, 0, logo_text[i], faCenter);
end;

procedure LogoRender;
var
i : integer;
s : single;
begin
glClearColor(0, 0, 0, 1);
glClear(GL_COLOR_BUFFER_BIT or GL_DEPTH_BUFFER_BIT);

ViewOrtho;

glDisable(GL_LIGHTING);
glDisable(GL_DEPTH_TEST);
glEnable(GL_BLEND);
glEnable(GL_TEXTURE_2D);

glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

for i := 1 to length(logo_text) do
  RenderLetter(i);
end;

procedure LogoUpdate(count : single);
begin
logo_count := logo_count + count;
end;

end.

