unit e_player;

interface

uses OpenGL, Windows, e_types, e_textures, e_models, e_global, e_utils, e_camera, e_sound,
     e_particles, e_font, e_weapons, e_console, Bass;

type
  TPlayer = class(TCreature)
    health       : integer;
    score        : integer;
    health_timer : single;
    health_alpha : single;
    health_pos   : TPoint;
    particles_trail : TParticlesEmitter;
  public
    constructor Create;
    procedure   GetDamage(power, distance : single);
    procedure   GetScore(_score : integer);
    procedure   Render;
    procedure   Update(count : single);
  end;

implementation

uses e_game;

constructor TPlayer.Create;
begin
health  := 100;
score   := 0;

pos     := GetPoint(0, 0);
size    := 0.7;
angle   := 180;
health_timer := 0;
health_alpha := 0;
end;

procedure TPlayer.GetDamage(power, distance : single);
var
n : integer;
begin
if distance < 1 then
  distance := 1;
n := round(power / distance * 1.5);
health := health - n;

if n > 0 then
  begin
    health_timer := 1000;
    health_pos := pos;
  end;

if health <= 0 then
  begin
    particles_trail.loop := false;
    game.GameOver(false);
  end;
//  game.GameOver(false);
end;

procedure TPlayer.GetScore(_score : integer);
begin
score := score + _score;
health := health + _score * 1;
if health > 100 then health := 100;
end;

procedure TPlayer.Render;
begin
if health <= 0 then
  exit;

glColor4f(1, 1, 1, health_alpha);
font.TextOut(20, health_pos.x, health_pos.y, inttostr(health), faCenter);

glColor4f(1, 1, 1, 1);
SetTexture(texture_player);
glPushMatrix;
with pos do glTranslatef(x, y, 0);
glScalef(size * 2, size * 2, 1);
glRotatef(angle, 0, 0, 1);
glScalef(1, 0.5, 1);
glCallList(model_quad);
glPopMatrix;
end;

procedure TPlayer.Update(count : single);
var
i : integer;
s : single;
begin
if health <= 0 then
  exit;

health_timer := health_timer - count;
if health_timer < 0 then
  health_timer := 0;
SqrAspire(health_alpha, sqr(health_timer / 1000), count * 0.002);

s := pos.x / 26;
if s < -1 then s := -1;
if s >  1 then s :=  1;
BASS_ChannelSetAttribute(sounds[music].handle, BASS_ATTRIB_PAN, s);

if game.CollideMap(self) then
  game.Explosion(pos, 1, 1000, true, 0.1);

i := 0;
if keys[ord('W')] then
  i := 1;
if keys[ord('S')] then
  i := -1;
Aspire(speed_max, 90 + i * 80, count * 0.1);

i := 0;
if keys[ord('A')] then
  i := 1;
if keys[ord('D')] then
  i := -1;
if keys[ord('Q')] then
  begin
    game.AddTower(1, pos, 0);
    keys[ord('Q')] := false;
  end;

if keys[ord('Z')] then
  begin
    game.AddEnemy(1, pos);
    keys[ord('Z')] := false;
  end;
if keys[ord('X')] then
  begin
    game.AddEnemy(2, pos);
    keys[ord('X')] := false;
  end;
if keys[ord('C')] then
  begin
    game.AddEnemy(3, pos);
    keys[ord('C')] := false;
  end;

angle := angle + i * count * 0.3 * (1 + 0*speed_max * 0.01);
NormalizeAngle(angle);

SqrAspire(speed.x, speed_max * cos(angle / 180 * pi) * 0.002, count * 0.005);
SqrAspire(speed.y, speed_max * sin(angle / 180 * pi) * 0.002, count * 0.005);

pos.x := pos.x + speed.x * count * 0.05;
pos.y := pos.y + speed.y * count * 0.05;
{pos.x := pos.x + speed_max * cos(angle / 180 * pi) * 0.002;
pos.y := pos.y + speed_max * sin(angle / 180 * pi) * 0.002;  }
end;

end.
