unit e_types;

interface

uses OpenGL;

type
  TString = string[40];
  PString = ^TString;

  TSelect = record
    elements : array of string;
    number   : integer;
  end;
  PSelect = ^TSelect;

  TPoint = record
    x, y : single;
  end;
  PPoint = ^TPoint;

  TPointD = record
    x, y, z : double;
  end;

  TPointI = record
    x, y : integer;
  end;

  TField = record
    a, b : TPointI;
  end;

  TColor = record
    r, g, b, a : single;
  end;

  TCreature = class
    pos          : TPoint;
    speed        : TPoint;
    speed_max    : single;
    angle        : single;
    size         : single;
  end;
  
procedure IncSelect(select : PSelect);
procedure DecSelect(select : PSelect);
function  GetPoint(x, y : single) : TPoint;
function  GetPointI(x, y : integer) : TPointI;
function  FlipPoint(p : TPoint) : TPoint;
procedure Normalize(var p : TPoint);
procedure NormalizeLength(var p : TPoint; l : single);
function  CrossProduct(a, b : TPoint) : TPoint;
function  GetField(a, b : TPointI) : TField;
function  EqualPoints(p1, p2 : TPoint; d : single) : boolean;
function  GetColor(red, green, blue, alpha : single) : TColor;
function  GetColorB(red, green, blue, alpha : byte) : TColor;
procedure SetColor(c : TColor);
procedure SetColorAlpha(c : TColor; alpha : single);

const
  null_point : TPoint = (x : 0; y : 0);
  null_field : TField = (a : (x : -1; y : -1); b : (x : -1; y : -1));

implementation

procedure IncSelect(select : PSelect);
begin
with select^ do
  begin
    inc(number);
    if number > high(elements) + 1 then number := 1;
  end;
end;

procedure DecSelect(select : PSelect);
begin
with select^ do
  begin
    dec(number);
    if number < 1 then number := high(elements) + 1;
  end;
end;

function GetPoint(x, y : single) : TPoint;
begin
result.x := x;
result.y := y;
end;

function GetPointI(x, y : integer) : TPointI;
begin
result.x := x;
result.y := y;
end;

function FlipPoint(p : TPoint) : TPoint;
begin
result.x := -p.x;
result.y := -p.y;
end;

procedure Normalize(var p : TPoint);
var
s : single;
begin
s := sqrt(sqr(p.x) + sqr(p.y));
p.x := p.x / s;
p.y := p.y / s;
end;

procedure NormalizeLength(var p : TPoint; l : single);
var
s : single;
begin
s := sqrt(sqr(p.x) + sqr(p.y)) / l;
if s < 0.001 then s := 0.001;
p.x := p.x / s;
p.y := p.y / s;
end;

function CrossProduct(a, b : TPoint) : TPoint;
begin
{result.x := a.y * b.z - a.z * b.y;
result.y := a.z * b.x - a.x * b.z;
result.z := a.x * b.y - a.y * b.z; }
end;

function GetField(a, b : TPointI) : TField;
begin
result.a := a;
result.b := b;
end;

function EqualPoints(p1, p2 : TPoint; d : single) : boolean;
begin
result := (abs(p1.x - p2.x) <= d) and (abs(p1.y - p2.y) <= d);
end;

function GetColor(red, green, blue, alpha : single) : TColor;
begin
result.r := red;
result.g := green;
result.b := blue;
result.a := alpha;
end;

function GetColorB(red, green, blue, alpha : byte) : TColor;
begin
result.r := red / 255;
result.g := green / 255;
result.b := blue / 255;
result.a := alpha / 255;
end;

procedure SetColor(c : TColor);
begin
with c do glColor4f(r, g, b, a);
end;

procedure SetColorAlpha(c : TColor; alpha : single);
begin
with c do glColor4f(r, g, b, alpha);
end;

end.
