unit e_weapons;

interface

uses OpenGL, e_types, e_models, e_textures, e_utils, e_particles, e_sound,
     e_camera, e_global, e_font, e_console;

type
  TWeaponParams = record
    damage       : single;
    delay        : single;
    speed        : single;
  end;

const
  weapons_params : array[1..3] of TWeaponParams = (
    (damage : 8;  delay : 750; speed : 0.003),
    (damage : 15; delay : 300; speed : 0.07),
    (damage : 15; delay : 500; speed : 0.055)
    );

type
  TChuck = class(TCreature)
    map          : integer;
    dead         : boolean;
    trapped      : boolean;
    parent       : TCreature;
    texture      : gluint;
    radius       : single;
    power        : single;
    particles_trail : TParticlesEmitter;
    cone         : single;
  public
    constructor Create(_power : single; _parent : TCreature);    virtual;
    destructor  Destroy;                                      override;
    procedure   Render;                                       virtual;
    procedure   Update(count : single);                       virtual;
  end;

  TRocket1 = class(TChuck)
    target : TCreature;
    timer_target : single;
  public
    constructor Create(_power : single; _parent : TCreature); override;
    procedure   Update(count : single);                       override;
  end;

  TRocket2 = class(TChuck)
    target : TPoint;
    power_normal : single;
  public
    constructor Create(_power : single; _parent : TCreature); override;
    procedure   Update(count : single);                       override;
  end;

  TRocket3 = class(TChuck)
    target : TPoint;
  public
    constructor Create(_power : single; _parent : TCreature); override;
    destructor  Destroy;                                      override;
    procedure   Update(count : single);                       override;
  end;

function WeaponShoot(mode : byte; parent : TCreature) : TChuck;

implementation

uses e_game, e_enemies;

function WeaponShoot(mode : byte; parent : TCreature) : TChuck;
begin
result := nil;
if game.player.health <= 0 then
  exit;

if mode <> 3 then
  particles.AddEmitter(particles_shoot, parent.pos, 10, GetColor(1, 1, 1, 1));
  
SoundPlay(sound_shoot, parent.pos.x);
case mode of
  1   : result := TRocket1.Create(20, parent);
  2   : result := TRocket2.Create(10, parent);
  3   : result := TRocket3.Create(30, parent);
  else   result := TRocket1.Create(1, parent);
end;
end;

constructor TChuck.Create(_power : single; _parent : TCreature);
begin
power  := _power;
parent := _parent;
pos    := _parent.pos;
dead   := false;
texture := texture_circle;
trapped := false;

angle := PointsAngle(game.player.pos, pos);
NormalizeAngle(angle);

radius := 1;
cone   := 179;

speed.x := speed_max * cos(angle / 180 * pi) * 0.002;
speed.y := speed_max * sin(angle / 180 * pi) * 0.002;
end;

destructor TChuck.Destroy;
begin
if particles_trail <> nil then
  particles_trail.loop := false;

inherited Destroy;
end;

procedure TChuck.Render;
begin
glColor4f(1, 1, 1, 1);
SetTexture(texture);
glPushMatrix;
with pos do glTranslatef(x, y, 0);
glScalef(size * 2, size * 2, 1);
glRotatef(angle, 0, 0, 1);
glScalef(1, 0.5, 1);
glCallList(model_quad);
glPopMatrix;
end;

function ConeDelta(creature : TCreature; pos : TPoint) : single;
begin
result := creature.angle - PointsAngle(pos, creature.pos);
NormalizeAngle(result);
NormalizeAngle(result);
result := abs(result);
end;

procedure TChuck.Update(count : single);
begin
if game.CollideMap(self) then
  begin
    dead := true;
    game.Explosion(pos, radius, power);
    exit;
  end;
end;

///

constructor TRocket1.Create(_power : single; _parent : TCreature);
begin
speed_max := 50;
inherited Create(_power, _parent);

texture := texture_rocket1;

particles_trail := particles.AddEmitter(particles_smoke, pos, 50, GetColor(1, 1, 1, 1));
particles_trail.parent := self;

target := game.player;

speed_max := 80;
radius := 1.5;
size := 0.7;
end;

procedure TRocket1.Update(count : single);
var
d, s, p : single;
i : integer;
find : boolean;
begin
if target <> nil then
  begin
    s := PointsDistance(pos, target.pos);
    if s < radius then
      if ConeDelta(self, target.pos) < cone then
        begin
          dead := true;
          game.Explosion(pos, radius, power, true, s);
          exit;
        end;
  end;

inherited Update(count);
if dead or trapped then exit;

{size := 0.7;
if target <> game.player then size := 0.5;
if target = nil then size := 0.3;  }

if target <> nil then
  begin
    p := PointsAngle(target.pos, pos);
    NormalizeAngle(p);
    AspireAngle(angle, p, count * 0.2, s);
  end;

speed.x := speed_max * cos(angle / 180 * pi) * 0.002;
speed.y := speed_max * sin(angle / 180 * pi) * 0.002;

pos.x := pos.x + speed.x * count * 0.05;
pos.y := pos.y + speed.y * count * 0.05;   

find := false;  
//  
if target <> nil then
  if ConeDelta(self, target.pos) > cone then
    begin
      target := nil;
      find := true;
    end;

d := -1;
//   ,   
if find then
  if ConeDelta(self, game.player.pos) < cone then
    begin
      target := game.player; 
      d := PointsDistance(pos, target.pos);
    end;

if not dead then
for i := 0 to high(game.bullets) do
  if game.bullets[i] <> nil then
    if (not (game.bullets[i] = self)) and (not game.bullets[i].dead) then
      begin
        //    
        if find then
        begin
        s := PointsDistance(pos, game.bullets[i].pos);
        if (s < d) or (d < 0) then
          if ConeDelta(self, game.bullets[i].pos) < cone then
            if not (game.bullets[i] is TRocket1) then
              begin
                target := game.bullets[i];
                d := s;
              end
            else
              //     
              if TRocket1(game.bullets[i]).target <> self then
                begin
                  target := game.bullets[i];
                  d := s;
                end;
        end;

        if CollideCreatures(self, game.bullets[i]) then
          begin
            dead := true;
            game.bullets[i].dead := true;
            game.Explosion(pos, radius, power);
            with game.bullets[i] do
              game.Explosion(pos, radius, power, false);
          end;
      end;
end;

///

constructor TRocket2.Create(_power : single; _parent : TCreature);
begin
speed_max := -10;
inherited Create(_power, _parent);

texture := texture_rocket2;

particles_trail := particles.AddEmitter(particles_smoke, pos, 30, GetColor(1, 1, 1, 1));
particles_trail.parent := self;

target := game.player.pos;
power_normal := power;

speed_max := 100;
radius := 1.5;
size := 0.5;
end;

procedure TRocket2.Update(count : single);
var
p, s : single;
i, n : integer;
begin
s := PointsDistance(pos, game.player.pos);
if s < radius then
  if ConeDelta(self, game.player.pos) < cone then
    begin
      dead := true;
      game.Explosion(pos, radius, power, true, s);
      exit;
    end;

inherited Update(count);
if dead or trapped then exit;

p := PointsAngle(target, pos);
NormalizeAngle(p);
AspireAngle(angle, p, count * 0.4, s);

if PointsDistance(pos, target) < 4 then
  begin
    s := PointsDistance(pos, game.player.pos) * 1.5;
    if s < 2 then
      s := 0;

    target.x := game.player.pos.x + game.player.speed.x * s;
    target.y := game.player.pos.y + game.player.speed.y * s;
  end;

{SqrAspire(speed.x, speed_max * cos(angle / 180 * pi) * 0.002, count * 0.002);
SqrAspire(speed.y, speed_max * sin(angle / 180 * pi) * 0.002, count * 0.002);}
speed.x := speed_max * cos(angle / 180 * pi) * 0.002;
speed.y := speed_max * sin(angle / 180 * pi) * 0.002;

pos.x := pos.x + speed.x * count * 0.05;
pos.y := pos.y + speed.y * count * 0.05;

if not dead then
for i := 0 to high(game.bullets) do
  if game.bullets[i] <> nil then
    if (not (game.bullets[i] = self)) and (not game.bullets[i].dead) then
      begin
        s := PointsDistance(pos, game.bullets[i].pos);

        if (s <= (size + game.bullets[i].size) * 0.5) and (not (game.bullets[i] is TRocket2)) then
          begin
            dead := true;
            game.bullets[i].dead := true;
            game.Explosion(pos, radius, power);
            with game.bullets[i] do
              game.Explosion(pos, radius, power, false);
          end
        else
          if s < 0.1 then
            begin
              if CollideCreatures(self, game.bullets[i]) then
              size := Max(size, game.bullets[i].size) + 0.2;
              if size > 1.5 then
                size := 1.5;
              power := size * power_normal;
              particles.AddEmitter(particles_bonus, pos, 10, GetColor(1, 0, 0, 1));
              game.bullets[i].dead := true;
            end;
      end;
end;

///

constructor TRocket3.Create(_power : single; _parent : TCreature);
var
s : single;
begin
speed_max := 80;
inherited Create(_power, _parent);
speed_max := -0.5;

texture := texture_rocket3;

particles_trail := particles.AddEmitter(particles_smoke, pos, 50, GetColor(1, 1, 1, 1));
particles_trail.parent := self;

target := game.player.pos;

{s := PointsDistance(pos, game.player.pos) * 4;
if s > 10 then s := 10;
if s < 2 then s := 0;
target.x := game.player.pos.x + game.player.speed.x * s;
target.y := game.player.pos.y + game.player.speed.y * s;

angle := PointsAngle(target, pos);
NormalizeAngle(angle);
speed.x := 80 * cos(angle / 180 * pi) * 0.002;
speed.y := 80 * sin(angle / 180 * pi) * 0.002;  }

radius := 3;
size := 1;
end;

destructor TRocket3.Destroy;
begin
TEnemy3(parent).active := true;

inherited Destroy;
end;

procedure TRocket3.Update(count : single);
var
p, s : single;
i : integer;
begin
s := PointsDistance(pos, game.player.pos);
if s < (size + game.player.size) * 0.5 then
  begin
    dead := true;
    game.Explosion(pos, radius, power, true, s);
    exit;
  end;

inherited Update(count);
if dead or trapped then exit;

if speed_max < 1.5 then
  if parent <> nil then
    begin
      angle := parent.angle;
      speed.x := 80 * cos(angle / 180 * pi) * 0.002;
      speed.y := 80 * sin(angle / 180 * pi) * 0.002;
    end;

Aspire(speed_max, 2, count * 0.003);
pos.x := pos.x + speed.x * count * 0.05 * speed_max;
pos.y := pos.y + speed.y * count * 0.05 * speed_max;

if not dead then
for i := 0 to high(game.bullets) do
  if game.bullets[i] <> nil then
    if (not (game.bullets[i] = self)) and (not game.bullets[i].dead) then
      if CollideCreatures(self, game.bullets[i]) then
        begin
          dead := true;
          game.bullets[i].dead := true;
          game.Explosion(pos, radius, power);
          with game.bullets[i] do
            game.Explosion(pos, radius, power, false);
        end;
end;

end.
