#ifndef _Core_H_
#define _Core_H_

#include <hge.h>
#include <hgesprite.h>
#include <hgeresource.h>
#include <hgestrings.h>
#include <HGEDebugDraw.h>

#define GAME_GLOBAL_SIZE 10.0f
#define rad2deg 180/3.14
#define deg2rad 3.14/180

class CCore;
class CLevel;

class DestructionListener : public b2DestructionListener
{
public:
	void SayGoodbye(b2Shape* shape) { B2_NOT_USED(shape); }
	void SayGoodbye(b2Joint* joint);

	CCore* core;
};

class BoundaryListener : public b2BoundaryListener	
{
public:
	void Violation(b2Body* body);

	CCore* core;
};

const int32 k_maxContactPoints = 2048;

class ContactListener : public b2ContactListener
{
public:
	void Add(const b2ContactPoint* point);
	void Persist(const b2ContactPoint* point);
	void Remove(const b2ContactPoint* point);

	CCore* core;
};

enum ContactState
{
	e_contactAdded,
	e_contactPersisted,
	e_contactRemoved,
};

struct ContactPoint
{
	b2Shape* shape1;
	b2Shape* shape2;
	b2Vec2 normal;
	b2Vec2 position;
	b2Vec2 velocity;
	b2ContactID id;
	ContactState state;
};

class CLevel;
class CCore {
public:
	CCore(void);
	~CCore(void);

	static CCore *Engine;
	CLevel *Level;
	HGE *hge;
	hgeFont* fnt;
	float GAME_UPDATE_STEP_TIME,GAME_UPDATE_OLD_TIME;

	bool Init(void);

	bool Update(float delta);
	bool Render(void);
	void FocusLost(void);
	void FocusReturn(void);
	static bool StaticUpdate(void);
	static bool StaticRender(void);
	static bool StaticFocusLost(void);
	static bool StaticFocusReturn(void);

	bool Exit(void);
	bool Focus;

	int Width, Height, halfWidth, halfHeight;
	b2AABB worldAABB;
	b2World* myWorld;
	int32 iterations;
	ContactPoint m_points[k_maxContactPoints];
	int32 m_pointCount;
	b2MouseJoint *mouseJoint;
	virtual void JointDestroyed(b2Joint* joint) { B2_NOT_USED(joint); }
	virtual void BoundaryViolated(b2Body* body) { B2_NOT_USED(body); }
protected:
	DestructionListener m_destructionListener;
	BoundaryListener m_boundaryListener;
	ContactListener m_contactListener;

	bool ShowDebug;
	HGEDebugDraw DebugDraw;
	//Flags use to manage Drawing in Box2D
	uint32 flagsDraw;
protected:
	float Timer;
};

#endif