#include <object.h>
#include <core.h>

#include <hge.h>
#include <hgeresource.h>
#include <hgesprite.h>
#include <hgefont.h>
#include <hgeparticle.h>
#include <Box2D.h>

CEnemyFast::CEnemyFast(HGE *Hge ,b2World *MyWorld, CGameObjectManager *MyObjectManager, CGameInfo *MyGameInfo) : CGameObject(Hge,MyWorld,MyObjectManager,MyGameInfo) {}

CEnemyFast::~CEnemyFast(void) {
	delete par;
	delete parExpl;
	delete sprPar;
	delete sprBody;
	hge->Texture_Free(texPar);
	hge->Texture_Free(texBody);
	World->DestroyBody(body);
	hge->Effect_Free(sndExpl);
	World = 0;
	hge=0;
	delete BodyInfo;
}

void CEnemyFast::Add(float _x,float _y,float _speed) {
	Color = 0x00ffffff;
	Alpha = 0xff000000;
	TimerDelay = 5.f;
	Timer = hge->Random_Float(0.f,TimerDelay);
	TimerDamage = 0.f;
	DelayDamage = 0.2f;
	TimerDying = 0.f;
	DelayDying = 0.8f;
	BodyInfo = new CBodyInfo;
	BodyInfo->Team = 4;
	BodyInfo->isCollide = false;
	Dying = false;
	life = 100.f;
	speed = _speed;
	b2BodyDef bodyDef;	
	bodyDef.linearDamping = 1.5f;
	bodyDef.angularDamping = 1.5f;
	bodyDef.position.Set(_x,_y);
	body = World->CreateBody(&bodyDef);
	b2CircleDef ShapeDef;
	ShapeDef.radius = 1.2f;
	ShapeDef.density = 2.0f;
	//ShapeDef.filter.groupIndex = -4;
	ShapeDef.userData = BodyInfo;
	ShapeDef.localPosition.Set(0.f,0.f);
	shape = body->CreateShape(&ShapeDef);
	body->SetMassFromShapes();

	target.Set(hge->Random_Int(0,80),hge->Random_Int(0,60));
	b2Vec2 pos = body->GetPosition();
	pos.operator *=(GAME_GLOBAL_SIZE);
	par->FireAt(pos.x,pos.y);
}

bool CEnemyFast::Init(void) {
	texPar = hge->Texture_Load("data/graphics/particles.png");
	texBody = hge->Texture_Load("data/graphics/enemy_Fast.png");
	sndExpl = hge->Effect_Load("data/sound/enemy_expl_fast.wav");
	if(!texPar || !texBody || !sndExpl)
	{
		MessageBox(NULL, "Can't load #1", "Error", MB_OK | MB_ICONERROR | MB_APPLMODAL);
		hge->System_Shutdown();
		hge->Release();
		return false;
	}

	sprBody=new hgeSprite(texBody, 0.f, 0.f, hge->Texture_GetWidth(texBody), hge->Texture_GetHeight(texBody));
	sprBody->SetHotSpot(hge->Texture_GetWidth(texBody)/2.f,hge->Texture_GetHeight(texBody)/2.f);

	sprPar=new hgeSprite(texPar, 96, 64, 32, 32);
	sprPar->SetBlendMode(BLEND_COLORMUL | BLEND_ALPHAADD | BLEND_NOZWRITE);
	sprPar->SetHotSpot(16,16);
	sprParExpl=new hgeSprite(texPar, 0, 0, 32, 32);
	sprParExpl->SetBlendMode(BLEND_COLORMUL | BLEND_ALPHAADD | BLEND_NOZWRITE);
	sprParExpl->SetHotSpot(16,16);
	par=new hgeParticleSystem("data/particle/particle_go_fast.psi",sprPar);
	parExpl = new hgeParticleSystem("data/particle/particle_expl_fast.psi",sprParExpl);
	parHit = new hgeParticleSystem("data/particle/particle_collision.psi",sprParExpl);

	return true;
}

void CEnemyFast::Update(float delta) {
	b2Vec2 pos = body->GetPosition();
	pos.operator *=(GAME_GLOBAL_SIZE);
	//par->info.nEmission=(int)(dx*dx+dy*dy)*2;
	par->MoveTo(pos.x,pos.y);
	par->Update(delta);
	if (parExpl) parExpl->MoveTo(pos.x,pos.y);
	parExpl->Update(delta);
	if (parHit) parHit->MoveTo(pos.x,pos.y);
	parHit->Update(delta);

	if (Dying&&isAlive)
	{
		TimerDying += delta;
		if (TimerDying > DelayDying)
		{
			isAlive = false;
			int j = hge->Random_Int(0,1);
			CGameObject *obj;
			ObjectManager->AddObject(obj = new CEnemyFast(hge,World,ObjectManager,GameInfo));
			if (j==0) obj->Add(hge->Random_Float(-20.f,-10.f), hge->Random_Float(0.f,60.f), speed);
			else obj->Add(hge->Random_Float(90.f,100.f), hge->Random_Float(0.f,60.f), speed);
			obj->TimerDelay = TimerDelay;
		}
		if (Alpha > 0x05000000) Alpha -= 0x05000000;
	} 
	else
	{	
		pos = body->GetPosition();
		b2Vec2 f((target.x-pos.x)*speed,(target.y-pos.y)*speed);
		b2Vec2 lp(0.f,-1.6f);
		b2Vec2 p = body->GetWorldPoint(lp);	
		body->ApplyForce(f,p);

		Timer += delta;
		if (Timer>=TimerDelay) 
		{
			Timer = 0.f;
			CGameObject *obj;
			ObjectManager->AddObject(obj = new CRocketFunny(hge,World,ObjectManager,GameInfo));
			float ang = body->GetAngle()-90.f;
			obj->Add(pos.x+cos(ang)*3.2f, pos.y+sin(ang)*3.2f, 4.0f);
			if (TimerDelay > 0.5f) TimerDelay -= 0.1f;
		}

		pos = body->GetPosition();
		if (sqrtf((pos.x - target.x) * (pos.x - target.x) + (pos.y - target.y) * (pos.y - target.y)) < 5.f)
		{
			target.Set(hge->Random_Int(0,80),hge->Random_Int(0,60));
		}
		
		TimerDamage += delta;
		if (TimerDamage > DelayDamage)
		{
			if (BodyInfo->isCollide)
			{
				TimerDamage = 0.f;
				life -= 50.f;
				if (life <= 0.f) Boom();
				BodyInfo->isCollide = false;
				b2Vec2 pos = body->GetPosition();
				pos.operator *=(GAME_GLOBAL_SIZE);
				parHit->FireAt(pos.x,pos.y);
			}
		}else BodyInfo->isCollide = false;
	}
}

void CEnemyFast::Render(void) {
	par->Render();

	b2Vec2 pos = body->GetPosition();
	float32 ang = body->GetAngle();
	pos.operator *=(GAME_GLOBAL_SIZE);
	sprBody->SetColor(0x00555555+Alpha);
	sprBody->RenderEx(pos.x+3,pos.y+3,ang,0.35f);
	sprBody->SetColor(Color+Alpha);
	sprBody->RenderEx(pos.x,pos.y,ang,0.35f);

	parHit->Render();
	parExpl->Render();
}

void CEnemyFast::Boom(void) {
	if (Dying == false)
	{
		b2Vec2 pos = body->GetPosition();
		pos.operator *=(GAME_GLOBAL_SIZE);
		parExpl->FireAt(pos.x,pos.y);
		Dying = true;
		GameInfo->Score += 70;
		hge->Effect_Play(sndExpl);
	}
}