#ifndef _BALL_H_
#define _BALL_H_

#include <hge.h>
#include <Box2D.h>
#include <core.h>

#define LEVELOBJECT_MAX 500

class CGameObjectManager;
class CGameObject;

class CGameInfo {
public:
	CGameInfo(void);
	bool isGameOver;
	int Score;
};

class CBodyInfo {
public:
	int Team;
	bool isCollide;
};

class CGameObject {
public:
	CGameObject(HGE *Hge, b2World *MyWorld, CGameObjectManager *MyObjectManager, CGameInfo *MyGameInfo);
	virtual ~CGameObject(void);

	hgeSprite			*sprPar,*sprBody;
	hgeParticleSystem	*par;
	HTEXTURE			texPar,texBody;

	float speed;
	float angle;
	bool isAlive;
	float life;

	b2Shape		*shape;
	b2Body		*body,*bodyHead,*bodyEnd;
	b2Joint		*jointBase;

	CBodyInfo *BodyInfo;
	CGameInfo *GameInfo;

	DWORD		Color,Alpha;

	float Timer,TimerDelay;

	virtual bool Init(void) { return true; };
	virtual void Update(float delta) {};
	virtual void Render(void) {};
	virtual void Add(float _x,float _y,float _speed) {};
	virtual void Joing(b2Body* b1,b2Body* b2,b2Vec2 anchor1,b2Vec2 anchor2);
protected:
	HGE *hge;
	b2World *World;
	CGameObjectManager *ObjectManager;
};

class CGameObjectManager {
public:
	CGameObjectManager(CCore *MyEngine, HGE *Hge, b2World *MyWorld);
	~CGameObjectManager(void);

	int Count;

	bool Init(void);
	void Update(float delta);
	void Render(void);

	int AddObject(CGameObject *obj);
	void DeleteAllObject(void);
	void DeleteDeadObject(void);
	void Remove(CGameObject *obj);
	bool Load(void);
	//void ContactPointDelete(b2Body* body);
protected:
	CCore *Engine;
	HGE *hge;
	b2World *World;
	CGameObject *Objects[LEVELOBJECT_MAX];
};

class CPlayer : public CGameObject {
public:
	CPlayer(HGE *Hge, b2World *MyWorld, CGameObjectManager *MyObjectManager, CGameInfo *MyGameInfo);
	~CPlayer(void);

	HEFFECT				sndCollision;
	hgeParticleSystem	*parHit;
	hgeFont				*fnt;

	float TimerDamage, DelayDamage;
	bool Init(void);
	void Update(float delta);
	void Render(void);
};

class CHandle : public CGameObject {
public:
	CHandle(HGE *Hge, b2World *MyWorld, CGameObjectManager *MyObjectManager, CGameInfo *MyGameInfo);
	~CHandle(void);

	HEFFECT		sndSelect;
	
	b2Shape		*shapeHandle[8];
	b2Body		*bodyHandle[8];
	b2Joint		*joint[7];
	bool Init(void);
	void Update(float delta);
	void Render(void);
};

class CMace : public CGameObject {
public:
	CMace(HGE *Hge, b2World *MyWorld, CGameObjectManager *MyObjectManager, CGameInfo *MyGameInfo);
	~CMace(void);

	HEFFECT		sndCollision;
	HCHANNEL	sndChannelCollision;

	bool Init(void);
	void Update(float delta);
	void Render(void);
};

class CRocket : public CGameObject {
public:
	CRocket(HGE *Hge, b2World *MyWorld, CGameObjectManager *MyObjectManager, CGameInfo *MyGameInfo);
	~CRocket(void);

	b2Body *bodyTarget,*body2;
	hgeParticleSystem	*parExpl;
	bool Dying;
	HEFFECT		sndExpl;

	hgeSprite			*sprBody2;
	HTEXTURE			texBody2;
	
	bool Init(void);
	void Update(float delta);
	void Render(void);
	void Boom(float delta);
	void Add(float _x,float _y,float _speed);
};

class CRocketFunny : public CGameObject {
public:
	CRocketFunny(HGE *Hge, b2World *MyWorld, CGameObjectManager *MyObjectManager, CGameInfo *MyGameInfo);
	~CRocketFunny(void);

	b2Body *bodyTarget;
	hgeParticleSystem	*parExpl;
	bool Dying;
	HEFFECT		sndExpl;
	
	bool Init(void);
	void Update(float delta);
	void Render(void);
	void Boom(float delta);
	void Add(float _x,float _y,float _speed);
};

class CEnemyHeavy : public CGameObject {
public:
	CEnemyHeavy(HGE *Hge, b2World *MyWorld, CGameObjectManager *MyObjectManager, CGameInfo *MyGameInfo);
	~CEnemyHeavy(void);

	hgeSprite			*sprParExpl;
	hgeParticleSystem	*parExpl,*parHit;
	b2Vec2				target;
	float				TimerDamage, DelayDamage, TimerDying, DelayDying;
	bool				Dying;
	HEFFECT				sndExpl;
	
	b2Shape *shapeShield[2];
	b2Body *bodyShield[2];
	b2Joint *joint[2];
	bool Init(void);
	void Update(float delta);
	void Render(void);
	void Add(float _x,float _y,float _speed);
	void Boom(void);
};

class CEnemyFast : public CGameObject {
public:
	CEnemyFast(HGE *Hge, b2World *MyWorld, CGameObjectManager *MyObjectManager, CGameInfo *MyGameInfo);
	~CEnemyFast(void);

	hgeSprite			*sprParExpl;
	hgeParticleSystem	*parExpl,*parHit;
	b2Vec2				target;
	float				TimerDamage, DelayDamage, TimerDying, DelayDying;
	bool				Dying;
	HEFFECT				sndExpl;
	
	b2Shape *shapeShield[2];
	b2Body *bodyShield[2];
	b2Joint *joint[2];
	bool Init(void);
	void Update(float delta);
	void Render(void);
	void Add(float _x,float _y,float _speed);
	void Boom(void);
};

#endif