#include <object.h>
#include <core.h>

#include <hge.h>
#include <hgeresource.h>
#include <hgesprite.h>
#include <hgefont.h>
#include <hgeparticle.h>
#include <Box2D.h>

CRocketFunny::CRocketFunny(HGE *Hge ,b2World *MyWorld, CGameObjectManager *MyObjectManager, CGameInfo *MyGameInfo) : CGameObject(Hge,MyWorld,MyObjectManager,MyGameInfo) {}

void CRocketFunny::Add(float _x,float _y,float _speed) {
	Color = 0x00ffffff;
	Alpha = 0xff000000;
	Timer = 0.f;
	TimerDelay = 4.f;
	speed = _speed;
	BodyInfo = new CBodyInfo;
	BodyInfo->Team = 3;
	BodyInfo->isCollide = false;
	Dying = false;

	b2BodyDef bodyDef;	
	bodyDef.linearDamping = 1.5f;
	bodyDef.angularDamping = 1.5f;
	bodyDef.position.Set(_x,_y);
	body = World->CreateBody(&bodyDef);
	b2CircleDef ShapeDef;
	ShapeDef.radius = 0.8f;
	ShapeDef.density = 1.5f;
	ShapeDef.filter.groupIndex = -4;
	ShapeDef.userData = BodyInfo;
	ShapeDef.localPosition.Set(0.f,0.f);
	shape = body->CreateShape(&ShapeDef);
	body->SetMassFromShapes();

	bodyTarget = NULL;
	angle = 0;
	b2Vec2 pos = body->GetPosition();
	pos.operator *=(GAME_GLOBAL_SIZE);
	par->FireAt(pos.x,pos.y);
}

CRocketFunny::~CRocketFunny(void){
	bodyTarget = 0;
	delete par;
	delete sprPar;
	delete sprBody;
	hge->Texture_Free(texPar);
	hge->Texture_Free(texBody);
	hge->Effect_Free(sndExpl);
	World->DestroyBody(body);
	delete BodyInfo; 
}

bool CRocketFunny::Init(void) {
	texPar = hge->Texture_Load("data/graphics/particles.png");
	texBody = hge->Texture_Load("data/graphics/RocketFunny.png");
	sndExpl = hge->Effect_Load("data/sound/expl.wav");
	if(!texPar || !texBody || !sndExpl)
	{
		MessageBox(NULL, "Can't load #1", "Error", MB_OK | MB_ICONERROR | MB_APPLMODAL);
		hge->System_Shutdown();
		hge->Release();
		return false;
	}

	sprBody=new hgeSprite(texBody, 0.f, 0.f, hge->Texture_GetWidth(texBody), hge->Texture_GetHeight(texBody));
	sprBody->SetHotSpot(hge->Texture_GetWidth(texBody)/2.f,hge->Texture_GetHeight(texBody)/2.f);

	sprPar=new hgeSprite(texPar, 96, 64, 32, 32);
	sprPar->SetBlendMode(BLEND_COLORMUL | BLEND_ALPHAADD | BLEND_NOZWRITE);
	sprPar->SetHotSpot(16,16);
	par = new hgeParticleSystem("data/particle/particle_go_rocketfunny.psi",sprPar);
	parExpl = new hgeParticleSystem("data/particle/particle_expl_rocketfunny.psi",sprPar);

	return true;
}

float AngleTo2(b2Vec2 pos, b2Vec2 tpos) {
	if(pos.x == tpos.x && pos.y == tpos.y) return -1.f;

	float angle = atanf((tpos.y - pos.y) / (tpos.x - pos.x)) * 180 / 3.14f;
	if(tpos.x >= pos.x)
		return 180.f - angle + 90.f;
	else
		return 180.f - angle - 90.f;		

	if(tpos.x >= pos.x)
		return angle + 90.f;
	else
		return angle - 90.f;
}

void CRocketFunny::Update(float delta) {
	b2Vec2 pos = body->GetPosition();
	pos.operator *=(GAME_GLOBAL_SIZE);
	//par->info.nEmission=(int)(dx*dx+dy*dy)*2;
	par->MoveTo(pos.x,pos.y);
	par->Update(delta);
	if (parExpl) parExpl->MoveTo(pos.x,pos.y);
	parExpl->Update(delta);

	if (Dying == false)
	{
		if (bodyTarget==NULL)
		{
			for (b2Body* b = World->GetBodyList(); b; b = b->GetNext())
			{	
				for (b2Shape* s = b->GetShapeList(); s; s = s->GetNext())
				{
					CBodyInfo *tmp = (CBodyInfo*)s->GetUserData();
					if (tmp && tmp->Team==0) 
					{
						bodyTarget = b;
						break;
					}
				}
			}
		}

		if (bodyTarget)
		{			
			pos = body->GetPosition();
			b2Vec2 tpos = bodyTarget->GetPosition();
			angle = -AngleTo2(pos, tpos);
			b2Vec2 f(cos(angle)*speed,sin(angle)*speed);
			body->ApplyImpulse(f,pos);
		}
	}

	if (BodyInfo->isCollide)
	{
		Timer += delta;
		if (Timer > 0.15f) Boom(delta);
	}
}

void CRocketFunny::Render(void) {
	par->Render();

	b2Vec2 pos = body->GetPosition();
	float32 ang = body->GetAngle();
	pos.operator *=(GAME_GLOBAL_SIZE);
	sprBody->SetColor(0x00555555+Alpha);
	sprBody->RenderEx(pos.x+3,pos.y+3,(angle+90)*deg2rad,0.5f);
	sprBody->SetColor(Color+Alpha);
	sprBody->RenderEx(pos.x,pos.y,(angle+90)*deg2rad,0.5f);

	parExpl->Render();
}

void CRocketFunny::Boom(float delta) {
	if (Dying == false)
	{
		b2Vec2 pos = body->GetPosition();
		pos.operator *=(GAME_GLOBAL_SIZE);
		parExpl->FireAt(pos.x,pos.y);
		Dying = true;
		GameInfo->Score += 10;
		hge->Effect_Play(sndExpl);
	}
	if (Alpha > 0x05000000) Alpha -= 0x05000000;
	Timer += delta;
	if (Timer > 1.2f)
	{
		isAlive = false;
	}
}