#ifndef __WORLD_H__
#define __WORLD_H__

#include "Player.h"
#include "Enemy.h"
#include "ParticlesEffects.h"
#include "../Utils/Camera.h"
#include "../Utils/Font.h"


struct WaveParam
{
	int type;
	int time;
	int count;
};

class World
{
public:
	World()
	{
		player.p = vec2 ( -100.0f, -100.0f );
		useSlowMotion = false;
		ellapsedTime = 0.0f;
		slowTime = 10.0f;
		score = 0.0f;
	}
	~World()
	{
		clear ();
	}

	void clear ()
	{
		clearEnemys ();
		clearRockets ();
		clearBullets ();
		clearEffects ();
		useSlowMotion = false;
		ellapsedTime = 0.0f;
		slowTime = 10.0f;
		score = 0.0f;
		player.health = 100.0f;
		player.v = vec2 ( 0.0f );
		player.p = vec2 ( ( float ) WINDOW_WIDTH * 0.5f, ( float ) WINDOW_HEIGHT * 0.5f );
	}

	void clearEnemys ()
	{
		for ( unsigned int i = 0; i < enemys.getCount (); i++ )
		{
			Enemy * forDelete = enemys [i];
			delete forDelete;
		}
		enemys.clear ();
	}

	void clearRockets ()
	{
		for ( unsigned int i = 0; i < rockets.getCount (); i++ )
		{
			Rocket * forDelete = rockets [i];
			delete forDelete;
		}
		for ( unsigned int i = 0; i < bigRockets.getCount (); i++ )
		{
			RocketBig * forDelete = bigRockets [i];
			delete forDelete;
		}

		rockets.clear ();
		bigRockets.clear ();
	}

	void clearBullets ()
	{
		for ( unsigned int i = 0; i < bullets.getCount (); i++ )
		{
			Bullet * forDelete = bullets [i];
			delete forDelete;
		}
		bullets.clear ();
	}

	void clearEffects ()
	{
		particlesEngine.clear ();
	}

	String getFormatTime ()
	{
		String str;
		int m, s, h;
		int t = ( int ) ellapsedTime;
		h = t / 3600;
		t -= h * 3600;
		m = t / 60;
		t -= m * 60;
		s = t;
		
		if ( h < 10 ) str.sprintf ( "0%d:", h ); else str.sprintf ( "%d:", h );
		if ( m < 10 ) str.sprintf ( "0%d:", m ); else str.sprintf ( "%d:", m );
		if ( s < 10 ) str.sprintf ( "0%d", s );	else str.sprintf ( "%d", s );
		return str;
	}

	String getFormatScore ()
	{
		String str ( "Score: " );
		str.appendInt ( ( int )score );
		return str;
	}

	vec2 getPointOutside ( float offset )
	{
		vec2 pt;
		int r = ( int ) frand ( 4 );
		switch ( r )
		{
		case 0:						// left
			pt.x = -offset;
			pt.y = frand ( ( float ) WINDOW_HEIGHT );
			break;
		case 1:						// top
			pt.x = frand ( ( float ) WINDOW_WIDTH );
			pt.y = -offset;
			break;
		case 2:						// right
			pt.x = ( float ) WINDOW_WIDTH + offset;
			pt.y = frand ( ( float ) WINDOW_HEIGHT );
			break;
		case 3:						// bottom
			pt.x = frand ( ( float ) WINDOW_WIDTH );
			pt.y = ( float ) WINDOW_HEIGHT + offset;
			break;
		}
		return pt;
	}


	void updateWaves ();
	float updateSlowMo ( float dt );
	void update ( float dt );				//  
	void render();							// 
	void drawBoardText ();

	void resolveBullets ( int bulletType, Shell * shell );

	void destroyEnemy ( int index );
	void destroyRocket ( int index );
	void destroyBigRocket ( int index );
	void destroyBullet ( int index );

	int findFreeEnemy ( int type );			//     
	int findFreeRocket ();
	int findFreeBigRocket ();
	int findFreeBullet ();

	int addEnemyLaser ( vec2 & pos );
	int addEnemyRocket ( vec2 & pos );
	int addEnemy3 ( vec2 & pos );
	int addEnemyShock ( vec2 & pos );
	int addEnemyBigRocket ( vec2 & pos );
	int addRocket ( Enemy * owner, vec2 & pos, vec2 & vel );
	int addBigRocket ( Enemy * owner, vec2 & pos, vec2 & vel );
	int addBullet ( Enemy * owner, vec2 & pos, vec2 & vel );

	void loadLevel ( const char * fileName );

public:
	Player player;
	Array < Enemy* > enemys;
	Array < Rocket* > rockets;
	Array < RocketBig* > bigRockets;
	Array < Bullet* > bullets;

	float ellapsedTime;
	float score;
	Array < WaveParam > waves;

	float slowTime;					//  
	float slowTransitionTime;		//  
	bool useSlowMotion;
};

extern World world;

#endif

