#include "Tokenizer.h"

#include "Data.h"
#include <stdio.h>


bool isWhiteSpace ( const char ch )
{
	return ( ch == ' ' || ch == '\t' || ch == '\r' || ch == '\n' );
}

bool isNumeric ( const char ch )
{
	return ( ch >= '0' && ch <= '9' );
}

bool isAlphabetical ( const char ch )
{
	return ( ( ch >= 'a' && ch <= 'z' ) || ( ch >= 'A' && ch <= 'Z' ) || ch == '_' );
}

bool isNewLine ( const char ch )
{
	return ( ch == '\r' || ch == '\n' );
}


Tokenizer :: Tokenizer ( unsigned int nBuffers )
{
	str = NULL;
	
	buffers.setCount ( nBuffers );
	memset ( buffers.getArray (), 0, nBuffers * sizeof ( TokBuffer ) );

	reset ();
}

Tokenizer :: ~Tokenizer ()
{
	for ( unsigned int i = 0; i < buffers.getCount (); i++ )
		if ( buffers[i].buffer )
			delete buffers [i].buffer;

	delete str;
}

void Tokenizer :: setString ( const char *string )
{
	delete str;

	length = ( unsigned int ) strlen ( string );

	str = new char [length + 1];
	strcpy ( str, string );

	reset ();
}

bool Tokenizer :: setFile ( const char *fileName )
{
	delete str;

	Data *file = new Data ( fileName );
	if ( file->isOk () )
	{
		length = file->getLength ();
		str = new char [length + 1];
		file->getBytes ( str, length );
		str[length] = '\0';
		reset ();

		file->clear ();
		delete file;

		return true;
	}

	delete file;

	return false;
}

void Tokenizer :: reset ()
{
	end = 0;
}

bool Tokenizer :: goToNext ( BOOLFUNC isAlpha )
{
	start = end;

	while ( start < length && isWhiteSpace ( str [start] ) )
		start++;

	end = start + 1;
	
	if ( start < length )
	{										// check number
		if ( isNumeric ( str [start] ) )
		{
			while ( isNumeric ( str [end] ) || str [end] == '.' )
				end++;
		}
		else
		{									// check other symbol
			if ( isAlpha ( str [start] ) )
			{
				while ( isAlpha ( str [end] ) || isNumeric ( str [end] ) )
					end++;
			}
		}

		return true;
	}

	return false;
}

bool Tokenizer :: goToNextLine ()
{
	if ( end < length )
	{
		start = end;

		while ( end < length && !isNewLine ( str [end] ) )
			end++;

		if ( isNewLine ( str [end + 1] ) && str [end] != str [end + 1] )
			end += 2;
		else
			end++;

		return true;
	}

	return false;
}


char *Tokenizer :: next ( BOOLFUNC isAlpha )
{
	if ( goToNext ( isAlpha ) )
	{
		unsigned int size = end - start;
		char *buffer = getBuffer ( size + 1 );
		strncpy ( buffer, str + start, size );
		buffer [size] = '\0';

		return buffer;
	}

	return NULL;
}

char *Tokenizer :: nextAfterToken ( const char *token, BOOLFUNC isAlpha )
{
	while ( goToNext ( isAlpha ) )
		if ( strncmp ( str + start, token, end - start ) == 0 )
			return next ();

	return NULL;
}

char *Tokenizer :: nextLine ()
{
	if ( goToNextLine () )
	{
		unsigned int size = end - start;
		char *buffer = getBuffer ( size + 1 );
		strncpy ( buffer, str + start, size );
		buffer [size] = '\0';

		return buffer;
	}

	return NULL;
}

char *Tokenizer :: getBuffer ( unsigned int size )
{
	currentBuffer++;
	if ( currentBuffer >= buffers.getCount () )
		currentBuffer = 0;

	if ( size > buffers [currentBuffer].bufferSize )
	{
		delete buffers [currentBuffer].buffer;
		buffers [currentBuffer].buffer = new char [buffers[currentBuffer].bufferSize = size];
	}

	return buffers [currentBuffer].buffer;
}
