{==============================}
{= Ogg Vorbis player class    =}
{= with DirectSound output... =}
{==============================}
{= by XProger                 =}
{= mail: XProger@list.ru      =}
{= http://xproger.mirgames.ru =}
{==============================}
unit ogg;

interface

uses
  Windows;

const
  OGG_BUFFER_SIZE = 64 * 1024;
  OGG_FILL_TIME   = 100;
  
type
  TOGG = class
    constructor Create(const FileName: string = '');
    destructor Destroy; override;
   private
    FReady   : Boolean;
    FLoop    : Boolean;
    FDone    : Boolean;
    FAllDone : Boolean;
    Stream   : Pointer;
    OggFile  : Pointer;
    Buffer   : IUnknown;
    Thread   : LongWord;
    LastPos  : LongWord;
    function GetVolume: Integer;
    procedure SetVolume(Value: Integer);
    procedure Update;
   public
    procedure Play(Loop: Boolean);
    procedure Pause;
    procedure Stop;
    function Playing: Boolean;
    property Ready: Boolean read FReady;
    property Volume: Integer read GetVolume write SetVolume;
  end;

  function OggInit(Handle: Cardinal): Boolean;

implementation

type
  TStream = class
    constructor Create(const FileName: string);
    destructor Destroy; override;
   private
    F : Cardinal;
   public
    procedure Seek(Pos: Integer);
    function Valid: Boolean;
    function Read(var Buf; Count: Integer): Cardinal;
    function Size: Integer;
    function Pos: Integer;
  end;

//=====> DIRECT SOUND HEADER <=====//
const
  DS_OK                         = $00000000;
  DSSCL_PRIORITY                = $00000002;
  DSBPLAY_LOOPING               = $00000001;
  DSBVOLUME_MIN                 = -10000;
  DSBSTATUS_PLAYING             = $00000001;
  DSBSTATUS_BUFFERLOST          = $00000002;
  DSBCAPS_GETCURRENTPOSITION2   = $00010000;
  DSBCAPS_CTRLVOLUME            = $00000080;
  DSBCAPS_GLOBALFOCUS           = $00008000;

type
  IDirectSoundBuffer = interface;
  IDirectSound = interface;

  PIDirectSound = ^IDirectSound;

  IDirectSound = interface (IUnknown)
    ['{279AFA83-4981-11CE-A521-0020AF0BE560}']
    function CreateSoundBuffer(lpDSBufferDesc: Pointer;
        out lpIDirectSoundBuffer: IDirectSoundBuffer;
        pUnkOuter: IUnknown) : Integer; stdcall;
    function GetCaps(lpDSCaps: Pointer) : Integer; stdcall;
    function DuplicateSoundBuffer(lpDsbOriginal: IDirectSoundBuffer;
        out lpDsbDuplicate: IDirectSoundBuffer) : Integer; stdcall;
    function SetCooperativeLevel(hwnd: Cardinal; dwLevel: Cardinal) : Integer; stdcall;
    function Compact: Integer; stdcall;
    function GetSpeakerConfig(var lpdwSpeakerConfig: Cardinal) : Integer; stdcall;
    function SetSpeakerConfig(dwSpeakerConfig: Cardinal) : Integer; stdcall;
    function Initialize(lpGuid: PGUID) : Integer; stdcall;
  end;

  IDirectSoundBuffer = interface (IUnknown)
    ['{279AFA85-4981-11CE-A521-0020AF0BE560}']
    function GetCaps(lpDSCaps: Pointer) : Integer; stdcall;
    function GetCurrentPosition
        (var lpdwPlayPosition, lpdwReadPosition : Cardinal) : Integer; stdcall;
    function GetFormat(lpwfxFormat: Pointer; dwSizeAllocated: Cardinal;
        var lpdwSizeWritten: Cardinal) : Integer; stdcall;
    function GetVolume(var lplVolume: integer) : Integer; stdcall;
    function GetPan(var lplPan: integer) : Integer; stdcall;
    function GetFrequency(var lpdwFrequency: Cardinal) : Integer; stdcall;
    function GetStatus(var lpdwStatus: Cardinal) : Integer; stdcall;
    function Initialize(lpDirectSound: IDirectSound;
        lpcDSBufferDesc: Pointer) : Integer; stdcall;
    function Lock(dwWriteCursor, dwWriteBytes: Cardinal;
        var lplpvAudioPtr1: Pointer; var lpdwAudioBytes1: Cardinal;
        var lplpvAudioPtr2: Pointer; var lpdwAudioBytes2: Cardinal;
        dwFlags: Cardinal) : Integer; stdcall;
    function Play(dwReserved1,dwReserved2,dwFlags: Cardinal) : Integer; stdcall;
    function SetCurrentPosition(dwPosition: Cardinal) : Integer; stdcall;
    function SetFormat(lpcfxFormat: Pointer) : Integer; stdcall;
    function SetVolume(lVolume: integer) : Integer; stdcall;
    function SetPan(lPan: integer) : Integer; stdcall;
    function SetFrequency(dwFrequency: Cardinal) : Integer; stdcall;
    function Stop: Integer; stdcall;
    function Unlock(lpvAudioPtr1: Pointer; dwAudioBytes1: Cardinal;
        lpvAudioPtr2: Pointer; dwAudioBytes2: Cardinal) : Integer; stdcall;
    function Restore: Integer; stdcall;
  end;

//=====> OGG VORBIS TYPES HEADER <=====//
const
  VorbisFileLib = 'vorbisfile.dll';

type
  p_vorbis_info = ^vorbis_info;
  vorbis_info = record
    version         : Integer;
    channels        : Integer;
    rate            : Integer;
    bitrate_upper   : Integer;
    bitrate_nominal : Integer;
    bitrate_lower   : Integer;
    bitrate_window  : Integer;
    codec_setup     : Pointer;
  end;

  OggVorbis_File = record
    some_data1 : array [0..17] of Integer;
    vi         : p_vorbis_info;
    some_data2 : array [0..160] of Integer;
  end;

  ov_callbacks = record
    read_func  : function(var ptr; size: Cardinal; nmemb: Cardinal; const datasource): Cardinal; cdecl;
    seek_func  : function(const datasource; offset: Int64; whence: Integer): Integer; cdecl;
    close_func : function(const datasource): Integer; cdecl;
    tell_func  : function(const datasource): Integer; cdecl;
  end;

var
  ops_callbacks : ov_callbacks;

  function ov_clear(OggFile: Pointer): Integer; cdecl; external VorbisfileLib;
  function ov_open_callbacks(const datasource; OggFile: Pointer; initial: PChar; ibytes: Integer; callbacks: ov_callbacks): Integer; cdecl; external VorbisFileLib;
  function ov_pcm_seek(OggFile: Pointer; pos: Int64): Integer; cdecl; external VorbisFileLib;
  function ov_read(OggFile: Pointer; const buffer; length: Integer; bigendianp: Integer; word: Integer; sgned: Integer; bitstream: Pointer): Integer; cdecl; external VorbisFileLib;
//=====================================//

function ops_read_func(var ptr; size, nmemb: Cardinal; const datasource): Cardinal; cdecl;
begin
  if (size = 0) or (nmemb = 0) then
  begin
    Result := 0;
    Exit;
  end;

  try
    Result := Int64(TStream(datasource).Read(ptr, size * nmemb)) div Int64(size);
  except
    Result := 0;
  end;
end;

function ops_seek_func(const datasource; offset: Int64; whence: Integer): Integer; cdecl;
begin
  try
    with TStream(datasource) do
      case whence of
        0: Seek(offset);
        1: Seek(Pos + offset);
        2: Seek(Size - offset);
      end;
    Result := 0;
  except
    Result := -1;
  end;
end;

function ops_close_func(const datasource): Integer; cdecl;
begin
  Result := 0;
end;

function ops_tell_func(const datasource): Integer; cdecl;
begin
  try
    Result := TStream(datasource).Pos;
  except
    Result := -1;
  end;
end;

var
  DSoundDLL         : HMODULE;
  DSMain            : IDirectSound;
  DSready           : Boolean = False;
  DirectSoundCreate : function (lpGuid: PGUID; out ppDS: IDirectSound; pUnkOuter: IUnknown): Integer; stdcall;

function OggInit(Handle: Cardinal): Boolean;
begin
  if not DSReady then
    if (@DirectSoundCreate <> nil) and
       (DirectSoundCreate(nil, DSMain, nil) = DS_OK) and
       (DSMain.SetCooperativeLevel(Handle, DSSCL_PRIORITY) = DS_OK) then
      DSReady  := True;
  Result := DSReady;
end;

function OggThreadProc(OGG: TOGG): Integer; stdcall;
begin
  Result := 0;
  try
    IDirectSoundBuffer(OGG.Buffer).Play(0, 0, DSBPLAY_LOOPING);
   	while (OGG.Buffer <> nil) and OGG.Playing do
	  begin
      OGG.Update;
      Sleep(OGG_FILL_TIME);
    end;
  except
    // :)
  end;
end;

//===== File Stream =====>
constructor TStream.Create(const FileName: string);
begin
  F := CreateFile(PChar(FileName), GENERIC_READ, FILE_SHARE_READ, nil, OPEN_EXISTING, 0, 0);
end;

destructor TStream.Destroy;
begin
  if Valid then
  begin
    CloseHandle(F);
    F := INVALID_HANDLE_VALUE;
  end;
end;

function TStream.Valid: Boolean;
begin
  Result := F <> INVALID_HANDLE_VALUE;
end;

procedure TStream.Seek(Pos: Integer);
begin
  SetFilePointer(F, Pos, nil, FILE_BEGIN);
end;

function TStream.Read(var Buf; Count: Integer): Cardinal;
begin
  try
    ReadFile(F, Buf, Count, Result, nil);
  except
    Result := 0;
  end;
end;

function TStream.Size: Integer;
begin
  Result := GetFileSize(F, nil);
end;

function TStream.Pos: Integer;
begin
  Result := SetFilePointer(F, 0, nil, FILE_CURRENT);
end;

//===== OGG Vorbis =====>
constructor TOGG.Create;
var
  BufferDesc : packed record
    dwSize          : Cardinal;
    dwFlags         : Cardinal;
    dwBufferBytes   : Cardinal;
    dwReserved      : Cardinal;
    lpwfxFormat     : Pointer;
    guid3DAlgorithm : TGUID;
  end;

  Format     : packed record
                 wFormatTag      : Word;
                 nChannels       : Word;
                 nSamplesPerSec  : LongWord;
                 nAvgBytesPerSec : LongWord;
                 nBlockAlign     : Word;
                 wBitsPerSample  : Word;
                 cbSize          : Word;
               end;
label
  ext;
begin
  if (not DSReady) or (FileName = '') then Exit;
// getting binary data from file
  Stream := TStream.Create(FileName);
  if not TStream(Stream).Valid then
    goto ext;

  GetMem(OggFile, SizeOf(OggVorbis_File));

// reading hearer
  if ov_open_callbacks(Stream, OggFile, nil, 0, ops_callbacks) < 0 then
    goto ext;

// fill DSound PCM format structures
  with Format do
  begin
    wFormatTag      := 1;
    nChannels       := OggVorbis_File(OggFile^).vi^.channels;
    nSamplesPerSec  := OggVorbis_File(OggFile^).vi^.rate;
    wBitsPerSample  := 16;
    nBlockAlign     := (wBitsPerSample div 8) * nChannels;
    nAvgBytesPerSec := nSamplesPerSec * nBlockAlign;
    cbSize          := SizeOf(Format);
  end;

// DSound buffer description
  ZeroMemory(@BufferDesc, SizeOf(BufferDesc));
  with BufferDesc do
  begin
    dwSize        := SizeOf(BufferDesc);
    dwFlags       := DSBCAPS_GETCURRENTPOSITION2 or
                     DSBCAPS_CTRLVOLUME or
                     DSBCAPS_GLOBALFOCUS; 
    dwBufferBytes := OGG_BUFFER_SIZE;
    lpwfxFormat   := @Format.wFormatTag;
  end;

// Create DirectSound buffer
  if DSMain.CreateSoundBuffer(@BufferDesc, IDirectSoundBuffer(Buffer), nil) <> DS_OK then
    goto ext;

  Volume := 100;
  FReady := True;
  Exit;
ext:
  TStream(Stream).Free;
  FreeMem(OggFile);
end;

destructor TOGG.Destroy;
begin
  if not Ready then Exit;
  Stop;
  if Buffer <> nil then
    Buffer := nil;
  TStream(Stream).Free;
  ov_clear(OggFile);
  FreeMem(OggFile);
  inherited;
end;

function TOGG.GetVolume: Integer;
begin
  Result := 0;
  if not Ready then Exit;
  IDirectSoundBuffer(Buffer).GetVolume(Result);
  Result := Trunc(100 * exp(ln(10) * (Result / 33.22 / 100)));
end;

procedure TOGG.SetVolume(Value: Integer);
var
  db : Integer;
begin
  if not Ready then Exit;
  if Value > 100 then Value := 100;
  if Value < 0   then Value := 0;
  if Value = 0 then
    db := DSBVOLUME_MIN
  else
    db := Trunc(33.22 * 100 * ln(Value/100)/ln(10));
  IDirectSoundBuffer(Buffer).SetVolume(db);
end;

procedure TOGG.Update;
var
  Status    : LongWord;
  Pos, Size : LongWord;
  p   : array [0..1] of Pointer;
  s   : array [0..1] of LongWord;
  Ret : LongWord;
  i   : Integer;
begin
  IDirectSoundBuffer(Buffer).GetStatus(Status);
  if Status and DSBSTATUS_BUFFERLOST <> 0 then
    if IDirectSoundBuffer(Buffer).Restore = 0 then
      Exit;

  Ret := 0;    
  IDirectSoundBuffer(Buffer).GetCurrentPosition(Pos, Ret);
  if Pos > LastPos then
    Size := Pos - LastPos
  else
    if Pos < LastPos then
      Size := Integer(OGG_BUFFER_SIZE - LastPos + Pos)
    else
      Exit;

  IDirectSoundBuffer(Buffer).Lock(LastPos, Size, p[0], s[0], p[1], s[1], 0);
  LastPos := Pos;
  for i := 0 to 1 do
    if s[i] <> 0 then
    begin
      Pos := 0;
      repeat
        Ret := ov_read(oggFile, Pointer(LongWord(p[i]) + Pos)^, s[i] - Pos, 0, 2, 1, nil);
        Pos := Pos + Ret;
      until (Ret = 0) or (Pos = s[i]);

      if Ret = 0 then
        if not FLoop then
        begin
          FillChar(Pointer(LongWord(p[i]) + Pos)^, s[i] - Pos, 0);
          FAllDone := True;
        end else
          ov_pcm_seek(oggFile, 0);
    end;
  IDirectSoundBuffer(Buffer).Unlock(p[0], s[0], p[1], s[1]);
end;

procedure TOGG.Play(Loop: Boolean);
var
  ThreadID : LongWord;
begin
  if not Ready then Exit;
  if Playing then Stop;
  if FDone then
  begin
    FDone    := False;
    FAllDone := False;
    ov_pcm_seek(oggFile, 0);
  end;
  FLoop   := Loop;

// First fill
  IDirectSoundBuffer(Buffer).SetCurrentPosition(OGG_BUFFER_SIZE);
  LastPos := 0;
  Update;
  IDirectSoundBuffer(Buffer).SetCurrentPosition(0);

  Thread  := CreateThread(nil, 0, @OggThreadProc, Self, 0, ThreadID);
end;

procedure TOGG.Pause;
begin
  if not Ready then Exit;
  IDirectSoundBuffer(Buffer).Stop;
end;

procedure TOGG.Stop;
begin
  if not Ready then Exit;
  TerminateThread(Thread, 0);
  CloseHandle(Thread);
  IDirectSoundBuffer(Buffer).Stop;
  IDirectSoundBuffer(Buffer).SetCurrentPosition(0);
  FDone    := True;
  FAllDone := True;
end;

function TOGG.Playing: Boolean;
var
  Status : LongWord;
begin
  Result := False;
  if not Ready then Exit;
  if Buffer <> nil then
    IDirectSoundBuffer(Buffer).GetStatus(Status);
  Result := Status and DSBSTATUS_PLAYING <> 0;
end;

initialization
  DSoundDLL := LoadLibrary('DSound.dll');
  DirectSoundCreate := GetProcAddress(DSoundDLL, 'DirectSoundCreate');
  ops_callbacks.read_func  := ops_read_func;
  ops_callbacks.seek_func  := ops_seek_func;
  ops_callbacks.close_func := ops_close_func;
  ops_callbacks.tell_func  := ops_tell_func;
finalization
  FreeLibrary(DSoundDLL);
end.
