unit raPlayer;

interface
  uses OMEGA, Helper;

type
  TPlayer = record
  private
    X, Y: single;
    VelX, VelY: single;
    Size: single;
    MaxVel: single;
    Health: integer;
    procedure Default;
  public
    procedure Create(PosX, PosY: single; NewSize: single = 4; HP: integer = 100);
    procedure SetPos(PosX, PosY: single);
    procedure SetSize(NewSize: single);
    procedure SetMaxVel(Value: single);
    procedure Draw;
    procedure Update;
    function GetVel: oePoint;
    function GetPos: oePoint;
    function GetHealth: integer;
    function GetSize: single;
    procedure Damage(Dam: integer);
  end;

var
  player: TPlayer;

implementation

{ TPlayer }

procedure TPlayer.Create(PosX, PosY, NewSize: single; HP: integer);
begin
  X := PosX;
  Y := PosY;
  Size := NewSize;
  Health := HP;
  MaxVel := 10;
end;

procedure TPlayer.Damage(Dam: integer);
begin
  Health := Health - Dam;
end;

procedure TPlayer.Default;
begin
end;

procedure TPlayer.Draw;
begin
  Default;

  Pr2D_Circle(X, Y, Size*2, OmegaColor(255, 255, 255), 255, 32, PR_FILL);
  Pr2D_Circle(X, Y, Size*2, OmegaColor(0, 0, 0), 255, 32, PR_SMOOTH);
  Pr2D_Circle(X, Y, Size, OmegaColor(128, 255, 128), 255, 32, PR_FILL);
end;

function TPlayer.GetHealth: integer;
begin
  Result := Health;
end;

function TPlayer.GetPos: oePoint;
begin
  Result.X := X;
  Result.Y := Y;
end;

function TPlayer.GetSize: single;
begin
  Result := Size;
end;

function TPlayer.GetVel: oePoint;
begin
  Result.X := VelX;
  Result.Y := VelY;
end;

procedure TPlayer.SetMaxVel(Value: single);
begin
  MaxVel := Value;
end;

procedure TPlayer.SetPos(PosX, PosY: single);
begin
  X := PosX;
  Y := PosY;
end;

procedure TPlayer.SetSize(NewSize: single);
begin
  Size := NewSize;
end;

procedure TPlayer.Update;
begin
  if Key_Down(K_W) then
      VelY := VelY - 1.0;
  if Key_Down(K_S) then
      VelY := VelY + 1.0;

  if Key_Down(K_A) then
      VelX := VelX - 1.0;
  if Key_Down(K_D) then
      VelX := VelX + 1.0;

  if VelX > MaxVel then
    VelX := MaxVel;
  if VelX < -MaxVel then
    VelX := -MaxVel;

  if VelY > MaxVel then
    VelY := MaxVel;
  if VelY < -MaxVel then
    VelY := -MaxVel;

  if VelX > 0 then
    VelX := VelX - 0.1;
  if VelX < 0 then
    VelX := VelX + 0.1;

  if VelY > 0 then
    VelY := VelY - 0.1;
  if VelY < 0 then
    VelY := VelY + 0.1;

  X := X + VelX;
  Y := Y + VelY;

  if X > OE_GetOptions.wnd_Width-10 then
  begin
    X := OE_GetOptions.wnd_Width-10;
    VelX := 0;
  end;
  if X < 10 then
  begin
    X := 10;
    VelX := 0;
  end;
  if Y > OE_GetOptions.wnd_Height-10 then
  begin
    Y := OE_GetOptions.wnd_Height-10;
    VelY := 0;
  end;
  if Y < 10 then
  begin
    Y := 10;
    VelY := 0;
  end;
end;

end.
